@@ -1,25 +1,8 @@
 # -*- coding: iso-8859-1 -*-
 # -----------------------------------------------------------------------
-# config.py - Handle the configuration file init. Also start logging.
+# Handle the configuration file init. Also start logging.
 # -----------------------------------------------------------------------
-# $Id: config.py 10547 2008-03-19 07:06:04Z duncan $
-#
-# Notes:
-#
-#   Try to find the freevo_config.py config file in the following places:
-#   1) ~/.freevo/freevo_config.py       The user's private config
-#   2) /etc/freevo/freevo_config.py     Systemwide config
-#   3) ./freevo_config.py               Defaults from the freevo dist
-#
-#   Customize freevo_config.py from the freevo dist and copy it to one
-#   of the other places where it will not get overwritten by new
-#   checkouts/installs of freevo.
-#
-#   The format of freevo_config.py might change, in that case you'll
-#   have to update your customized version.
-#
-# Todo:
-#
+# $Id: config.py 10983 2008-09-03 19:22:38Z duncan $
 # -----------------------------------------------------------------------
 # Freevo - A Home Theater PC framework
 # Copyright (C) 2002 Krister Lagerstrom, et al.
@@ -41,8 +24,18 @@
 #
 # -----------------------------------------------------------------------
 
+"""
+Handle the configuration file initialization
+and start logging.
 
-import sys, os, time, re, string, pwd, thread
+Try to find the freevo_config.py config file in the following places:
+    1. ~/.freevo/freevo_config.py       The user's private config
+    2. /etc/freevo/freevo_config.py     Systemwide config
+    3. ./freevo_config.py               Defaults from the freevo dist
+"""
+
+import sys, os, time, re, string, pwd
+from threading import RLock
 import setup_freevo
 import traceback
 import __builtin__
@@ -149,45 +142,34 @@ class Logger:
 
 class VideoGroup:
     """
-    @ivar vdev:
-        The video recording device, such as /dev/video0.
-    @ivar vvbi:
-        The video vbi device, such as /dev/vbi0.
-    @ivar adev:
-        The audio device, such as: None, /dev/dsp.
-    @ivar input_type:
-        tuner, webcam
-    @ivar input_num:
-        The number of this input according to V4L
-    @ivar tuner_type:
-        internal (on a v4l device), or external (cable or sat box)
-    @ivar tuner_norm:
-        NTSC, PAL, SECAM
-    @ivar tuner_chanlist:
-        us-cable,
-    @ivar tuner_chan:
-        If using input_type=tuner and tuner_type=external set this to
-        what channel it needs to be to get the signal, usually 3 or 4.
-    @ivar irsend_trans:
-        IR transmitter to use for multiple external tuners.
-    @ivar record_group:
-        VideoGroup that records for this tuner, default is to use the
-        same device for record and play
-    @ivar desc:
-        A nice description for this VideoGroup.
-    @ivar group_type:
-        Special variable to identify devices like dvb or ivtv.  This
-        can be one of: 'normal', 'ivtv', 'dvb', 'tvalsa' or 'webcam'.
-    @ivar cmd:
-        Command for execute external prog after the channel switched,
-        such as 'sudo /usr/local/bin/setuptuner'
     """
-
     def __init__(self, vdev=None, vvbi='/dev/vbi', adev=None, input_type=None,
                  input_num=0, tuner_norm=None, tuner_chanlist=None,
                  tuner_type='internal', tuner_chan=None, irsend_trans=None,
                  record_group=None, desc='Freevo Default Video Group',
                  group_type='normal', cmd=None):
+        """
+        Initialise an instance of a VideoGroup
+
+        @ivar vdev: The video recording device, such as /dev/video0.
+        @ivar vvbi: The video vbi device, such as /dev/vbi0.
+        @ivar adev: The audio device, such as: None, /dev/dsp.
+        @ivar input_type: tuner, webcam
+        @ivar input_num: The number of this input according to V4L
+        @ivar tuner_type: internal (on a v4l device), or external (cable or sat box)
+        @ivar tuner_norm: NTSC, PAL, SECAM
+        @ivar tuner_chanlist: us-cable,
+        @ivar tuner_chan: If using input_type=tuner and tuner_type=external set this to
+            what channel it needs to be to get the signal, usually 3 or 4.
+        @ivar irsend_trans: IR transmitter to use for multiple external tuners.
+        @ivar record_group: VideoGroup that records for this tuner, default is to use the
+            same device for record and play
+        @ivar desc: A nice description for this VideoGroup.
+        @ivar group_type: Special variable to identify devices like dvb or ivtv.  This
+            can be one of: 'normal', 'ivtv', 'dvb', 'tvalsa' or 'webcam'.
+        @ivar cmd: Command for execute external prog after the channel switched,
+            such as 'sudo /usr/local/bin/setuptuner'
+        """
 
         (v_norm, v_input, v_clist, v_dev) = TV_SETTINGS.split()
         if vdev is None:
@@ -219,8 +201,7 @@ class VideoGroup:
             self.cmd = cmd.strip()
 
     def __str__(self):
-        s = '<%s: %s %s %s %s %s %s>' % (self.group_type, self.vdev, self.vvbi, self.adev,
-            self.input_type, self.tuner_chanlist, self.tuner_chan)
+        s = '<%s: %s %s:%r %r>' % (self.group_type, self.vdev, self.input_num, self.input_type, self.tuner_norm)
         return s
 
 
@@ -338,7 +319,7 @@ if sys.argv[0].find('main.py') == -1:
 #
 # Send debug to stdout as well as to the logfile?
 #
-DEBUG_STDOUT = 1
+DEBUG_STDOUT = 0
 
 #
 # debugging messages are set by the logging level
@@ -419,8 +400,8 @@ OS_CACHEDIR, FREEVO_CACHEDIR = make_freevodir('CACHEDIR', '/var/cache', '/var/db
 #
 # Redirect stdout and stderr to stdout and /tmp/freevo.log
 #
+lock = RLock()
 #if not HELPER:
-lock = thread.allocate_lock()
 sys.stdout = Logger(sys.argv[0] + ':stdout')
 sys.stderr = Logger(sys.argv[0] + ':stderr')
 ts = time.asctime(time.localtime(time.time()))
@@ -467,25 +448,31 @@ def _debug_function_(s, level=1):
                 s = s.encode(encoding, 'replace')
             where =  traceback.extract_stack(limit = 2)[0]
             msg = '%s (%s): %s' % (where[0][where[0].rfind('/')+1:], where[1], s)
+            prefix = ''
             # log all the messages
             if level <= DCRITICAL:
                 logging.critical(msg)
+                prefix = _('CRITICAL')
             elif level == DERROR:
                 logging.error(msg)
+                prefix = _('ERROR')
             elif level == DWARNING:
                 logging.warning(msg)
+                prefix = _('WARNING')
             elif level == DINFO:
                 logging.info(msg)
+                prefix = _('INFO')
             else:
                 logging.debug(msg)
+                prefix = _('DEBUG')
             # print the message for info, warning, error and critical
-            if level <= DWARNING and DEBUG_STDOUT:
-                sys.__stdout__.write('%s\n' % (s))
+            if level <= DWARNING or DEBUG_STDOUT:
+                sys.__stdout__.write('%s: %s\n' % (prefix, s))
                 sys.__stdout__.flush()
         except UnicodeEncodeError:
-            print "_debug_ failed. %r" % msg
-        except Exception, e:
-            print "_debug_ failed: %r" % e
+            print "_debug_ failed: %r" % msg
+        except Exception, why:
+            print "_debug_ failed: %r" % why
     finally:
         if lock:
             lock.release()
@@ -577,7 +564,7 @@ for dirname in cfgfilepath:
         w, h = CONF.geometry.split('x')
         x, y = CONF.position.split(',')
         CONF.width, CONF.height = int(w), int(h)
-        CONF.x, CONF.w = int(x), int(y)
+        CONF.x, CONF.y = int(x), int(y)
         break
 else:
     print
@@ -644,7 +631,11 @@ for dirname in cfgfilepath:
     _debug_('Trying local configuration file "%s"...' % overridefile)
     if os.path.isfile(overridefile):
         _debug_('Loading local configuration file "%s"' % overridefile, DINFO)
-        execfile(overridefile, globals(), locals())
+        #new_globals = {}
+        new_locals = {}
+        execfile(overridefile, globals(), new_locals)
+        #globals().update(new_globals)
+        locals().update(new_locals)
 
         try:
             CONFIG_VERSION
@@ -656,23 +647,19 @@ for dirname in cfgfilepath:
             print
             sys.exit(1)
 
-        if int(str(CONFIG_VERSION).split('.')[0]) != \
-           int(str(LOCAL_CONF_VERSION).split('.')[0]):
+        if int(str(CONFIG_VERSION).split('.')[0]) != int(str(LOCAL_CONF_VERSION).split('.')[0]):
             print
             print 'Error: The version information in freevo_config.py doesn\'t'
             print 'match the version in your local_conf.py.'
             print 'Please check freevo_config.py for changes and set CONFIG_VERSION'
             print 'in %s to %s' % (overridefile, LOCAL_CONF_VERSION)
-            print_config_changes(LOCAL_CONF_VERSION, CONFIG_VERSION,
-                                 LOCAL_CONF_CHANGES)
+            print_config_changes(LOCAL_CONF_VERSION, CONFIG_VERSION, LOCAL_CONF_CHANGES)
             sys.exit(1)
 
-        if int(str(CONFIG_VERSION).split('.')[1]) != \
-           int(str(LOCAL_CONF_VERSION).split('.')[1]):
+        if int(str(CONFIG_VERSION).split('.')[1]) != int(str(LOCAL_CONF_VERSION).split('.')[1]):
             print
             print 'Warning: freevo_config.py was changed, please check local_conf.py'
-            print_config_changes(LOCAL_CONF_VERSION, CONFIG_VERSION,
-                                 LOCAL_CONF_CHANGES)
+            print_config_changes(LOCAL_CONF_VERSION, CONFIG_VERSION, LOCAL_CONF_CHANGES)
         break
 
 else:
@@ -804,7 +791,7 @@ if ROM_DRIVES == None:
         re_cd        = re.compile('^(/dev/cdrom[0-9]*|/dev/[am]?cd[0-9]+[a-z]?)[ \t]+([^ \t]+)[ \t]+', re.I)
         re_cdrec     = re.compile('^(/dev/cdrecorder[0-9]*)[ \t]+([^ \t]+)[ \t]+', re.I)
         re_dvd       = re.compile('^(/dev/dvd[0-9]*)[ \t]+([^ \t]+)[ \t]+', re.I)
-        re_iso       = re.compile('^([^ \t]+)[ \t]+([^ \t]+)[ \t]+(iso|cd)9660', re.I)
+        re_iso       = re.compile('^([^ \t#]+)[ \t]+([^ \t]+)[ \t]+(iso|cd)9660', re.I)
         re_automount = re.compile('^none[ \t]+([^ \t]+).*supermount.*dev=([^,]+).*', re.I)
         re_bymountcd = re.compile('^(/dev/[^ \t]+)[ \t]+([^ ]*cdrom[0-9]*)[ \t]+', re.I)
         re_bymountdvd= re.compile('^(/dev/[^ \t]+)[ \t]+([^ ]*dvd[0-9]*)[ \t]+', re.I)
