@@ -1,8 +1,8 @@
 # -*- coding: iso-8859-1 -*-
 # -----------------------------------------------------------------------
-# encodingcore.py, part of EncodingServer - for use with Freevo
+# Interface to the encoding server
 # -----------------------------------------------------------------------
-# $Id: encodingcore.py 10347 2008-02-06 19:16:57Z duncan $
+# $Id: encodingcore.py 11178 2008-11-17 21:42:33Z duncan $
 #
 # Author: den_RDC
 # some parts taken or inspired by Quickrip (by T. Chance, GPL,
@@ -28,6 +28,10 @@
 #
 # -----------------------------------------------------------------------
 
+"""
+Interface to the encoding server, to re-encode video to a different format.
+"""
+
 #Import statements
 import threading
 from time import sleep
@@ -37,6 +41,9 @@ import kaa.metadata
 from copy import copy
 from string import split, join
 
+#precompiled regular expression to obtain mencoder progress
+re_progress = re.compile('(\d+)\%\) .*Trem:\s*(\d+\w+)\s+')
+
 #some temporary hardcoded programs
 
 mencoder = config.CONF.mencoder
@@ -44,7 +51,7 @@ mplayer = config.CONF.mplayer
 
 #some data
 __author__ = 'den_RDC (rdc@kokosnoot.com)'
-__revision__ = '$Rev: 10347 $'
+__revision__ = '$Rev: 11178 $'
 __copyright__ = 'Copyright (C) 2004 den_RDC'
 __license__ = 'GPL'
 
@@ -53,13 +60,20 @@ mappings = {
     'lists' : {
         'containers'  : [ 'avi', 'mp4', 'mpeg' ], # add mkv back later
         'videocodecs' : [ 'MPEG 4 (lavc)','MPEG 2 (lavc)', 'XviD', 'H.264' ],
-        'audiocodecs' : [ 'MPEG 1 Layer 3 (mp3)', 'MPEG 1 Layer 2 (mp2)', 'AAC (iPod)', 'AC3', 'Vorbis', 'WMAv1',' WMAv2', 'copy' ]
+        'audiocodecs' : [ 'MPEG 1 Layer 3 (mp3)', 'MPEG 1 Layer 2 (mp2)', 'AAC (iPod)', 'AC3', 'Vorbis',
+            'WMAv1',' WMAv2', 'copy' ]
     },
     'vcodec' : {
-        'MPEG 4 (lavc)' : [ 'lavc', '-lavcopts', 'vcodec=mpeg4:mbd=2:trell:v4mv:last_pred=2:dia=-1:vmax_b_frames=2:vb_strategy=1:cmp=3:subcmp=3:precmp=0:vqcomp=0.6:vbitrate=%s:threads=%s%s%s'],
-        'MPEG 2 (lavc)' : [ 'lavc', '-lavcopts', 'vcodec=mpeg2video:vhq:vqmin=2:trell:vrc_buf_size=1835:vrc_maxrate=9800:keyint=18:vstrict=0:vbitrate=%s:threads=%s%s%s'],
+        'MPEG 4 (lavc)' : [ 'lavc', '-lavcopts', 'vcodec=mpeg4:mbd=2:trell:v4mv:last_pred=2:dia=-1:vmax_b_frames=2:'+
+            'vb_strategy=1:cmp=3:subcmp=3:precmp=0:vqcomp=0.6:vbitrate=%s:threads=%s%s%s'],
+        'MPEG 2 (lavc)' : [ 'lavc', '-lavcopts', 'vcodec=mpeg2video:vhq:vqmin=2:trell:vrc_buf_size=1835:'+
+            'vrc_maxrate=9800:keyint=18:vstrict=0:vbitrate=%s:threads=%s%s%s'],
         'XviD'          : [ 'xvid', '-xvidencopts', 'chroma_opt:vhq=4:bvhq=1:bitrate=%s:threads=%s%s%s'],
-        'H.264'         : [ 'x264', '-x264encopts', 'subq=5:8x8dct:frameref=2:bframes=3:b_pyramid:weight_b:bitrate=%s:threads=%s%s%s']
+        #'H.264'         : [ 'x264', '-x264encopts', 'subq=5:8x8dct:frameref=2:bframes=3:b_pyramid:weight_b:'+
+        #    'bitrate=%s:threads=%s%s%s']
+        'H.264'         : [ 'x264', '-x264encopts', 'subq=7:global_header:trellis=2:partitions=all:no-fast-pskip:'+
+            'me=umh:deblock:direct_pred=auto:level_idc=30:frameref=6:no8x8dct:me_range=32:bframes=0:nob_pyramid:'+
+            'nobrdo:cabac:bitrate=%s:threads=%s%s%s']
     },
     'container' : {
         'mpeg' : [ 'mpeg', '-mpegopts', 'format=dvd:tsaf'],
@@ -69,7 +83,7 @@ mappings = {
     },
     'acodec' : {
         'MPEG 1 Layer 3 (mp3)' : ['lavc', '-lavcopts', 'acodec=libmp3lame:abitrate=%s:aglobal=1'],
-        'AAC (iPod)'           : ['lavc', '-lavcopts', 'acodec=libfaac:abitrate=%s:aic=2:aglobal=1'],
+        'AAC (iPod)'           : ['lavc', '-lavcopts', 'acodec=libfaac:abitrate=%s:aglobal=1'],
         'AC3'                  : ['lavc', '-lavcopts', 'acodec=ac3:abitrate=%s:aglobal=1'],
         'MPEG 1 Layer 2 (mp2)' : ['lavc', '-lavcopts', 'acodec=mp2:abitrate=%s:aglobal=1'],
         'Vorbis'               : ['lavc', '-lavcopts', 'acodec=vorbis:abitrate=%s:aglobal=1'],
@@ -96,46 +110,101 @@ mappings = {
 
 
 
-#from pytvgrab enum.py, see http://pytvgrab.sourceforge.net
 class Enum(dict):
     """Enum
+    from pytvgrab enum.py, see http://pytvgrab.sourceforge.net
 
     Enum(names, x=0)"""
-
     def __init__(self, names, x=0):
+        """ Initialize an Enum """
         for i in range(x, x+len(names)):
             self.__dict__[names[i-x]]=i
             self[i]=names[i-x]
     # __init__()
 
-
 status = Enum(['notset', 'apass', 'vpass1', 'vpassfinal', 'postmerge'])
 
+
+
 class EncodingOptions:
+    """
+    Encoding options
+    """
     def getContainerList(self):
         """Return a list of possible containers"""
         return mappings['lists']['containers']
 
+
     def getVideoCodecList(self):
         """Return a list of possible video codecs"""
         return mappings['lists']['videocodecs']
 
+
     def getAudioCodecList(self):
         """Return a possible audio codec list"""
         return mappings['lists']['audiocodecs']
 
+
     def getVideoFiltersList(self):
         """Return a list of possible video filters"""
         return mappings['filtertype']
 
 
-class EncodingJob:
-    """Class for creation & configuration of EncodingJobs. This generates the mencoder commands"""
 
-    def __init__(self, source, output, friendlyname, idnr, chapter=None):
-        """Initialize class instance"""
-        _debug_('encodingcore.EncodingJob.__init__(%s, %s, %s, %s, %s)' % \
-            (source, output, friendlyname, idnr, chapter), 2)
+class EncodingJob:
+    """
+    Class for creation & configuration of EncodingJobs. This generates the mencoder commands
+
+    @ivar _generateCL: function to generate the command line
+    @ivar encodingopts: options for the different encoding steps
+    @ivar source: source file to encode
+    @ivar output: output file after encoding
+    @ivar name: friendly name of the job
+    @ivar idnr: job id number
+    @ivar chapter: chapter number to re-encode
+    @ivar rmsource: remove the source when done
+    @type rmsource: boolean
+    @ivar container: type of video container
+    @ivar tgtsize: target size for video
+    @ivar length: length of source video in seconds
+    @ivar vcodec: video codec to use
+    @ivar altprofile: user defined encoding profile
+    @ivar vbrate: video bit rate
+    @ivar multipass: number of encoding passes
+    @ivar vfilters: list of video filters to apply
+    @ivar crop: crop settings
+    @ivar cropres: crop settings
+    @ivar timeslice: start and end positions to encode
+    @ivar timeslice_mencoder: mencoder options for start and end
+    @ivar acodec: audio codec to use
+    @ivar abrate: audio bit rate
+    @ivar afilters: list of audio filters
+    @ivar cls: codec job list
+    @ivar percentage: mencoders current percentage
+    @ivar trem: mencoders current trem
+    @ivar status: current job status
+    @ivar pid: job process id
+    @ivar ana: is the source anamorphic?
+    @ivar fps: frames per second of the source
+    @ivar finishedanalyze: has the analysis finished?
+    @ivar info: Meta data of the source
+    @ivar resx: target video width
+    @ivar resy: target video height
+    @ivar threads: number of threads
+    @ivar failed: has the job failed?
+    """
+    def __init__(self, source, output, friendlyname, idnr, chapter=None, rmsource=False):
+        """
+        Initialize an instance of an EncodingJob
+        @param source: input source name
+        @param output: output file name
+        @param friendlyname: name of the job
+        @param idnr: job id number
+        @param chapter: chapter number to reencode
+        @param rmsource: remove the source
+        """
+        _debug_('encodingcore.EncodingJob.__init__(%s, %s, %s, %s, %s, %s)' % \
+            (source, output, friendlyname, idnr, chapter, rmsource), 2)
         #currently only MEncoder can be used, but who knows what will happen in the future :)
         self._generateCL = self._GenerateCLMencoder
         self.encodingopts = EncodingOptions()
@@ -144,6 +213,7 @@ class EncodingJob:
         self.name = friendlyname
         self.idnr = idnr
         self.chapter = chapter
+        self.rmsource = rmsource
 
         self.container = 'avi'
         self.tgtsize = None
@@ -157,6 +227,11 @@ class EncodingJob:
         self.crop = None
         self.cropres = None
 
+        # list of initial and end point of slice to encode
+        self.timeslice = [ None , None ]
+        # corresponding arguments for mencoder
+        self.timeslice_mencoder = []
+
         self.acodec = mappings['lists']['audiocodecs'][0]
         self.abrate = 128
         self.afilters = {} # Not used atm, might be used in the future
@@ -179,7 +254,7 @@ class EncodingJob:
         self.resy = None
 
         self.threads = 1 # How many threads to use during encoding (multi core systems)
-        self.failed=False
+        self.failed = False
         if self.source:
             try:
                 self.info = kaa.metadata.parse(self.source)
@@ -195,6 +270,24 @@ class EncodingJob:
                 self.finishedanalyze = True
 
 
+    def setTimeslice(self, timeslice):
+        "Set the encoding timeslice"
+        self.timeslice = timeslice
+        assert(type(timeslice) == type([]))
+        assert(len(timeslice) == 2)
+        self.timeslice_mencoder = []
+        start=0
+        if timeslice[0]:
+            self.timeslice_mencoder += ['-ss', str(timeslice[0])]
+            start = timeslice[0]
+        if timeslice[1]:
+            self.timeslice_mencoder += ['-endpos', str(timeslice[1]-start)]
+            if timeslice[1] < start:
+                self.timeslice_mencoder = []
+                self.timeslice = [ None , None ]
+                return 'Invalid slice of times: end is before start ??'
+
+
     def setContainer(self, container):
         """Set a container to hold the audio & video streams"""
         #safety checks
@@ -210,13 +303,13 @@ class EncodingJob:
         else:
             self.output = ('%s.%s' % (self.output, self.container))
 
+
     def setVideoCodec(self, vcodec, tgtsize, multipass=False, vbitrate=0, altprofile=None):
         """Set video codec and target filesize (in MB) or bit rate (in kbits/sec)"""
         _debug_('setVideoCodec(self, vcodec=%s, tgtsize=%s, multipass=%s, vbitrate=%s)' % \
             (vcodec, tgtsize, multipass, vbitrate))
-        #safety checks first
         if vcodec not in self.encodingopts.getVideoCodecList():
-            return 'Unknown video codec'
+            return 'Unknown video codec %r' % vcodec
 
         self.vcodec = vcodec
         if vbitrate:
@@ -230,28 +323,33 @@ class EncodingJob:
 
     def setAudioCodec(self, acodec, abrate=128):
         """Set audio codec & bitrate"""
-        #safety first
         if acodec not in self.encodingopts.getAudioCodecList():
-            return 'Unknown audio codec'
+            return 'Unknown audio codec %r' % acodec
 
         self.acodec = acodec
         self.abrate = abrate
 
+
     def setVideoFilters(self, videofilters):
         """Set video filters"""
         for vfilter, option in videofilters:
             if mappings['filter'].has_key(option):
                 self.vfilters += [ mappings['filter'][option]]
 
+
     def setVideoRes(self, videores):
+        """Set video resolution"""
         if videores == 'Optimal':
             (self.resx, self.resy) = (0, 0)
         else:
             (self.resx, self.resy) = videores.split(':')
 
+
     def setNumThreads(self, numthreads):
+        """Set the number of threads to use"""
         self.threads = numthreads
 
+
     def _CalcVideoBR(self):
         """Calculates the video bitrate"""
 
@@ -276,7 +374,7 @@ class EncodingJob:
 
         if self.chapter:
             #check some things, like length
-            _debug_('source=\"%s\" chapter=%s' % (self.source, self.chapter))
+            _debug_('source=%r" chapter=%s' % (self.source, self.chapter))
             dvddata = self.info
             dvdtitle = dvddata.tracks[self.chapter - 1]
             self.length = dvdtitle['length']
@@ -287,7 +385,7 @@ class EncodingJob:
             if config.DEBUG >= 2:
                 for f in dir(data):
                     _debug_('%s: %s' % (f, eval('data["%s"]' % f)), 2)
-            if data.has_key('length'):
+            if data.has_key('length') and data['length'] is not None:
                 self.length = data['length']
                 _debug_('Video length is %s' % self.length)
             else:
@@ -296,18 +394,25 @@ class EncodingJob:
         self._CropDetect()
 
 
-    def _CropDetect(self): #contains pieces of QuickRip
-        """Detect cropping
+    def _CropDetect(self):
+        """Detect cropping, contains pieces of QuickRip
 
         Function is always called because cropping is a good thing, and we can pass our ideal values
         back to the client wich can verify them visually if needed.""" #not true atm
         #build mplayer parameters
-        if hasattr(self, 'length'):
-            sstep = int(self.length / 27)
+        start = 0
+        if self.timeslice[0]:
+            start = self.timeslice[0]
+        if self.timeslice[1]:
+            sstep = int( (self.timeslice[1] - start) / 27)
+        elif hasattr(self, "length"):
+            sstep = int( (self.length - start) / 27)
         else:
             sstep = 60
 
-        arguments = [ '-vf', 'cropdetect=30', '-nosound', '-vo', 'null', '-frames', '10','-fps=540', '-sstep', str(sstep)]
+        arguments = self.timeslice_mencoder + [ "-vf", "cropdetect=30", "-nosound", "-vo", "null", "-fps", "540"]
+        if sstep > 0:
+            arguments +=  [ "-sstep", str(sstep)]
 
         if self.info.mime == 'video/dvd':
             arguments += [ '-dvd-device', self.source, 'dvd://%s' % self.chapter ]
@@ -319,6 +424,7 @@ class EncodingJob:
         #print (' '.join([mplayer]+arguments))
         self._run(mplayer, arguments, self._CropDetectParse, None, 0, None)
 
+
     def _GenerateCLMencoder(self):
         """Generate the command line(s) to be executed, using MEncoder for encoding"""
         #calculate the videobitrate
@@ -344,6 +450,7 @@ class EncodingJob:
 
             self.cls = [ videopass1, videopass2 ]
 
+
     def _GCLMSource(self):
         """Returns source part of mencoder"""
         if self.info.mime == 'video/dvd':
@@ -355,6 +462,7 @@ class EncodingJob:
         else:
             return [ self.source ]
 
+
     def _GCLMVideopass(self, passnr):
         """Returns video pass specefic part of mencoder cl"""
         vf = copy(self.vfilters)
@@ -364,7 +472,7 @@ class EncodingJob:
         #deinterlacer test vf += ['pp=lb']
         #set appropriate videopass, codec independant (lavc is vpass, xvid is pass)
         if passnr > 0:
-            if self.vcodec == 'XviD':
+            if self.vcodec == 'XviD' or self.vcodec == 'H.264':
                 passname = ':pass=%s'
             else:
                 passname = ':vpass=%s'
@@ -427,7 +535,7 @@ class EncodingJob:
 
         aspect= ''
         if self.vcodec=='MPEG 2 (lavc)' and self.cropres[0] != '720':
-            if self.ana == True:
+            if self.ana:
                 aspect=':aspect=16/9'
             else:
                 aspect=':aspect=4/3'
@@ -435,7 +543,7 @@ class EncodingJob:
             aspect =  ':autoaspect'
 
         # set video encoder options
-        if self.altprofile == None:
+        if self.altprofile is None:
             args = [
                 '-ovc', mappings['vcodec'][self.vcodec][0], mappings['vcodec'][self.vcodec][1],
                         mappings['vcodec'][self.vcodec][2] % (self.vbrate, self.threads, vpass, aspect ) ]
@@ -449,7 +557,7 @@ class EncodingJob:
         if self.acodec != 'copy':
             args += [ mappings['acodec'][self.acodec][1],
                       mappings['acodec'][self.acodec][2] % self.abrate ]
-        args += [ '-o', output]
+        args += [ '-o', output + '~incomplete~' ]
 
         # don't pass video filter in we have none
         if len(vfilter) != 0:
@@ -478,13 +586,16 @@ class EncodingJob:
         if yscaled:
             args += ['-sws', '1']
 
+        args = self.timeslice_mencoder + args
+
         return args
 
 
-    #from QuickRip, heavily adapted, new algo
-    #TODO give this another name, it does more then crop detection only
     def _CropDetectParse(self, lines, data): #seek to remove data
-        """Parses Mplayer output to obtain ideal cropping parameters, and do PAL/NTSC detection"""
+        """Parses Mplayer output to obtain ideal cropping parameters, and do PAL/NTSC detection
+        from QuickRip, heavily adapted, new algo
+        TODO give this another name, it does more then crop detection only
+        """
         #print '_CropDetectParse(self, lines=%r, data=%r)' % (lines, data)
 
         re_crop = re.compile('.*-vf crop=(\d*:\d*:\d*:\d*).*')
@@ -613,12 +724,14 @@ class EncodingJob:
         #end analyzing
         self.finishedanalyze = True
 
+
     def _CalcBPP(self, x, y):
         """Perform a BPP (Bits per Pixel calculation)"""
         bpp = (self.vbrate * 1000) / (x * y * self.fps)
         _debug_('_CalcBPP() = %s, fps=%s' % (bpp, self.fps))
         return bpp
 
+
     def _OptimalRes(self, x, y):
         """Using BPP calculations, try to find out the ideal resolution for this movie"""
         nonoptimal = True
@@ -639,33 +752,37 @@ class EncodingJob:
 
         return ( int(optx), int (opty) )
 
-    #from Quickrip, adapted
-    def _MencoderParse(self, line, data): #seek to remove data
-        """Parses mencoder stdout to get progress and trem"""
-        #(passtype, title) = data
 
-        re_progress = re.compile('(\d+)\%\) .*Trem:\s*(\d+\w+)\s+')
-        if re_progress.search(line):
-            self.percentage = re_progress.search(line).group(1)
-            self.trem = re_progress.search(line).group(2)
+    def _MencoderParse(self, line, data):
+        """Parses mencoder stdout to get progress and them
+        from Quickrip, adapted
+        seek to remove data
+        """
+        #(passtype, title) = data
+        # re_progress is precompiled at the beginning of this file, to speed up
+        s=re_progress.search(line)
+        if s:
+            self.percentage = s.group(1)
+            self.trem = s.group(2)
             #self.ui_updateProgress(perc, trem, passtype)
 
-    #from QuickRip, adapted
-    def _run(self, program, arguments, finalfunc, updatefunc=None,
-                flushbuffer=0, data=None, lock=None): # seek to remove data and/or crop (not really used)
-        """Runs a program; supply program name (string) and arguments (list)"""
+
+    def _run(self, program, arguments, finalfunc, updatefunc=None, flushbuffer=0, data=None, lock=None):
+        """Runs a program; supply program name (string) and arguments (list)
+        seek to remove data and/or crop (not really used)
+        """
         command = [program]
         command += arguments
 
-        self.thread = CommandThread(self, command, updatefunc, finalfunc,
-                                    flushbuffer, data, None) #self.lock)
+        self.thread = CommandThread(self, command, updatefunc, finalfunc, flushbuffer, data, None)
         self.thread.start()
 
 
 
-#command executing class - Taken from Quickrip & adapted.
 class CommandThread(threading.Thread): # seek to remove data andor crop (not really used)
-    """Handle threading of external commands"""
+    """Handle threading of external commands
+    command executing class - Taken from Quickrip & adapted.
+    """
     def __init__(self, parent, command, updatefunc, finalfunc, flushbuffer, data, lock):
         threading.Thread.__init__(self)
         self.parent = parent
@@ -677,6 +794,7 @@ class CommandThread(threading.Thread): # seek to remove data andor crop (not rea
         self.lock = lock
         _debug_('command=\"%s\"' % ' '.join(command))
 
+
     def run(self):
         #self.lock.acquire()
         self.pipe = popen2.Popen4(self.command)
@@ -736,7 +854,6 @@ class CommandThread(threading.Thread): # seek to remove data andor crop (not rea
 
 class EncodingQueue:
     """Class for generating an encoding queue"""
-
     def __init__(self):
         #we keep a list and a dict because a dict doesn't store an order
         self.qlist = []
@@ -746,12 +863,14 @@ class EncodingQueue:
         #remove old files
         self._removeTmp()
 
+
     def addEncodingJob(self, encjob):
         """Adds an encodingjob to the queue"""
 
         self.qlist += [encjob]
         self.qdict[encjob.idnr] = encjob
 
+
     def getProgress(self):
         """Gets progress on the current job"""
         if hasattr(self, 'currentjob'):
@@ -760,6 +879,7 @@ class EncodingQueue:
         else:
             return 'No job currently running'
 
+
     def startQueue(self):
         """Start the queue"""
         if not self.running:
@@ -767,6 +887,7 @@ class EncodingQueue:
             _debug_('queue started', DINFO)
             self._runQueue()
 
+
     def listJobs(self):
         """Returns a list of queue'ed jobs"""
         if self.qdict == {}:
@@ -777,6 +898,7 @@ class EncodingQueue:
                 jlist += [ (idnr, job.name, job.status) ]
             return jlist
 
+
     def _removeTmp(self):
         """Removes possible temporary files created during encoding"""
         tmpfiles = ['frameno.avi', 'divx2pass.log', 'xvid-twopass.stats', 'x264_2pass.log' ]
@@ -785,6 +907,7 @@ class EncodingQueue:
             if os.path.exists(tmpfile):
                 os.remove(tmpfile)
 
+
     def _runQueue(self, line='', data=''):
         """Executes the jobs in the queue, and gets called after every mencoder run is completed"""
         if self.qlist == []:
@@ -803,6 +926,17 @@ class EncodingQueue:
 
         if self.currentjob.status == status.vpassfinal:
             _debug_('Job %s finished' % self.currentjob.idnr, DINFO)
+            if self.currentjob.rmsource is True:
+                _debug_('Removing source: %s' % self.currentjob.source)
+                try:
+                    os.remove(self.currentjob.source)
+                except OSError :
+                    _debug_(' cannot remove file '+self.currentjob.source, DWARNING)
+            #
+            try:
+                os.rename(self.currentjob.output + '~incomplete~', self.currentjob.output)
+            except OSError :
+                _debug_(' cannot rename file to remove ~incomplete~ suffix '+self.currentjob.output, DWARNING)
             #we are done with this job, remove it
             del self.qlist[0]
             del self.qdict[self.currentjob.idnr]
