@@ -1,8 +1,8 @@
 # -*- coding: iso-8859-1 -*-
 # -----------------------------------------------------------------------
-# plugins.py - list all plugins and prints help about them
+# A helper to list all plugins and prints help about them
 # -----------------------------------------------------------------------
-# $Id: plugins.py 10519 2008-03-13 19:20:38Z duncan $
+# $Id: plugins.py 11016 2008-09-17 08:59:10Z duncan $
 #
 # Notes:       This script prints out information aboyt plugins in
 #              Freevo. As descriptions the Python class description
@@ -125,12 +125,12 @@ def parse_plugins2():
     return ''
 
 
-def parse_plugins(plugin_name=None):
+def parse_plugins(plugin_name=''):
     start = re.compile('^class *(.*)\((.*Plugin\s*).:')
     stop  = re.compile('^[\t ]*def.*:')
     comment = re.compile('^[\t ]*"""')
     config_start = re.compile('^[ \t]+def +config *\( *self *\) *:')
-    config_end   = re.compile(' *(class|def)')
+    config_end   = re.compile(' *(class|def|@)')
 
     print_line = 0
     ptypes = {}
@@ -141,12 +141,13 @@ def parse_plugins(plugin_name=None):
     for p in plugin.__all_plugins__:
         active.append(p[0])
 
+    plugin_shortname = plugin_name.split('.')[-1]+'.py'
+    plugin_init_name = os.path.join(plugin_name.split('.')[-1], '__init__.py')
     for file in util.recursefolders(os.environ['FREEVO_PYTHON'], 1, '*.py', 1):
         if file.find('plugin.py') > 0:
             continue
         if plugin_name:
-            plugin_shortname = plugin_name.split('.')[-1]+'.py'
-            if file.find(plugin_shortname) < 0:
+            if file.find(plugin_shortname) < 0 and file.find(plugin_init_name) < 0:
                 continue
         parse_status = 0
         whitespaces  = 0
@@ -163,8 +164,7 @@ def parse_plugins(plugin_name=None):
                 config      = 'def return_config():\n'
                 scan_config = line.find('def')
 
-            if (comment.match(line) and parse_status == 2) or \
-                   (stop.match(line) and parse_status == 1):
+            if (comment.match(line) and parse_status == 2) or (stop.match(line) and parse_status == 1):
                 parse_status = 0
                 all_plugins[-1][-2] = desc
 
@@ -254,9 +254,8 @@ def iscode(line):
     @note: this does not work too well in all cases
     """
     return line.strip().startswith('|')
-    return (len(line) > 2 and line[:2].upper() == line[:2] and \
-            line.find('=') > 0) or \
-            (line and line[0] in ('#', ' ', '[', '(')) or (line.find('plugin.') == 0)
+    return (len(line) > 2 and line[:2].upper() == line[:2] and line.find('=') > 0) or \
+        (line and line[0] in ('#', ' ', '[', '(')) or (line.find('plugin.') == 0)
 
 
 def html_info(plugin_name, all_plugins):
@@ -274,7 +273,7 @@ def html_info(plugin_name, all_plugins):
             else:
                 ret += '<br><b>Description</b>:'
                 ret += '<p>'
-                tmp  = desc
+                tmp  = Unicode(desc)
                 desc = []
                 for block in tmp.split('\n\n'):
                     for line in block.split('\n'):
@@ -291,8 +290,7 @@ def html_info(plugin_name, all_plugins):
                         code = 1
 
                         try:
-                            if (desc[i+1] and not iscode(desc[i+1])) or \
-                                   (desc[i+2] and not iscode(desc[i+2])):
+                            if (desc[i+1] and not iscode(desc[i+1])) or (desc[i+2] and not iscode(desc[i+2])):
                                 ret += '</pre>'
                                 code = 0
                         except IndexError:
@@ -327,9 +325,9 @@ def html_info(plugin_name, all_plugins):
 
 
 def wiki_word(s, mode):
-    '''
+    """
     Need to escape mixed case words for the wiki
-    '''
+    """
     if mode == 'moin':
         return s
     needs_pling=re.compile('^([A-Z]+[^.]*[A-Z]+[^.]*)')
@@ -477,7 +475,7 @@ if __name__ == '__main__':
     elif len(sys.argv)>1 and sys.argv[1] == '-html':
         all_plugins = parse_plugins()
 
-        print '''<html>
+        print """<html>
       <head>
         <meta HTTP-EQUIV="Content-Type" CONTENT="text/html;charset=iso-8859-1">
         <title>Freevo Plugins</title>
@@ -485,7 +483,7 @@ if __name__ == '__main__':
       </head>
       <body>
         <font class="headline" size="+3">Freevo Plugin List</font>
-        <p><b>Index</b><ol>'''
+        <p><b>Index</b><ol>"""
 
         for p in all_plugins:
             print '<li><a href="#%s">%s</a></li>' % (p[0], p[0])
