@@ -2,7 +2,7 @@
 # -----------------------------------------------------------------------
 # A network aware TV recording server.
 # -----------------------------------------------------------------------
-# $Id: recordserver.py 10559 2008-03-22 11:42:02Z duncan $
+# $Id: recordserver.py 11173 2008-11-17 21:34:29Z duncan $
 #
 # -----------------------------------------------------------------------
 # Freevo - A Home Theater PC framework
@@ -63,7 +63,12 @@ if __name__ == '__main__':
 # The twisted modules will go away when all the recordserver clients
 # have been updated to use kaa.rpc
 
-from video.commdetectclient import initCommDetectJob, queueIt, listJobs, connectionTest
+from video.commdetectclient import initCommDetectJob
+from video.commdetectclient import queueIt as queueCommdetectJob
+from video.commdetectclient import listJobs as listCommdetectJobs
+from video.commdetectclient import connectionTest as commdetectConnectionTest
+
+from video.encodingclient import EncodingClientActions
 
 import tv.record_types
 from tv.record_types import TYPES_VERSION
@@ -72,7 +77,7 @@ import tv.epg_xmltv
 import util.tv_util as tv_util
 import plugin
 import util.popen3
-from tv.channels import FreevoChannels
+from tv.channels import FreevoChannels, CHANNEL_ID
 from util.videothumb import snapshot
 from event import *
 
@@ -133,6 +138,7 @@ class RecordServer:
         self.delay_recording = None
         self.schedule = ScheduledRecordings()
         self.updateFavoritesSchedule()
+        self.es = EncodingClientActions()
 
 
     @kaa.rpc.expose('ping')
@@ -218,10 +224,10 @@ class RecordServer:
     @kaa.rpc.expose('isPlayerRunning')
     def isPlayerRunning(self):
         """
-        returns the state of a player, mplayer, xine, etc.
+        Test is the player is running
 
-        @todo: real player running test, check /dev/videoX.  This could go into the
-        upsoon client
+        TODO: real player running test, check /dev/videoX.  This could go into the upsoon client
+        @returns: the state of a player, mplayer, xine, etc.
         """
         _debug_('isPlayerRunning()', 2)
         res = (os.path.exists(config.FREEVO_CACHEDIR + '/playing'))
@@ -252,7 +258,7 @@ class RecordServer:
             try:
                 file_ver = schedule.TYPES_VERSION
             except AttributeError:
-                _debug_('The cache does not have a version and must be recreated', DWARNING)
+                _debug_('The cache does not have a version and must be recreated', DINFO)
 
             if file_ver != TYPES_VERSION:
                 _debug_(('ScheduledRecordings version number %s is stale (new is %s), must be reloaded') % \
@@ -808,7 +814,7 @@ class RecordServer:
         max_results = 500
 
         if find:
-            find = Unicode(find)
+            find = Unicode(find).replace('(','\(').replace(')','\)').replace('.','\.')
         elif not movies_only:
             _debug_('nothing to find', DINFO)
             return (False, _('nothing to find'))
@@ -933,7 +939,7 @@ class RecordServer:
                     self.delay_recording = None
 
                 if self.delay_recording:
-                    _debug_('delaying: %s' % prog, DWARNING)
+                    _debug_('delaying: %s' % prog, DINFO)
                 else:
                     _debug_('going to record: %s' % prog, DINFO)
                     prog.isRecording = True
@@ -962,7 +968,7 @@ class RecordServer:
                 self.removeScheduledRecording(rec_prog)
                 return
 
-            self.vg = self.fc.getVideoGroup(rec_prog.channel_id, False)
+            self.vg = self.fc.getVideoGroup(rec_prog.channel_id, False, CHANNEL_ID)
             suffix = self.vg.vdev.split('/')[-1]
             self.tv_lock_file = config.FREEVO_CACHEDIR + '/record.'+suffix
             self.record_app.Record(rec_prog)
@@ -1340,15 +1346,53 @@ class RecordServer:
                 if config.VCR_POST_REC:
                     util.popen3.Popen3(config.VCR_POST_REC)
                 if config.TV_RECORD_REMOVE_COMMERCIALS:
-                    (result, response) = connectionTest('connection test')
+                    (result, response) = commdetectConnectionTest('connection test')
                     if result:
                         (status, idnr) = initCommDetectJob(prog.filename)
-                        (status, output) = listJobs()
+                        (status, output) = listCommdetectJobs()
                         _debug_(output, DINFO)
-                        (status, output) = queueIt(idnr, True)
+                        (status, output) = queueCommdetectJob(idnr, True)
                         _debug_(output, DINFO)
                     else:
                         _debug_('commdetect server not running', DINFO)
+                if config.TV_REENCODE:
+                    (result, response) = self.es.ping('connection test')
+                    if result:
+                        source = prog.filename
+                        output = prog.filename
+                        multipass = config.REENCODE_NUMPASSES > 1
+
+                        (status, resp) = self.es.initEncodingJob(source, output, prog.title, None,
+                            config.TV_REENCODE_REMOVE_SOURCE)
+                        _debug_('initEncodingJob:status:%s resp:%s' % (status, resp))
+
+                        idnr = resp
+
+                        (status, resp) = self.es.setContainer(idnr, config.REENCODE_CONTAINER)
+                        _debug_('setContainer:status:%s resp:%s' % (status, resp))
+
+                        (status, resp) = self.es.setVideoCodec(idnr, config.REENCODE_VIDEOCODEC, 0, multipass,
+                            config.REENCODE_VIDEOBITRATE, config.REENCODE_ALTPROFILE)
+                        _debug_('setVideoCodec:status:%s resp:%s' % (status, resp))
+
+                        (status, resp) = self.es.setAudioCodec(idnr, config.REENCODE_AUDIOCODEC,
+                            config.REENCODE_AUDIOBITRATE)
+                        _debug_('setAudioCodec:status:%s resp:%s' % (status, resp))
+
+                        (status, resp) = self.es.setNumThreads(idnr, config.REENCODE_NUMTHREADS)
+                        _debug_('setNumThreads:status:%s resp:%s' % (status, resp))
+
+                        (status, resp) = self.es.setVideoRes(idnr, config.REENCODE_RESOLUTION)
+                        _debug_('setVideoRes:status:%s resp:%s' % (status, resp))
+
+                        (status, resp) = self.es.listJobs()
+                        _debug_('listJobs:status:%s resp:%s' % (status, resp))
+
+                        (status, resp) = self.es.queueIt(idnr, True)
+                        _debug_('queueIt:status:%s resp:%s' % (status, resp))
+                    else:
+                        _debug_('encoding server not running', DINFO)
+
                 # This is a really nasty hack but if it fixes the problem then great
                 if self.delay_recording:
                     prog = self.delay_recording
@@ -1361,7 +1405,7 @@ class RecordServer:
                     rec_prog = prog
                     _debug_('start delayed recording: %s' % rec_prog, DINFO)
                     self.record_app = plugin.getbyname('RECORD')
-                    self.vg = self.fc.getVideoGroup(rec_prog.channel_id, False)
+                    self.vg = self.fc.getVideoGroup(rec_prog.channel_id, False, CHANNEL_ID)
                     suffix = self.vg.vdev.split('/')[-1]
                     self.record_app.Record(rec_prog)
                     self.delay_recording = None
@@ -1416,6 +1460,14 @@ if __name__ == '__main__':
         _debug_('Removed old record lock \"%s\"' % f, DINFO)
         os.remove(f)
 
-    _debug_('main() starting')
-    main()
-    _debug_('main() finished')
+    try:
+        _debug_('main() starting')
+        main()
+        _debug_('main() finished')
+    except SystemExit:
+        _debug_('main() stopped')
+        pass
+    except Exception, why:
+        import traceback
+        traceback.print_exc()
+        _debug_(why, DWARNING)
