@@ -1,12 +1,8 @@
 # -*- coding: iso-8859-1 -*-
 # -----------------------------------------------------------------------
-# viewer.py - Freevo image viewer
+# Freevo image viewer
 # -----------------------------------------------------------------------
-# $Id: viewer.py 10204 2007-12-13 20:34:08Z duncan $
-#
-# Notes:
-# Todo:        Change the signal stuff for slideshows
-#
+# $Id: viewer.py 11167 2008-11-16 17:51:46Z duncan $
 # -----------------------------------------------------------------------
 # Freevo - A Home Theater PC framework
 # Copyright (C) 2002 Krister Lagerstrom, et al.
@@ -28,6 +24,9 @@
 #
 # -----------------------------------------------------------------------
 
+"""
+Freevo image viewer
+"""
 
 import os
 
@@ -41,6 +40,7 @@ from gui import GUIObject
 from event import *
 
 import time
+import datetime
 from animation import render, Transition
 import pygame
 
@@ -71,7 +71,8 @@ class ImageViewer(GUIObject):
                            str(IMAGE_ZOOM_GRID6):6, str(IMAGE_ZOOM_GRID7):7,
                            str(IMAGE_ZOOM_GRID8):8, str(IMAGE_ZOOM_GRID9):9 }
 
-        self.slideshow   = True
+        self.slideshow   = config.IMAGEVIEWER_AUTOPLAY
+        self.duration    = config.IMAGEVIEWER_DURATION
         self.app_mode    = 'image'
         self.last_image  = (None, None)
         self.render      = render.get_singleton()
@@ -95,9 +96,9 @@ class ImageViewer(GUIObject):
 
 
     def view(self, item, zoom=0, rotation=0):
-        '''
+        """
         view an image
-        '''
+        """
         _debug_('view(item, zoom=%s, rotation=%s)' % (zoom, rotation), 2)
         if zoom:
             self.app_mode    = 'image_zoom'
@@ -119,13 +120,13 @@ class ImageViewer(GUIObject):
             image = self.osd.loadbitmap(filename, cache=self.bitmapcache)
         else:
             # Using Container-Image
-            image = item.loadimage()
+            image, w, h = item.loadimage()
 
         rc.app(self)
 
         if not image:
             self.osd.clearscreen(color=self.osd.COL_BLACK)
-            self.osd.drawstringframed(_('Can\'t Open Image\n\'%s\'') % Unicode(filename),
+            self.osd.drawstringframed(_('Can\'t Open Image\n"%s"') % Unicode(filename),
                 config.OSD_OVERSCAN_LEFT + 20, config.OSD_OVERSCAN_TOP + 20,
                 self.osd.width - (config.OSD_OVERSCAN_LEFT+config.OSD_OVERSCAN_RIGHT) - 40,
                 self.osd.height - (config.OSD_OVERSCAN_TOP+config.OSD_OVERSCAN_BOTTOM) - 40,
@@ -296,13 +297,23 @@ class ImageViewer(GUIObject):
         self.osd.update()
 
         # start timer
-        if self.fileitem.duration and self.slideshow and not self.signal_registered:
-            rc.register(self.signalhandler, False, self.fileitem.duration*100)
+        if self.duration and self.slideshow and not self.signal_registered:
+            rc.register(self.signalhandler, False, self.duration*100)
             self.signal_registered = True
 
-        self.last_image  = (item, (image, x, y, scale, bbx, bby, bbw, bbh,
-                                   self.rotation))
+        self.last_image  = (item, (image, x, y, scale, bbx, bby, bbw, bbh, self.rotation))
 
+        # stop slideshow at the end if configured
+        try:
+            index = item.parent.play_items.index(item)+1
+            length = len(item.parent.play_items)
+            if index == length:
+                self.slideshow = config.IMAGEVIEWER_AUTOPLAY
+
+            # send information event to LCD2
+            rc.post_event(Event('IMAGE_VIEW_INFO', arg=(index, length, item.name)))
+        except Exception, why:
+            _debug_('Invalid parent item: %s' % (why,), DWARNING)
 
         # XXX Hack to move the selected item to the current showing image
         # XXX TODO: find a way to add it to directory.py or playlist.py
@@ -342,14 +353,17 @@ class ImageViewer(GUIObject):
 
     def eventhandler(self, event, menuw=None):
         _debug_('eventhandler(event=%s, menuw=%s)' % (event, menuw), 2)
-        if event == PAUSE or event == PLAY:
+        # SELECT also should act as PLAY/PAUSE (-> could be done with event rerouting!?)
+        if event == PAUSE or event == PLAY or (event == BUTTON and event.arg == 'SELECT'):
             if self.slideshow:
                 rc.post_event(Event(OSD_MESSAGE, arg=_('pause')))
+                rc.post_event(Event('IMAGE_PAUSE_INFO', arg=''))
                 self.slideshow = False
                 rc.unregister(self.signalhandler)
                 self.signal_registered = False
             else:
-                rc.post_event(Event(OSD_MESSAGE, arg=_('play')))
+                rc.post_event(Event(OSD_MESSAGE, arg=_('play')+(' %ss'%self.duration)))
+                rc.post_event(Event('IMAGE_PLAY_INFO', arg='%s' % self.duration))
                 self.slideshow = True
                 rc.register(self.signalhandler, False, 100)
                 self.signal_registered = True
@@ -358,6 +372,7 @@ class ImageViewer(GUIObject):
         elif event == STOP:
             self.last_image  = None, None
             self.signal_registered = False
+            self.slideshow = config.IMAGEVIEWER_AUTOPLAY
             rc.unregister(self.signalhandler)
             rc.app(None)
             self.fileitem.eventhandler(event)
@@ -432,6 +447,20 @@ class ImageViewer(GUIObject):
                 except Exception, e:
                     print 'getbyname(\'shoppingcart\')', e
 
+        # change slideshow duration and send event to OSD and LCD2
+        elif (event == BUTTON) and (event.arg == 'REW'):
+            if self.duration > 7: self.duration -= 2
+            elif self.duration > 1: self.duration -= 1
+            rc.post_event(Event(OSD_MESSAGE, arg="Timer %ss" % self.duration)) # not yet internationalised
+            rc.post_event(Event('IMAGE_PLAY_INFO', arg='%s' % self.duration))
+            return True
+        elif (event == BUTTON) and (event.arg == 'FFWD'):
+            if self.duration < 6: self.duration += 1
+            elif self.duration < 11: self.duration += 2
+            rc.post_event(Event(OSD_MESSAGE, arg="Timer %ss" % self.duration)) # not yet internationalised
+            rc.post_event(Event('IMAGE_PLAY_INFO', arg='%s' % self.duration))
+            return True
+
         else:
             return self.fileitem.eventhandler(event)
 
@@ -448,6 +477,9 @@ class ImageViewer(GUIObject):
             i = self.fileitem.getattr(strtag[1])
             if i:
                 osdstring.append('%s %s' % (strtag[0], i))
+            else:
+                if strtag[1] == 'date' and self.fileitem['timestamp']:
+                    osdstring.append('%s %s' % (strtag[0], datetime.datetime.fromtimestamp(self.fileitem['timestamp'])))
 
         # If after all that there is nothing then tell the users that
         if osdstring == []:
