@@ -1,12 +1,8 @@
 # -*- coding: iso-8859-1 -*-
 # -----------------------------------------------------------------------
-# menu.py - freevo menu handling system
+# Freevo menu handling system
 # -----------------------------------------------------------------------
-# $Id: menu.py 10562 2008-03-22 21:57:15Z duncan $
-#
-# Notes:
-# Todo:
-#
+# $Id: menu.py 11205 2008-12-01 02:42:54Z duncan $
 # -----------------------------------------------------------------------
 # Freevo - A Home Theater PC framework
 # Copyright (C) 2002 Krister Lagerstrom, et al.
@@ -28,6 +24,10 @@
 #
 # -----------------------------------------------------------------------
 
+"""
+Freevo menu handling system
+"""
+
 import string
 import copy
 
@@ -42,14 +42,18 @@ from event import *
 from item import Item
 from gui import GUIObject, AlertBox
 
+from util.benchmark import benchmark
+benchmarking = config.DEBUG_BENCHMARKING
+benchmarkcall = config.DEBUG_BENCHMARKCALL
+
 
 class MenuItem(Item):
     """
     Default item for the menu. It includes one action
     """
-    def __init__(self, name, action=None, arg=None, type=None, image=None,
-                  icon=None, parent=None, skin_type=None):
-        Item.__init__(self, parent, skin_type = skin_type)
+    @benchmark(benchmarking, benchmarkcall)
+    def __init__(self, name='', action=None, arg=None, type=None, image=None, icon=None, parent=None, skin_type=None):
+        Item.__init__(self, parent, skin_type=skin_type)
         if name:
             self.name  = Unicode(name)
         if icon:
@@ -62,6 +66,7 @@ class MenuItem(Item):
         self.type     = type
 
 
+    #@benchmark(benchmarking, benchmarkcall)
     def __str__(self):
         """
         return the event as string
@@ -70,13 +75,26 @@ class MenuItem(Item):
         if hasattr(self, 'action'):    s += ' action=%s' % self.action
         #if hasattr(self, 'arg') and self.arg: s += ' arg=%s' % self.arg[0]
         if hasattr(self, 'type'):      s += ' type=%s' % self.type
-        if hasattr(self, 'image'):     s += ' image=%s' % self.image
+        if hasattr(self, 'image'):     s += ' image=%s' % String(self.image)
         if hasattr(self, 'icon'):      s += ' icon=%s' % self.icon
         if hasattr(self, 'skin_type'): s += ' skin_type=%s' % self.skin_type
         #if hasattr(self, 'parent'):    s += ' parent=%s' % self.parent
         return s
 
 
+    #@benchmark(benchmarking, benchmarkcall)
+    def __repr__(self):
+        """
+        return the menu item as a raw string
+        """
+        if hasattr(self, 'name') and self.name:
+            return '<%-.16s: %r>' % (self.name, self.__class__)
+        if hasattr(self, 'type') and self.type:
+            return '<%r: %r>' % (self.type, self.__class__)
+        return '<%r: %r>' % (self.__class__,)
+
+
+    @benchmark(benchmarking, benchmarkcall)
     def actions(self):
         """
         return the default action
@@ -84,6 +102,7 @@ class MenuItem(Item):
         return [ (self.select, self.name, 'MENU_SUBMENU') ]
 
 
+    @benchmark(benchmarking, benchmarkcall)
     def select(self, arg=None, menuw=None):
         """
         call the default acion
@@ -97,8 +116,9 @@ class Menu:
     """
     a Menu with Items for the MenuWidget
     """
-    def __init__(self, heading, choices, fxd_file=None, umount_all=0, reload_func=None,
-            item_types=None, force_skin_layout=-1):
+    @benchmark(benchmarking, benchmarkcall)
+    def __init__(self, heading, choices, fxd_file=None, umount_all=0, reload_func=None, item_types=None,
+        force_skin_layout=-1):
 
         self.heading = heading
         self.choices = choices          # List of MenuItems
@@ -130,6 +150,7 @@ class Menu:
         self.back_one_menu = 1
 
 
+    #@benchmark(benchmarking, benchmarkcall)
     def __str__(self):
         """
         return the class as string
@@ -138,6 +159,15 @@ class Menu:
         return s
 
 
+    #@benchmark(benchmarking, benchmarkcall)
+    def __repr__(self):
+        """
+        return the class as string
+        """
+        return '<%-.16s: %r>' % (self.heading, self.__class__)
+
+
+    @benchmark(benchmarking, benchmarkcall)
     def items_per_page(self):
         """
         return the number of items per page for this skin
@@ -150,6 +180,7 @@ class MenuWidget(GUIObject):
     """
     The MenuWidget handles a stack of Menus
     """
+    @benchmark(benchmarking, benchmarkcall)
     def __init__(self):
         GUIObject.__init__(self)
         self.menustack = []
@@ -161,16 +192,27 @@ class MenuWidget(GUIObject):
         self.show_callbacks = []
         self.force_page_rebuild = False
 
+        self.first_drawing = False
 
+
+    #@benchmark(benchmarking, benchmarkcall)
     def __str__(self):
         """
         return the class as string
         """
-        s = '%s' % self.label
-        s += ', top=%s left=%s' % (self.top, self.left)
+        s = '%s' % (self.label,)
+        #s += ', rect=%s' % (self.rect,)
         return s
 
 
+    #@benchmark(benchmarking, benchmarkcall)
+    def __repr__(self):
+        if self.label:
+            return '<%-.16s: %r>' % (self.label, self.__class__)
+        return '%r' % (self.__class__)
+
+
+    @benchmark(benchmarking, benchmarkcall)
     def get_event_context(self):
         """
         return the event context
@@ -180,6 +222,7 @@ class MenuWidget(GUIObject):
         return self.event_context
 
 
+    @benchmark(benchmarking, benchmarkcall)
     def show(self):
         if not self.visible:
             self.visible = 1
@@ -189,6 +232,7 @@ class MenuWidget(GUIObject):
         rc.app(None)
 
 
+    @benchmark(benchmarking, benchmarkcall)
     def hide(self, clear=True):
         if self.visible:
             self.visible = 0
@@ -196,6 +240,7 @@ class MenuWidget(GUIObject):
                 skin.clear(osd_update=clear)
 
 
+    @benchmark(benchmarking, benchmarkcall)
     def delete_menu(self, arg=None, menuw=None, allow_reload=True):
         if len(self.menustack) > 1:
             self.menustack = self.menustack[:-1]
@@ -212,6 +257,7 @@ class MenuWidget(GUIObject):
             self.init_page()
 
 
+    @benchmark(benchmarking, benchmarkcall)
     def delete_submenu(self, refresh=True, reload=False, osd_message=''):
         """
         Delete the last menu if it is a submenu. Also refresh or reload the
@@ -230,6 +276,7 @@ class MenuWidget(GUIObject):
             rc.post_event(Event(OSD_MESSAGE, arg=osd_message))
 
 
+    @benchmark(benchmarking, benchmarkcall)
     def back_one_menu(self, arg=None, menuw=None):
         if len(self.menustack) > 1:
             try:
@@ -261,6 +308,7 @@ class MenuWidget(GUIObject):
                 self.refresh()
 
 
+    @benchmark(benchmarking, benchmarkcall)
     def goto_main_menu(self, arg=None, menuw=None):
         self.menustack = [self.menustack[0]]
         menu = self.menustack[0]
@@ -268,6 +316,7 @@ class MenuWidget(GUIObject):
         self.refresh()
 
 
+    @benchmark(benchmarking, benchmarkcall)
     def goto_media_menu(self, media='audio'):
         """
         Go to a main menu item media = 'tv' or 'audio' or 'video' or 'image' or 'games'
@@ -310,11 +359,6 @@ class MenuWidget(GUIObject):
         level = 0
         for mediaitem in media.split('.'):
             for menuitem in self.menustack[level].choices:
-                if config.DEBUG:
-                    try:
-                        print 'menuitem=%s' % menuitem
-                    except:
-                        pass
                 try:
                     if menuitem.arg[0] == mediaitem:
                         if config.OSD_SOUNDS:
@@ -333,6 +377,7 @@ class MenuWidget(GUIObject):
             level += 1
 
 
+    @benchmark(benchmarking, benchmarkcall)
     def goto_prev_page(self, arg=None, menuw=None):
         menu = self.menustack[-1]
 
@@ -350,6 +395,7 @@ class MenuWidget(GUIObject):
             self.refresh()
 
 
+    @benchmark(benchmarking, benchmarkcall)
     def goto_next_page(self, arg=None, menuw=None):
         menu = self.menustack[-1]
         self.rows, self.cols = menu.items_per_page()
@@ -375,6 +421,7 @@ class MenuWidget(GUIObject):
             self.refresh()
 
 
+    @benchmark(benchmarking, benchmarkcall)
     def pushmenu(self, menu):
         self.menustack.append(menu)
         if isinstance(menu, Menu):
@@ -386,6 +433,7 @@ class MenuWidget(GUIObject):
             menu.refresh()
 
 
+    @benchmark(benchmarking, benchmarkcall)
     def refresh(self, reload=0):
         menu = self.menustack[-1]
 
@@ -393,11 +441,11 @@ class MenuWidget(GUIObject):
             # Do not draw if there are any children
             if self.children:
                 return False
-
             return menu.refresh()
 
-        if self.menustack[-1].umount_all == 1:
-            util.umount_all()
+        #DJW: why do we do this?
+        #if self.menustack[-1].umount_all == 1:
+        #    util.umount_all()
 
         if reload:
             if menu.reload_func:
@@ -409,10 +457,15 @@ class MenuWidget(GUIObject):
                 self.rebuild_page()
             self.init_page()
 
-        skin.draw('menu', self, self.menustack[-1])
+        blend = config.FREEVO_USE_ALPHABLENDING and self.first_drawing
+        skin.draw('menu', self, self.menustack[-1], blend)
+
+        self.first_drawing = False
 
 
+    @benchmark(benchmarking, benchmarkcall)
     def make_submenu(self, menu_name, actions, item):
+        #print 'make_submenu(menu_name=%r, actions=%r, item=%r)' % (menu_name, actions, item)
         items = []
         for a in actions:
             if isinstance(a, Item):
@@ -437,6 +490,269 @@ class MenuWidget(GUIObject):
         self.pushmenu(s)
 
 
+    @benchmark(benchmarking, benchmarkcall)
+    def _handle_up(self, menu, event):
+        curr_selected = self.all_items.index(menu.selected)
+        sounds.play_sound(sounds.MENU_NAVIGATE)
+        if curr_selected-self.cols < 0 and \
+               menu.selected != menu.choices[0]:
+            self.goto_prev_page(arg='no_refresh')
+            try:
+                if self.cols == 1:
+                    curr_selected = self.rows - 1
+                elif self.rows != 1:
+                    curr_selected = self.all_items.index(menu.selected)
+                else:
+                    curr_selected+=self.cols
+            except ValueError:
+                curr_selected += self.cols
+        curr_selected = max(curr_selected-self.cols, 0)
+        menu.selected = self.all_items[curr_selected]
+        self.refresh()
+
+
+    @benchmark(benchmarking, benchmarkcall)
+    def _handle_down(self, menu, event):
+        curr_selected = self.all_items.index(menu.selected)
+        sounds.play_sound(sounds.MENU_NAVIGATE)
+        if curr_selected+self.cols > len(self.all_items)-1 and \
+               menu.page_start + len(self.all_items) < len(menu.choices):
+
+            self.goto_next_page(arg='no_refresh')
+            try:
+                if self.cols == 1:
+                    curr_selected = 0
+                elif self.rows != 1:
+                    curr_selected = self.all_items.index(menu.selected)
+                else:
+                    curr_selected-=self.cols
+            except ValueError:
+                curr_selected -= self.cols
+        curr_selected = min(curr_selected+self.cols, len(self.all_items)-1)
+        menu.selected = self.all_items[curr_selected]
+        self.refresh()
+
+
+    @benchmark(benchmarking, benchmarkcall)
+    def _handle_pageup(self, menu, event):
+        # Do nothing for an empty file list
+        if not len(self.menu_items):
+            return
+
+        curr_selected = self.all_items.index(menu.selected)
+
+        # Move to the previous page if the current position is at the
+        # top of the list, otherwise move to the top of the list.
+        if curr_selected == 0:
+            self.goto_prev_page()
+        else:
+            curr_selected = 0
+            menu.selected = self.all_items[curr_selected]
+            self.refresh()
+        return
+
+
+    @benchmark(benchmarking, benchmarkcall)
+    def _handle_pagedown(self, menu, event):
+        # Do nothing for an empty file list
+        if not len(self.menu_items):
+            return
+
+        if menu.selected == menu.choices[-1]:
+            return
+
+        curr_selected = self.all_items.index(menu.selected)
+        bottom_index = self.menu_items.index(self.menu_items[-1])
+
+        # Move to the next page if the current position is at the
+        # bottom of the list, otherwise move to the bottom of the list.
+        if curr_selected >= bottom_index:
+            self.goto_next_page()
+        else:
+            curr_selected = bottom_index
+            menu.selected = self.all_items[curr_selected]
+            self.refresh()
+        return
+
+    @benchmark(benchmarking, benchmarkcall)
+    def _handle_left(self, menu, event):
+        # Do nothing for an empty file list
+        if not len(self.menu_items):
+            return
+
+        sounds.play_sound(sounds.MENU_NAVIGATE)
+        curr_selected = self.all_items.index(menu.selected)
+        if curr_selected == 0:
+            self.goto_prev_page(arg='no_refresh')
+            try:
+                curr_selected = self.all_items.index(menu.selected)
+                if self.rows == 1:
+                    curr_selected = len(self.all_items)
+            except ValueError:
+                curr_selected += self.cols
+        curr_selected = max(curr_selected-1, 0)
+        menu.selected = self.all_items[curr_selected]
+        self.refresh()
+        return
+
+
+    @benchmark(benchmarking, benchmarkcall)
+    def _handle_right(self, menu, event):
+        # Do nothing for an empty file list
+        if not len(self.menu_items):
+            return
+
+        sounds.play_sound(sounds.MENU_NAVIGATE)
+        curr_selected = self.all_items.index(menu.selected)
+        if curr_selected == len(self.all_items)-1:
+            self.goto_next_page(arg='no_refresh')
+            try:
+                curr_selected = self.all_items.index(menu.selected)
+                if self.rows == 1:
+                    curr_selected -= 1
+            except ValueError:
+                curr_selected -= self.cols
+
+        curr_selected = min(curr_selected+1, len(self.all_items)-1)
+        menu.selected = self.all_items[curr_selected]
+        self.refresh()
+        return
+
+
+    @benchmark(benchmarking, benchmarkcall)
+    def _handle_play_item(self, menu, event):
+        action = None
+        arg    = None
+
+        sounds.play_sound(sounds.MENU_SELECT)
+        try:
+            action = menu.selected.action
+        except AttributeError:
+            actions = menu.selected.actions() or []
+
+            # Add the actions of the plugins to the list of actions.  This is needed when a
+            # Item class has no actions but plugins provides them. This case happens with an
+            # empty disc.
+            #
+            # FIXME The event MENU_SELECT is called when selecting a submenu entry too. The
+            # item passed to the plugin is then the submenu entry instead its parent item. So
+            # if we are in a submenu we don't want to call the actions of the plugins.
+            # because we'll break some (or all) plugins behavior.  Does that sound correct?
+
+            if config.OSD_SOUNDS:
+                if hasattr(menu.selected, 'arg'):
+                    try:
+                        key = "menu." + menu.selected.arg[0]
+                        if config.OSD_SOUNDS[key]:
+                            sounds.play_sound(sounds.load_sound(key))
+                    except:
+                        pass
+                else:
+                    try:
+                        key = "menu." + menu.selected.__class__.__name__
+                        if config.OSD_SOUNDS[key]:
+                            sounds.play_sound(sounds.load_sound(key))
+                    except:
+                        pass
+
+            if not hasattr(menu, 'is_submenu'):
+                plugins = plugin.get('item') + plugin.get('item_%s' % menu.selected.type)
+
+                if hasattr(menu.selected, 'display_type'):
+                    plugins += plugin.get('item_%s' % menu.selected.display_type)
+
+                plugins.sort(lambda l, o: cmp(l._level, o._level))
+
+                for p in plugins:
+                    for a in p.actions(menu.selected):
+                        if isinstance(a, MenuItem):
+                            actions.append(a)
+                        else:
+                            actions.append(a[:2])
+
+            if actions:
+                action = actions[0]
+                if isinstance(action, MenuItem):
+                    action = action.function
+                    arg    = action.arg
+                else:
+                    action = action[0]
+        if not action:
+            AlertBox(text=_('No action defined for this choice!')).show()
+            return
+        action(arg=arg, menuw=self)
+
+
+    @benchmark(benchmarking, benchmarkcall)
+    def _handle_submenu(self, menu, event):
+        action = None
+        arg    = None
+        #try:
+        #    action = menu.selected.action
+        #except AttributeError:
+        #    pass
+
+        actions = menu.selected.actions()
+        force   = False
+        if not actions:
+            actions = []
+            force   = True
+
+        if not hasattr(menu, 'is_submenu'):
+            plugins = plugin.get('item') + plugin.get('item_%s' % menu.selected.type)
+
+            if hasattr(menu.selected, 'display_type'):
+                plugins += plugin.get('item_%s' % menu.selected.display_type)
+
+            plugins.sort(lambda l, o: cmp(l._level, o._level))
+
+            for p in plugins:
+                for a in p.actions(menu.selected):
+                    if isinstance(a, MenuItem):
+                        actions.append(a)
+                    else:
+                        actions.append(a[:2])
+                        if len(a) == 3 and a[2] == 'MENU_SUBMENU':
+                            a[0](menuw=self)
+                            return
+
+        if actions:
+            if len(actions) > 1 or force:
+                self.make_submenu(menu.selected.name, actions, menu.selected)
+            elif len(actions) == 1:
+                # if there is only one action, call it!
+                action = actions[0]
+                if isinstance(action, MenuItem):
+                    action = action.function
+                    arg    = action.arg
+                else:
+                    action = action[0]
+                action(arg=arg, menuw=self)
+
+
+    @benchmark(benchmarking, benchmarkcall)
+    def _handle_call_item_action(self, menu, event):
+        _debug_('calling action %s' % event.arg)
+
+        for a in menu.selected.actions():
+            if not isinstance(a, Item) and len(a) > 2 and a[2] == event.arg:
+                a[0](arg=None, menuw=self)
+                return
+
+        plugins = plugin.get('item') + plugin.get('item_%s' % menu.selected.type)
+
+        if hasattr(menu.selected, 'display_type'):
+            plugins += plugin.get('item_%s' % menu.selected.display_type)
+
+        for p in plugins:
+            for a in p.actions(menu.selected):
+                if not isinstance(a, MenuItem) and len(a) > 2 and a[2] == event.arg:
+                    a[0](arg=None, menuw=self)
+                    return
+        _debug_('action %s not found' % event.arg)
+
+
+    @benchmark(benchmarking, benchmarkcall)
     def eventhandler(self, event):
         menu = self.menustack[-1]
 
@@ -515,7 +831,7 @@ class MenuWidget(GUIObject):
                 return
             menu = self.menustack[-2]
             if hasattr(menu.selected, 'eventhandler') and menu.selected.eventhandler:
-                if menu.selected.eventhandler(event = event, menuw=self):
+                if menu.selected.eventhandler(event=event, menuw=self):
                     return
             for p in self.eventhandler_plugins:
                 if p.eventhandler(event=event, menuw=self):
@@ -534,265 +850,34 @@ class MenuWidget(GUIObject):
             return
 
         if event == MENU_UP:
-            curr_selected = self.all_items.index(menu.selected)
-            sounds.play_sound(sounds.MENU_NAVIGATE)
-            if curr_selected-self.cols < 0 and \
-                   menu.selected != menu.choices[0]:
-                self.goto_prev_page(arg='no_refresh')
-                try:
-                    if self.cols == 1:
-                        curr_selected = self.rows - 1
-                    elif self.rows != 1:
-                        curr_selected = self.all_items.index(menu.selected)
-                    else:
-                        curr_selected+=self.cols
-                except ValueError:
-                    curr_selected += self.cols
-            curr_selected = max(curr_selected-self.cols, 0)
-            menu.selected = self.all_items[curr_selected]
-            self.refresh()
-            return
-
+            self._handle_up(menu, event)
 
         elif event == MENU_DOWN:
-            curr_selected = self.all_items.index(menu.selected)
-            sounds.play_sound(sounds.MENU_NAVIGATE)
-            if curr_selected+self.cols > len(self.all_items)-1 and \
-                   menu.page_start + len(self.all_items) < len(menu.choices):
-
-                self.goto_next_page(arg='no_refresh')
-                try:
-                    if self.cols == 1:
-                        curr_selected = 0
-                    elif self.rows != 1:
-                        curr_selected = self.all_items.index(menu.selected)
-                    else:
-                        curr_selected-=self.cols
-                except ValueError:
-                    curr_selected -= self.cols
-            curr_selected = min(curr_selected+self.cols, len(self.all_items)-1)
-            menu.selected = self.all_items[curr_selected]
-            self.refresh()
-            return
-
+            self._handle_down(menu, event)
 
         elif event == MENU_PAGEUP:
-            # Do nothing for an empty file list
-            if not len(self.menu_items):
-                return
-
-            curr_selected = self.all_items.index(menu.selected)
-
-            # Move to the previous page if the current position is at the
-            # top of the list, otherwise move to the top of the list.
-            if curr_selected == 0:
-                self.goto_prev_page()
-            else:
-                curr_selected = 0
-                menu.selected = self.all_items[curr_selected]
-                self.refresh()
-            return
-
+            self._handle_pageup(menu, event)
 
         elif event == MENU_PAGEDOWN:
-            # Do nothing for an empty file list
-            if not len(self.menu_items):
-                return
-
-            if menu.selected == menu.choices[-1]:
-                return
-
-            curr_selected = self.all_items.index(menu.selected)
-            bottom_index = self.menu_items.index(self.menu_items[-1])
-
-            # Move to the next page if the current position is at the
-            # bottom of the list, otherwise move to the bottom of the list.
-            if curr_selected >= bottom_index:
-                self.goto_next_page()
-            else:
-                curr_selected = bottom_index
-                menu.selected = self.all_items[curr_selected]
-                self.refresh()
-            return
-
+            self._handle_pagedown(menu, event)
 
         elif event == MENU_LEFT:
-            # Do nothing for an empty file list
-            if not len(self.menu_items):
-                return
-
-            sounds.play_sound(sounds.MENU_NAVIGATE)
-            curr_selected = self.all_items.index(menu.selected)
-            if curr_selected == 0:
-                self.goto_prev_page(arg='no_refresh')
-                try:
-                    curr_selected = self.all_items.index(menu.selected)
-                    if self.rows == 1:
-                        curr_selected = len(self.all_items)
-                except ValueError:
-                    curr_selected += self.cols
-            curr_selected = max(curr_selected-1, 0)
-            menu.selected = self.all_items[curr_selected]
-            self.refresh()
-            return
-
+            self._handle_left(menu, event)
 
         elif event == MENU_RIGHT:
-            # Do nothing for an empty file list
-            if not len(self.menu_items):
-                return
-
-            sounds.play_sound(sounds.MENU_NAVIGATE)
-            curr_selected = self.all_items.index(menu.selected)
-            if curr_selected == len(self.all_items)-1:
-                self.goto_next_page(arg='no_refresh')
-                try:
-                    curr_selected = self.all_items.index(menu.selected)
-                    if self.rows == 1:
-                        curr_selected -= 1
-                except ValueError:
-                    curr_selected -= self.cols
-
-            curr_selected = min(curr_selected+1, len(self.all_items)-1)
-            menu.selected = self.all_items[curr_selected]
-            self.refresh()
-            return
-
+            self._handle_right(menu, event)
 
         elif event == MENU_PLAY_ITEM and hasattr(menu.selected, 'play'):
             menu.selected.play(menuw=self)
 
-        elif event == MENU_SELECT or event == MENU_PLAY_ITEM:
-            action = None
-            arg    = None
-
-            sounds.play_sound(sounds.MENU_SELECT)
-            try:
-                action = menu.selected.action
-            except AttributeError:
-                actions = menu.selected.actions()
-                if not actions:
-                    actions = []
-
-                # Add the actions of the plugins to the list of actions.
-                # This is needed when a Item class has no actions but plugins
-                # provides them. This case happens with an empty disc.
-                #
-                # FIXME The event MENU_SELECT is called when selecting
-                # a submenu entry too. The item passed to the plugin is then
-                # the submenu entry instead its parent item. So if we are in
-                # a submenu we don't want to call the actions of the plugins.
-                # because we'll break some (or all) plugins behavior.
-                # Does that sound correct?
-                #
-                if config.OSD_SOUNDS:
-                    if hasattr(menu.selected, 'arg'):
-                        try:
-                            key = "menu." + menu.selected.arg[0]
-                            if config.OSD_SOUNDS[key]:
-                                sounds.play_sound(sounds.load_sound(key))
-                        except:
-                            pass
-                    else:
-                        try:
-                            key = "menu." + menu.selected.__class__.__name__
-                            if config.OSD_SOUNDS[key]:
-                                sounds.play_sound(sounds.load_sound(key))
-                        except:
-                            pass
-
-                if not hasattr(menu, 'is_submenu'):
-                    plugins = plugin.get('item') + plugin.get('item_%s' % menu.selected.type)
-
-                    if hasattr(menu.selected, 'display_type'):
-                        plugins += plugin.get('item_%s' % menu.selected.display_type)
-
-                    plugins.sort(lambda l, o: cmp(l._level, o._level))
-
-                    for p in plugins:
-                        for a in p.actions(menu.selected):
-                            if isinstance(a, MenuItem):
-                                actions.append(a)
-                            else:
-                                actions.append(a[:2])
-
-                if actions:
-                    action = actions[0]
-                    if isinstance(action, MenuItem):
-                        action = action.function
-                        arg    = action.arg
-                    else:
-                        action = action[0]
-            if not action:
-                print 'No action.. '
-                AlertBox(text=_('No action defined for this choice!')).show()
-            else:
-                action(arg=arg, menuw=self)
-            return
-
+        elif event == MENU_PLAY_ITEM or event == MENU_SELECT:
+            self._handle_play_item(menu, event)
 
         elif event == MENU_SUBMENU:
-            if hasattr(menu, 'is_submenu'):
-                return
-
-            actions = menu.selected.actions()
-            force   = False
-            if not actions:
-                actions = []
-                force   = True
-
-            plugins = plugin.get('item') + plugin.get('item_%s' % menu.selected.type)
-
-            if hasattr(menu.selected, 'display_type'):
-                plugins += plugin.get('item_%s' % menu.selected.display_type)
-
-            plugins.sort(lambda l, o: cmp(l._level, o._level))
-
-            for p in plugins:
-                for a in p.actions(menu.selected):
-                    if isinstance(a, MenuItem):
-                        actions.append(a)
-                    else:
-                        actions.append(a[:2])
-                        if len(a) == 3 and a[2] == 'MENU_SUBMENU':
-                            a[0](menuw=self)
-                            return
-
-            if actions:
-                if len(actions) > 1 or force:
-                    self.make_submenu(menu.selected.name, actions, menu.selected)
-                elif len(actions) == 1:
-                    # if there is only one action, call it!
-                    action = actions[0]
-                    arg = None
-                    if isinstance(action, MenuItem):
-                        action = action.function
-                        arg    = action.arg
-                    else:
-                        action = action[0]
-                    action(arg=arg, menuw=self)
-            return
+            self._handle_submenu(menu, event)
 
         elif event == MENU_CALL_ITEM_ACTION:
-            _debug_('calling action %s' % event.arg)
-
-            for a in menu.selected.actions():
-                if not isinstance(a, Item) and len(a) > 2 and a[2] == event.arg:
-                    a[0](arg=None, menuw=self)
-                    return
-
-            plugins = plugin.get('item') + plugin.get('item_%s' % menu.selected.type)
-
-            if hasattr(menu.selected, 'display_type'):
-                plugins += plugin.get('item_%s' % menu.selected.display_type)
-
-            for p in plugins:
-                for a in p.actions(menu.selected):
-                    if not isinstance(a, MenuItem) and len(a) > 2 and a[2] == event.arg:
-                        a[0](arg=None, menuw=self)
-                        return
-            _debug_('action %s not found' % event.arg)
-
+            self._handle_call_item_action(menu, event)
 
         elif event == MENU_CHANGE_STYLE and len(self.menustack) > 1:
             # did the menu change?
@@ -801,7 +886,6 @@ class MenuWidget(GUIObject):
                 self.refresh()
                 return
 
-
         elif hasattr(menu.selected, 'eventhandler') and menu.selected.eventhandler:
             if menu.selected.eventhandler(event=event, menuw=self):
                 return
@@ -815,6 +899,37 @@ class MenuWidget(GUIObject):
 
 
 
+    @benchmark(benchmarking, benchmarkcall)
+    def highlight_menuitem(self, clicked_menu):
+        i = 0
+        menu = self.menustack[-1]
+        _debug_('clicked_menu=%s, self.all_item=%s' % (clicked_menu, self.all_items[0]))
+        for menuitem in self.all_items:
+            if clicked_menu == menuitem:
+                sounds.play_sound(sounds.MENU_NAVIGATE)
+                curr_selected = i
+                menu.selected = self.all_items[curr_selected]
+                self.refresh()
+                return
+            i += 1
+
+    @benchmark(benchmarking, benchmarkcall)
+    def select_menuitem(self, clicked_menu):
+        self.eventhandler(MENU_SELECT)
+
+    @benchmark(benchmarking, benchmarkcall)
+    def submenu_menuitem(self):
+        self.eventhandler(MENU_SUBMENU)
+
+    @benchmark(benchmarking, benchmarkcall)
+    def up_menuitem(self):
+        self.eventhandler(MENU_UP)
+
+    @benchmark(benchmarking, benchmarkcall)
+    def down_menuitem(self):
+        self.eventhandler(MENU_DOWN)
+
+    @benchmark(benchmarking, benchmarkcall)
     def rebuild_page(self):
         menu = self.menustack[-1]
 
@@ -843,8 +958,11 @@ class MenuWidget(GUIObject):
         menu.display_style = skin.get_display_style(menu)
 
 
+    @benchmark(benchmarking, benchmarkcall)
     def init_page(self):
 
+        self.first_drawing = True
+
         menu = self.menustack[-1]
         if not menu:
             return
