@@ -1,13 +1,8 @@
 # -*- coding: iso-8859-1 -*-
 # -----------------------------------------------------------------------
-# osd.py - Low level graphics routines
+# Low level graphics routines
 # -----------------------------------------------------------------------
-# $Id: osd.py 10559 2008-03-22 11:42:02Z duncan $
-#
-# Notes: do not use the OSD object inside a thread
-#
-# Todo:
-#
+# $Id: osd.py 11205 2008-12-01 02:42:54Z duncan $
 # -----------------------------------------------------------------------
 # Freevo - A Home Theater PC framework
 # Copyright (C) 2002 Krister Lagerstrom, et al.
@@ -29,12 +24,17 @@
 #
 # -----------------------------------------------------------------------
 
+"""
+Low level OSD (On Screen Display) graphics routines
+
+B{NOTE} Do not use the OSD object inside a thread.
+"""
 
 # Python modules
 import time
 import os
 import stat
-import kaa.imlib2 as Image
+import kaa.imlib2
 import re
 import traceback
 import threading, thread
@@ -57,8 +57,6 @@ if __freevo_app__ == 'main':
     import animation
 
 
-
-
 help_text = """\
 h       HELP
 z       Toggle Fullscreen
@@ -239,12 +237,13 @@ class BusyIcon(threading.Thread):
     def __init__(self):
         threading.Thread.__init__(self)
         self.setDaemon(1)
-        self.mode_flag   = threading.Event()
-        threading.Thread.start(self)
+        self.lock = threading.Lock()
+        self.mode_flag = threading.Event()
         self.timer  = 0
         self.active = False
-        self.lock   = threading.RLock()
         self.rect   = None
+        threading.Thread.start(self)
+
 
     def wait(self, timer):
         self.lock.acquire()
@@ -255,11 +254,13 @@ class BusyIcon(threading.Thread):
         finally:
             self.lock.release()
 
+
     def stop(self):
         self.lock.acquire()
         self.active = False
         self.lock.release()
 
+
     def run(self):
         while (1):
             self.mode_flag.clear()
@@ -280,16 +281,15 @@ class BusyIcon(threading.Thread):
                         x = osd.width  - config.OSD_OVERSCAN_RIGHT - 20 - width
                         y = osd.height - config.OSD_OVERSCAN_BOTTOM - 20 - height
 
-                        self.rect = (x,y,width,height)
+                        self.rect = (x, y, width, height)
                         # backup the screen
-                        screen = pygame.Surface((width,height))
-                        screen.blit(osd.screen, (0,0), self.rect)
+                        screen = pygame.Surface((width, height))
+                        screen.blit(osd.screen, (0, 0), self.rect)
                         # draw the icon
                         osd.drawbitmap(image, x, y)
                         osd.update(rect=self.rect, stop_busyicon=False)
-
                         # restore the screen
-                        osd.screen.blit(screen, (x,y))
+                        osd.screen.blit(screen, (x, y))
                 finally:
                     self.lock.release()
 
@@ -298,7 +298,9 @@ class BusyIcon(threading.Thread):
 
 
 class OSD:
-
+    """
+    On-screen display class
+    """
     # Some default colors
     COL_RED = 0xff0000
     COL_GREEN = 0x00ff00
@@ -315,7 +317,7 @@ class OSD:
 
     def __init__(self):
         """
-        init the osd
+        Initialize an instance of OSD
         """
         self.fullscreen = 0 # Keep track of fullscreen state
         self.app_list = []
@@ -337,26 +339,25 @@ class OSD:
             os.environ['SDL_VIDEODRIVER'] = 'directfb'
 
         # sometimes this fails
-        if not os.environ.has_key('SDL_VIDEODRIVER') and \
-               config.CONF.display == 'x11':
+        if not os.environ.has_key('SDL_VIDEODRIVER') and config.CONF.display == 'x11':
             os.environ['SDL_VIDEODRIVER'] = 'x11'
 
 
         # disable term blanking for mga and fbcon and restore the
         # tty so that sdl can use it
         if config.CONF.display in ('mga', 'fbcon'):
-            for i in range(0,7):
+            for i in range(0, 7):
                 try:
-                    fd = os.open('/dev/tty%s' % i, os.O_RDONLY | os.O_NONBLOCK)
+                    device = '/dev/tty%s' % i
+                    fd = os.open(device, os.O_RDONLY | os.O_NONBLOCK)
                     try:
                         # set ioctl (tty, KDSETMODE, KD_TEXT)
                         ioctl(fd, 0x4B3A, 0)
-                    except:
-                        pass
+                    except Exception, why:
+                        print 'Cannot set KD_TEXT on term %s: %s' % (device, why)
                     os.close(fd)
                     os.system('%s -term linux -cursor off -blank 0 -clear -powerdown 0 ' \
-                              '-powersave off </dev/tty%s > /dev/tty%s 2>/dev/null' % \
-                              (config.CONF.setterm, i,i))
+                        '-powersave off </dev/tty%s > /dev/tty%s 2>/dev/null' %  (config.CONF.setterm, i, i))
                 except:
                     pass
 
@@ -372,26 +373,30 @@ class OSD:
         if config.CONF.display == 'dxr3':
             self.depth = 32
 
-        if config.CONF.x or config.CONF.y:
-            os.putenv('SDL_VIDEO_WINDOW_POS', '%d,%d' % (config.CONF.x, config.CONF.y))
+        os.putenv('SDL_VIDEO_WINDOW_POS', '%d,%d' % (config.CONF.x, config.CONF.y))
 
         try:
-            self.screen = pygame.display.set_mode((self.width, self.height),
-                                                  self.hw, self.depth)
+            self.screen = pygame.display.set_mode((self.width, self.height), self.hw, self.depth)
         except:
             self.screen = pygame.display.set_mode((self.width, self.height))
 
         self.depth     = self.screen.get_bitsize()
         self.must_lock = self.screen.mustlock()
+        self.main_layer = pygame.Surface((self.width, self.height))
+        self.dialog_layer_enabled = False
+        self.dialog_layer = self.screen.convert_alpha()
+        self.dialog_layer.fill((0,0,0,128))
 
         if config.CONF.display == 'x11' and config.START_FULLSCREEN_X == 1:
             self.toggle_fullscreen()
 
-        help = [ _('z = Toggle Fullscreen'),
-                 _('Arrow Keys = Move'),
-                 _('Spacebar = Select'),
-                 _('Escape = Stop/Prev. Menu'),
-                 _('h = Help') ]
+        help = [
+            _('z = Toggle Fullscreen'),
+            _('Arrow Keys = Move'),
+            _('Spacebar = Select'),
+            _('Escape = Stop/Prev. Menu'),
+            _('h = Help')
+        ]
 
         help_str = '    '.join(help)
 
@@ -407,7 +412,7 @@ class OSD:
             os.system(config.OSD_SDL_EXEC_AFTER_STARTUP)
 
         self.sdl_driver = pygame.display.get_driver()
-        _debug_('SDL Driver: %s' % (str(self.sdl_driver)),2)
+        _debug_('SDL Driver: %s' % (str(self.sdl_driver)), 2)
 
         pygame.mouse.set_visible(0)
         pygame.key.set_repeat(500, 30)
@@ -429,8 +434,7 @@ class OSD:
         # starts the render and it's thread
         self.render = animation.render.get_singleton()
 
-        pygame.time.delay(10)   # pygame.time.get_ticks don't seem to
-                                # work otherwise
+        pygame.time.delay(10)   # pygame.time.get_ticks don't seem to work otherwise
 
 
     def focused_app(self):
@@ -449,10 +453,16 @@ class OSD:
         for _time in range(_times):
             self.app_list.remove(app)
         if _times and hasattr(self.focused_app(), 'event_context'):
-            _debug_('app is %s' % self.focused_app(),2)
-            _debug_('osd: Setting context to %s' % self.focused_app().get_event_context(),2)
+            _debug_('app is %s' % self.focused_app(), 2)
+            _debug_('osd: Setting context to %s' % self.focused_app().get_event_context(), 2)
             rc.set_context(self.focused_app().get_event_context())
 
+    def __find_current_widget__(self, widget):
+        if not widget:
+            return None
+        if not hasattr(widget, 'menustack'):
+            return self.__find_current_widget__(widget.parent)
+        return widget
 
     def _cb(self, map=True):
         """
@@ -481,6 +491,78 @@ class OSD:
                 return
 
             _debug_('pygame event=%s' % (event), 2)
+            #print 'pygame event=%s' % (event)
+
+            if config.SYS_USE_MOUSE:
+                if event.type == MOUSEMOTION:
+                    app = self.focused_app()
+                    # Menu
+                    if app and hasattr(app, 'menustack'):
+                        menu = app.menustack[-1]
+                        if menu and hasattr(menu, 'choices'):
+                            for menuitem in menu.choices:
+                                if menuitem.rect.collidepoint(event.pos):
+                                    app.highlight_menuitem(menuitem)
+                    # Box with 2 buttons
+                    elif app and hasattr(app, 'b0') and hasattr(app, 'b1'):
+                        if app.b0.rect.collidepoint(event.pos):
+                            if not app.b0.selected:
+                                app.b0.toggle_selected()
+                                app.b1.toggle_selected()
+                                app.draw()
+                        elif app.b1.rect.collidepoint(event.pos):
+                            if not app.b1.selected:
+                                app.b0.toggle_selected()
+                                app.b1.toggle_selected()
+                                app.draw()
+
+
+                if event.type == MOUSEBUTTONDOWN:
+                    app = self.focused_app()
+                    # Menu
+                    if app and hasattr(app, 'back_one_menu'):
+                        # Left click = Enter
+                        if event.button == 1:
+                            menu = app.menustack[-1]
+                            if menu and hasattr(menu, 'choices'):
+                                for menuitem in menu.choices:
+                                    if menuitem.rect.collidepoint(event.pos):
+                                        app.highlight_menuitem(menuitem)
+                                        app.select_menuitem(menuitem)
+                        # Middle click
+                        elif event.button == 2:
+                            app.submenu_menuitem()
+                        # Right click = Esc
+                        elif event.button == 3:
+                            app.back_one_menu()
+                        # Wheel up
+                        elif event.button == 4:
+                            app.up_menuitem()
+                        # Wheel down
+                        elif event.button == 5:
+                            app.down_menuitem()
+
+                    # Box
+                    elif app and hasattr(app, 'destroy'):
+                        # Left click = Enter
+                        if event.button == 1:
+                            # Box with 2 buttons
+                            if app and hasattr(app, 'b0') and hasattr(app, 'b1'):
+                                if app.b0.rect.collidepoint(event.pos):
+                                    if not app.b0.selected:
+                                        app.b0.toggle_selected()
+                                        app.b1.toggle_selected()
+                                        app.draw()
+                                    app.send_enter_event()
+                                elif app.b1.rect.collidepoint(event.pos):
+                                    if not app.b1.selected:
+                                        app.b0.toggle_selected()
+                                        app.b1.toggle_selected()
+                                        app.draw()
+                                    app.send_enter_event()
+                        # Right click = Esc
+                        elif event.button == 3:
+                            app.destroy()
 
             if event.type == KEYDOWN:
                 if not map and event.key > 30:
@@ -504,8 +586,7 @@ class OSD:
 
                 elif event.key == K_F10:
                     # Take a screenshot
-                    pygame.image.save(self.screen,
-                                      '/tmp/freevo_ss%s.bmp' % self._screenshotnum)
+                    pygame.image.save(self.screen, '/tmp/freevo_ss%s.bmp' % self._screenshotnum)
                     self._screenshotnum += 1
 
                 else:
@@ -516,6 +597,7 @@ class OSD:
                     except:
                         return None
 
+
     def shutdown(self):
         """
         shutdown the display
@@ -540,7 +622,7 @@ class OSD:
 
             # backup the screen
             self.__stop_screen__ = pygame.Surface((self.width, self.height))
-            self.__stop_screen__.blit(self.screen, (0,0))
+            self.__stop_screen__.blit(self.screen, (0, 0))
             pygame.display.quit()
         finally:
             self.mutex.release()
@@ -561,7 +643,7 @@ class OSD:
             self.screen = pygame.display.set_mode((self.width, self.height), self.hw,
                                                   self.depth)
             if hasattr(self, '__stop_screen__'):
-                self.screen.blit(self.__stop_screen__, (0,0))
+                self.screen.blit(self.__stop_screen__, (0, 0))
                 del self.__stop_screen__
 
             # We need to go back to fullscreen mode if that was the mode before the shutdown
@@ -602,7 +684,7 @@ class OSD:
             color = self.default_bg_color
         self.mutex.acquire()
         try:
-            self.screen.fill(self._sdlcol(color))
+            self.main_layer.fill(self._sdlcol(color))
         finally:
             self.mutex.release()
 
@@ -613,93 +695,123 @@ class OSD:
         print 'mode=%s %s' % (type(data[2]), data[2])
 
 
+    def _load_image_imlib2(self, data):
+        """
+        Load an image from an imlib2 image object
+        """
+        _debug_('_load_image_imlib2(data=%r)' % (data,), 2)
+        if data.mode == 'BGRA':
+            data.mode = 'RGBA'
+        image = pygame.image.fromstring(str(data.get_raw_data(format=data.mode)), data.size, data.mode)
+        return image
+
+
+    def _load_image_filename(self, url):
+        """
+        Load an image from a file name
+        """
+        _debug_('_load_image_filename(url=%r)' % (url,), 2)
+        if url[:8] == 'thumb://':
+            filename = os.path.abspath(url[8:])
+            thumbnail = True
+        else:
+            filename = os.path.abspath(url)
+            thumbnail = False
+
+        if not os.path.isfile(filename):
+            fname = os.path.join(config.IMAGE_DIR, filename)
+            _debug_('Bitmap file "%s" doesn\'t exist!' % filename, DWARNING)
+            if config.DEBUG:
+                traceback.print_stack()
+            return None
+
+        try:
+            if filename.endswith('.raw'):
+                # load cache
+                data  = util.read_thumbnail(filename)
+                #self.printdata(data)
+                # convert to pygame image
+                image = pygame.image.fromstring(data[0], data[1], data[2])
+
+            elif thumbnail:
+                # load cache or create it
+                data = util.cache_image(filename)
+                #self.printdata(data)
+                # convert to pygame image
+                try:
+                    image = pygame.image.fromstring(data[0], data[1], data[2])
+                except:
+                    data = util.create_thumbnail(filename)
+                    image = pygame.image.fromstring(data[0], data[1], data[2])
+
+            else:
+                try:
+                    image = pygame.image.load(filename)
+                except pygame.error, e:
+                    _debug_('SDL image load problem: %s - trying imlib2' % e, DINFO)
+                    try:
+                        i = kaa.imlib2.open(filename)
+                        image = pygame.image.fromstring(i.tostring(), i.size, i.mode)
+                    except IOError, why:
+                        _debug_('imlib2 image load problem: %s' % (why), DERROR)
+                        return None
+
+        except SystemExit:
+            raise
+
+        except Exception, why:
+            _debug_('Problem while loading image %r: %r' % (url, why), DWARNING)
+            if config.DEBUG:
+                traceback.print_exc()
+            return None
+
+        return image
+
+
     def loadbitmap(self, url, cache=False):
         """
         Load a bitmap and return the pygame image object.
+
+        @param url: is the image to load
+        @type url: kaa.image, url or str
+        @param cache: Cache the image?
+        @type cache: bool or bitmapcache
+        @returns: pygame surfaceloadbitmap
+        @rtype: Surface or None
         """
-        if cache:
-            if cache == True:
-                cache = self.bitmapcache
-            s = cache[url]
-            if s:
-                return s
+        _debug_('loadbitmap(url=%r, cache=%r)' % (url, cache), 2)
 
         if not pygame.display.get_init():
             return None
 
-        try:
-            if url.mode == 'BGRA':
-                url.mode = 'RGBA'
-            image = pygame.image.fromstring(str(url.get_raw_data(format=url.mode)), url.size, url.mode)
-        except AttributeError:
-            pass
-
-            if url[:8] == 'thumb://':
-                filename = os.path.abspath(url[8:])
-                thumbnail = True
-            else:
-                filename = os.path.abspath(url)
-                thumbnail = False
-
-            if not os.path.isfile(filename):
-                filename = os.path.join(config.IMAGE_DIR, url[8:])
-
-            if not os.path.isfile(filename):
-                _debug_('Bitmap file "%s" doesn\'t exist!' % filename, DWARNING)
-                #raise 'Bitmap file'
-                return None
-
-            try:
-                if isstring(filename) and filename.endswith('.raw'):
-                    # load cache
-                    data  = util.read_thumbnail(filename)
-                    #self.printdata(data)
-                    # convert to pygame image
-                    image = pygame.image.fromstring(data[0], data[1], data[2])
-
-                elif thumbnail:
-                    # load cache or create it
-                    data = util.cache_image(filename)
-                    #self.printdata(data)
-                    # convert to pygame image
-                    try:
-                        image = pygame.image.fromstring(data[0], data[1], data[2])
-                    except:
-                        data = util.create_thumbnail(filename)
-                        #self.printdata(data)
-                        image = pygame.image.fromstring(data[0], data[1], data[2])
-                else:
-                    try:
-                        image = pygame.image.load(filename)
-                    except pygame.error, e:
-                        _debug_('SDL image load problem: %s - trying imlib2' % e, DINFO)
-                        try:
-                            i = Image.open(filename)
-                            image = pygame.image.fromstring(i.tostring(), i.size, i.mode)
-                        except IOError, why:
-                            _debug_('imlib2 image load problem: %s' % (why), DERROR)
-
-            except:
-                _debug_('Problem while loading image "%s"' % String(url), DWARNING)
-                if config.DEBUG:
-                    traceback.print_exc()
-                return None
-
-        except Exception, e:
-            _debug_('image.fromstring: %r' % e)
-            return None
+        if isinstance(url, kaa.imlib2.Image):
+            image = self._load_image_imlib2(url)
+        elif isinstance(url, basestring):
+            if cache:
+                if cache == True:
+                    cache = self.bitmapcache
+                s = cache[url]
+                if s:
+                    return s
+            image = self._load_image_filename(url)
+            if image:
+                if cache:
+                    cache[url] = image
+        else:
+            raise 'wrong image type%r' % url
 
         # convert the surface to speed up blitting later
-        if image and image.get_alpha():
-            image.set_alpha(image.get_alpha(), RLEACCEL)
-
-        if cache:
-            cache[url] = image
+        if image:
+            if image.get_alpha():
+                image.set_alpha(image.get_alpha(), RLEACCEL)
+            else:
+                #if image.get_alpha() is None:
+                #    image.set_colorkey(-1)
+                image = image.convert()
         return image
 
 
-    def drawbitmap(self, image, x=0, y=0, scaling=None,
-                   bbx=0, bby=0, bbw=0, bbh=0, rotation = 0, layer=None):
+    def drawbitmap(self, image, x=0, y=0, scaling=None, bbx=0, bby=0, bbw=0, bbh=0, rotation=0, layer=None):
         """
         Draw a bitmap on the OSD. It is automatically loaded into the cache
         if not already there.
@@ -711,8 +823,7 @@ class OSD:
         self.drawsurface(image, x, y, scaling, bbx, bby, bbw, bbh, rotation, layer)
 
 
-    def drawsurface(self, image, x=0, y=0, scaling=None,
-                   bbx=0, bby=0, bbw=0, bbh=0, rotation=0, layer=None):
+    def drawsurface(self, image, x=0, y=0, scaling=None, bbx=0, bby=0, bbw=0, bbh=0, rotation=0, layer=None):
         """
         scales and rotates a surface and then draws it to the screen.
         """
@@ -734,7 +845,7 @@ class OSD:
             if layer:
                 layer.blit(image, (x, y))
             else:
-                self.screen.blit(image, (x, y))
+                self.main_layer.blit(image, (x, y))
         finally:
             self.mutex.release()
 
@@ -793,20 +904,20 @@ class OSD:
                 width = 1
 
             if width == -1 or fill:
-                r,g,b,a = self._sdlcol(color)
+                r, g, b, a = self._sdlcol(color)
                 w = x1 - x0
                 h = y1 - y0
                 box = pygame.Surface((int(w), int(h)))
-                box.fill((r,g,b))
+                box.fill((r, g, b))
                 box.set_alpha(a)
                 if layer:
                     layer.blit(box, (x0, y0))
                 else:
-                    self.screen.blit(box, (x0, y0))
+                    self.main_layer.blit(box, (x0, y0))
             else:
                 c = self._sdlcol(color)
                 if not layer:
-                    layer = self.screen
+                    layer = self.main_layer
                 for i in range(0, width):
                     # looks strange, but sometimes thinkness doesn't work
                     pygame.draw.rect(layer, c, (x0+i, y0+i, x1-x0-2*i, y1-y0-2*i), 1)
@@ -821,9 +932,9 @@ class OSD:
         self.mutex.acquire()
         try:
             if rect != None:
-                return self.screen.subsurface(rect).convert()
+                return self.main_layer.subsurface(rect).convert()
             else:
-                return self.screen.subsurface( (x, y, width, height) ).convert()
+                return self.main_layer.subsurface( (x, y, width, height) ).convert()
         finally:
             self.mutex.release()
 
@@ -834,7 +945,7 @@ class OSD:
         """
         self.mutex.acquire()
         try:
-            self.screen.blit(surface, (x, y))
+            self.main_layer.blit(surface, (x, y))
         finally:
             self.mutex.release()
 
@@ -848,10 +959,10 @@ class OSD:
             if sourcerect:
                 w = sourcerect[2]
                 h = sourcerect[3]
-                ret = self.screen.blit(source, destpos, sourcerect)
+                ret = self.main_layer.blit(source, destpos, sourcerect)
             else:
                 w, h = source.get_size()
-                ret = self.screen.blit(source, destpos)
+                ret = self.main_layer.blit(source, destpos)
 
             if self.render:
                 self.render.damage( [(destpos[0], destpos[1], w, h)] )
@@ -861,7 +972,6 @@ class OSD:
         return ret
 
 
-
     def getfont(self, font, ptsize):
         """
         return cached font
@@ -875,8 +985,7 @@ class OSD:
             return fi
 
 
-    def __drawstringframed_line__(self, string, max_width, font, hard,
-                                  ellipses, word_splitter):
+    def __drawstringframed_line__(self, string, max_width, font, hard, ellipses, word_splitter):
         """
         calculate _one_ line for drawstringframed.
         @returns: a tuple containing
@@ -955,7 +1064,6 @@ class OSD:
         return (width+ellipses_size, string[:c]+ellipses, string[c:], False)
 
 
-
     def __draw_transparent_text__(self, surface, pixels=30):
         """
         Helper for drawing a transparency gradient end for strings
@@ -967,20 +1075,19 @@ class OSD:
             w, h       = surface.get_size()
             alpha      = pygame.surfarray.pixels_alpha(surface)
 
-            # transform all the alpha values in x,y range
+            # transform all the alpha values in x, y range
             # any speedup could help alot
             for x in range(max(w-pixels, 1), w):
                 for y in range(1, h):
-                    if alpha[x,y][0] != 0:
-                        alpha[x,y] = int(alpha[x,y][0]*opaque_mod)
+                    if alpha[x, y][0] != 0:
+                        alpha[x, y] = int(alpha[x, y][0]*opaque_mod)
                 opaque_mod -= opaque_stp
         except Exception, e:
             _debug_('__draw_transparent_text__: %s' % e, DERROR)
 
 
-    def drawstringframed(self, string, x, y, width, height, font, fgcolor=None,
-                         bgcolor=None, align_h='left', align_v='top', mode='hard',
-                         layer=None, ellipses='...', dim=True):
+    def drawstringframed(self, string, x, y, width, height, font, fgcolor=None, bgcolor=None,
+        align_h='left', align_v='top', mode='hard', layer=None, ellipses='...', dim=True):
         """
         draws a string (text) in a frame. This tries to fit the
         string in lines, if it can't, it truncates the text,
@@ -1005,10 +1112,10 @@ class OSD:
             or 'soft' (based on words)
         """
         if not pygame.display.get_init():
-            return '', (x,y,x,y)
+            return '', (x, y, x, y)
 
         if not string:
-            return '', (x,y,x,y)
+            return '', (x, y, x, y)
 
         shadow_x      = 0
         shadow_y      = 0
@@ -1055,7 +1162,7 @@ class OSD:
             line_height = int(line_height) + 1
 
         if width <= 0 or height < font.height:
-            return string, (x,y,x,y)
+            return string, (x, y, x, y)
 
         num_lines_left   = int((height+line_height-font.height) / line_height)
         lines            = []
@@ -1078,16 +1185,13 @@ class OSD:
             # rest that does not fit (r) and True if the breaking was because
             # of a \n (n)
             #
-            (w, s, r, n) = self.__drawstringframed_line__(string, width, font, hard,
-                                                          current_ellipses, ' ')
+            (w, s, r, n) = self.__drawstringframed_line__(string, width, font, hard, current_ellipses, ' ')
             if s == '' and not hard:
                 # nothing fits? Try to break words at ' -_' and no ellipses
-                (w, s, r, n) = self.__drawstringframed_line__(string, width, font, hard,
-                                                              None, ' -_')
+                (w, s, r, n) = self.__drawstringframed_line__(string, width, font, hard, None, ' -_')
                 if s == '':
                     # still nothing? Use the 'hard' way
-                    (w, s, r, n) = self.__drawstringframed_line__(string, width, font,
-                                                                  'hard', None, ' ')
+                    (w, s, r, n) = self.__drawstringframed_line__(string, width, font, 'hard', None, ' ')
             lines.append((w, s))
             while r and r[0] == '\n':
                 lines.append((0, ' '))
@@ -1121,7 +1225,7 @@ class OSD:
         max_x = 0
 
         if not layer and layer != '':
-            layer = self.screen
+            layer = self.main_layer
 
         fgcolor  = self._sdlcol(fgcolor)
         if border_color != None:
@@ -1149,9 +1253,8 @@ class OSD:
 
                     if bgcolor:
                         # draw a box around the text if we have a bgcolor
-                        self.drawbox(x0, y0, x0+render.get_size()[0],
-                                     y0+render.get_size()[1], color=bgcolor, fill=1,
-                                     layer=layer)
+                        self.drawbox(x0, y0, x0+render.get_size()[0], y0+render.get_size()[1],
+                            color=bgcolor, fill=1, layer=layer)
 
                     if border_color:
                         # draw the text 8 times with the border_color to get
@@ -1160,10 +1263,8 @@ class OSD:
                         if dim:
                             # draw on a tmp surface if we need to dim. It looks
                             # bad if we don't do that
-                            tmp = pygame.Surface((render.get_size()[0] + \
-                                                  2 * border_radius,
-                                                  render.get_size()[1] + \
-                                                  2 * border_radius)).convert_alpha()
+                            tmp = pygame.Surface((render.get_size()[0] + 2 * border_radius,
+                                                  render.get_size()[1] + 2 * border_radius)).convert_alpha()
                             tmp.fill((255, 0, 0, 0))
 
                             for ox in (0, border_radius, border_radius*2):
@@ -1171,7 +1272,7 @@ class OSD:
                                     if ox or oy:
                                         tmp.blit(re, (ox, oy))
                             tmp.blit(render, (border_radius, border_radius))
-                            self.__draw_transparent_text__(tmp,dim_size)
+                            self.__draw_transparent_text__(tmp, dim_size)
                             layer.blit(tmp, (x0-border_radius, y0-border_radius))
 
                         else:
@@ -1229,8 +1330,6 @@ class OSD:
         return r, (min_x, y, max_x, y+height_needed)
 
 
-
-
     def drawstring(self, string, x, y, fgcolor=None, bgcolor=None,
                    font=None, ptsize=0, align='left', layer=None):
         """
@@ -1259,9 +1358,8 @@ class OSD:
         elif align == 'right':
             tx -= width
 
-        self.drawstringframed(string, x, y, width, -1, self.getfont(font, ptsize),
-                              fgcolor, bgcolor, align_h = align, layer=layer,
-                              ellipses='')
+        self.drawstringframed(string, x, y, width, -1, self.getfont(font, ptsize), fgcolor, bgcolor,
+            align_h=align, layer=layer, ellipses='')
 
 
     def _savepixel(self, x, y, s):
@@ -1270,16 +1368,17 @@ class OSD:
         for drawcircle
         """
         try:
-            return (x, y, s.get_at((x,y)))
+            return (x, y, s.get_at((x, y)))
         except:
             return None
 
+
     def _restorepixel(self, save, s):
         """
         restore the saved pixel
         """
         if save:
-            s.set_at((save[0],save[1]), save[2])
+            s.set_at((save[0], save[1]), save[2])
 
 
     def drawcircle(self, s, color, x, y, radius):
@@ -1307,8 +1406,7 @@ class OSD:
         self._restorepixel(p6, s)
 
 
-    def drawroundbox(self, x0, y0, x1, y1, color=None, border_size=0, border_color=None,
-                     radius=0, layer=None):
+    def drawroundbox(self, x0, y0, x1, y1, color=None, border_size=0, border_color=None, radius=0, layer=None):
         """
         draw a round box
         """
@@ -1346,11 +1444,11 @@ class OSD:
                 box = pygame.Surface((w, h), SRCALPHA)
 
                 # clear surface
-                box.fill((0,0,0,0))
+                box.fill((0, 0, 0, 0))
             else:
                 box = layer
 
-            r,g,b,a = self._sdlcol(color)
+            r, g, b, a = self._sdlcol(color)
 
             if border_size:
                 if radius >= 1:
@@ -1376,14 +1474,12 @@ class OSD:
             pygame.draw.rect(box, c, (x, y+radius, w, h-2*radius))
 
             if not layer:
-                self.screen.blit(box, (x0, y0))
+                self.main_layer.blit(box, (x0, y0))
         finally:
             self.mutex.release()
 
 
-
-    def update(self, rect=None, blend_surface=None, blend_speed=0,
-               blend_steps=0, blend_time=None, stop_busyicon=True):
+    def update(self, rect=None, blend_surface=None, blend_speed=0, blend_steps=0, blend_time=None, stop_busyicon=True):
         """
         update the screen
         """
@@ -1399,13 +1495,28 @@ class OSD:
                 rect = None
 
             if rect and not (stop_busyicon and self.busyicon.rect):
+                if isinstance(rect[0], list) or isinstance(rect[0], tuple) or isinstance(rect[0], pygame.Rect):
+                    for sub_rect in rect:
+                        self.screen.blit(self.main_layer, (sub_rect[0], sub_rect[1]), sub_rect)
+                    if self.dialog_layer_enabled:
+                        for sub_rect in rect:
+                            self.screen.blit(self.dialog_layer, (sub_rect[0], sub_rect[1]), sub_rect)
+                else:
+                    try:
+                        self.screen.blit(self.main_layer, (rect[0], rect[1]), rect)
+                    except:
+                        traceback.print_exc()
+                    if self.dialog_layer_enabled:
+                        self.screen.blit(self.dialog_layer, (rect[0], rect[1]), rect)
                 try:
                     pygame.display.update(rect)
                 except:
-                    _debug_('osd.update(rect) failed, bad rect? - %s' % str(rect), 2)
-                    _debug_('updating whole screen')
+                    _debug_('osd.update(rect=%r) failed, bad rect?' % (rect,), DERROR)
                     pygame.display.flip()
             else:
+                self.screen.blit(self.main_layer, (0,0))
+                if self.dialog_layer_enabled:
+                    self.screen.blit(self.dialog_layer, (0,0))
                 pygame.display.flip()
 
             if stop_busyicon:
@@ -1416,10 +1527,9 @@ class OSD:
 
     def _helpscreen(self):
         if not pygame.display.get_init():
-            return None
+            return
 
         self._help = {0:1, 1:0}[self._help]
-
         if self._help:
             _debug_('Help on')
             # Save current display
@@ -1444,7 +1554,6 @@ class OSD:
                 if row >= 15:
                     row = 0
                     col += 1
-
             self.update()
         else:
             _debug_('Help off')
@@ -1452,10 +1561,10 @@ class OSD:
             self.update()
 
 
-    # Convert a 32-bit TRGB color to a 4 element tuple for SDL
     def _sdlcol(self, col):
+        """ Convert a 32-bit TRGB color to a 4 element tuple for SDL """
         if col==None:
-            return (0,0,0,255)
+            return (0, 0, 0, 255)
         a = 255 - ((col >> 24) & 0xff)
         r = (col >> 16) & 0xff
         g = (col >> 8) & 0xff
