@@ -2,11 +2,7 @@
 # -----------------------------------------------------------------------
 # Plug-in interface
 # -----------------------------------------------------------------------
-# $Id: plugin.py 10559 2008-03-22 11:42:02Z duncan $
-#
-# Notes: This file handles the Freevo plugin interface
-# Todo:
-#
+# $Id: plugin.py 11205 2008-12-01 02:42:54Z duncan $
 # -----------------------------------------------------------------------
 # Freevo - A Home Theater PC framework
 # Copyright (C) 2002 Krister Lagerstrom, et al.
@@ -28,6 +24,9 @@
 #
 # -----------------------------------------------------------------------
 
+"""
+Plug-in interface
+"""
 
 import os, sys
 import traceback
@@ -39,10 +38,6 @@ import rc
 
 DEBUG = 0
 
-if float(sys.version[0:3]) < 2.3:
-    True  = 1
-    False = 0
-
 #
 # Some basic plugins known to Freevo.
 #
@@ -78,8 +73,7 @@ class Plugin:
         of the global _().
         """
         try:
-            self._ = gettext.translation(application, os.environ['FREEVO_LOCALE'],
-                                         fallback=1).gettext
+            self._ = gettext.translation(application, os.environ['FREEVO_LOCALE'], fallback=1).gettext
         except:
             self._ = lambda m: m
 
@@ -450,9 +444,9 @@ def isevent(event):
     """
     plugin event parsing
     """
-    event = str(event)
-    if event[:12] == 'PLUGIN_EVENT':
-        return event[13:]
+    event = str(event).split('PLUGIN_EVENT ')
+    if len(event) == 2:
+        return event[1]
     else:
         return None
 
@@ -544,12 +538,12 @@ def __load_plugin__(name, type, level, args, number):
             if module:
                 object = module + '.%s' % name[name.rfind('.')+1:]
             else:
-                print 'can\'t locate plugin %s' % name
-                print 'start \'freevo plugins -l\' to get a list of plugins'
+                print 'can\'t locate plugin %r' % name
+                print 'start "freevo plugins -l" to get a list of plugins'
                 return
         else:
-            print 'can\'t locate plugin %s' % name
-            print 'start \'freevo plugins -l\' to get a list of plugins'
+            print 'can\'t locate plugin %r' % name
+            print 'start "freevo plugins -l" to get a list of plugins'
             return
 
     try:
