@@ -2,7 +2,7 @@
 # -----------------------------------------------------------------------
 # A plugin to obtain detailed weather forecast information
 # -----------------------------------------------------------------------
-# $Id: oneclick.py 10561 2008-03-22 21:47:34Z duncan $
+# $Id: oneclick.py 11195 2008-11-25 19:41:30Z duncan $
 #
 # Notes:
 #
@@ -49,7 +49,6 @@ import time
 import re
 
 # rdf modules
-from xml.dom.ext.reader import Sax2
 import urllib, urllib2
 
 #freevo modules
@@ -138,7 +137,8 @@ def wget(url):
     _debug_('wget(%s)' % (url), 2)
     txdata = None
     txheaders = {
-        'User-Agent' : 'Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.8.1.11) Gecko/20071127 Firefox/2.0.0.11'
+        'User-Agent' : 'Mozilla/5.0 (Windows; U; Windows NT 5.1; en-GB; rv:1.9.0.4) Gecko/2008102920 Firefox/3.0.4'
+
     }
     print 'getting url %r...' % (url)
     req = urllib2.Request(url, txdata, txheaders)
@@ -218,8 +218,8 @@ class PluginInterface(plugin.MainMenuPlugin):
         """
         """
         _debug_('PluginInterface.__init__()', 2)
-        if not config.USE_NETWORK:
-            self.reason = 'USE_NETWORK not enabled'
+        if not config.SYS_USE_NETWORK:
+            self.reason = 'SYS_USE_NETWORK not enabled'
             return
         if not config.ONECLICK_LOCATIONS:
             self.reason = 'ONECLICK_LOCATIONS not defined'
@@ -281,7 +281,7 @@ class WeatherItem(Item):
         self.temperature = None
         self.feeling = None
         self.current_conditions = None
-        self.icon = None
+        self.day_icon = None
         self.pressure = None
         self.pressure_change = None
         self.wind_speed = None
@@ -575,12 +575,12 @@ class WeatherItem(Item):
     def getCurrentCondition(self):
         """ gets the current conditions """
         _debug_('getCurrentCondition()', 2)
-        return "%s" % (self.current_conditions)
+        return "%s" % (_(self.current_conditions))
 
     def getIcon(self):
         """ gets the current conditions icon """
         _debug_('getIcon()', 2)
-        return "%s" % (self.icon)
+        return "%s" % (self.day_icon)
 
     def getPressure(self):
         _debug_('getPressure()', 2)
@@ -594,7 +594,7 @@ class WeatherItem(Item):
 
     def getPressureChange(self):
         _debug_('getPressureChange()', 2)
-        return "%s" % (self.pressure_change)
+        return "%s" % (_(self.pressure_change))
 
     def getWind(self):
         _debug_('getWind()', 2)
@@ -602,7 +602,7 @@ class WeatherItem(Item):
             return "%s" % (self.wind_speed)
         elif self.wind_direction == 'VAR':
             return "%s %s %s %s" % (_('Variable'), _('at a rate of'), self.wind_speed, self.unit_s)
-        return "%s %s %s %s" % (self.wind_direction, _('at a rate of'), self.wind_speed, self.unit_s)
+        return "%s %s %s %s" % (_(self.wind_direction), _('at a rate of'), self.wind_speed, self.unit_s)
 
     def getWindDir(self):
         _debug_('getWindDir()', 2)
@@ -622,11 +622,11 @@ class WeatherItem(Item):
 
     def getUvIndex(self):
         _debug_('getUvIndex()', 2)
-        return "%s" % (self.uv_index)
+        return "%s" % (_(self.uv_index))
 
     def getUvType(self):
         _debug_('getUvType()', 2)
-        return "%s" % (self.uv_type)
+        return "%s" % (_(self.uv_type))
 
     def getDewPoint(self):
         _debug_('getDewPoint()', 2)
@@ -638,7 +638,7 @@ class WeatherItem(Item):
 
     def getMoonPhase(self):
         _debug_('getMoonPhase()', 2)
-        return "%s" % (self.moon_phase)
+        return "%s" % (_(self.moon_phase))
 
     def getSunrise(self):
         _debug_('getSunrise()', 2)
@@ -703,7 +703,7 @@ class WeatherItem(Item):
         self.temperature = current.cc.tmp
         self.feeling = current.cc.flik
         self.current_conditions = current.cc.t
-        self.icon = current.cc.icon
+        self.day_icon = current.cc.icon
         self.pressure = current.cc.bar.r
         self.pressure_change = current.cc.bar.d
         self.wind_speed = current.cc.wind.s
@@ -717,7 +717,7 @@ class WeatherItem(Item):
         self.moon_phase = current.cc.moon.t
 
         self.description = '%s %s %s' % (self.current_conditions, _("at"), self.getTemperature())
-        self.image = self.getDayImage(self.icon)
+        self.image = self.getDayImage(self.day_icon)
 
         # skip today in the days
         for day in forecast.dayf.days[1:]:
@@ -836,7 +836,7 @@ class WeatherDetailHandler:
         self.subtitle = self.getSubtitle(self.skin_num)
 
         # Fire up splashscreen and load the plugins
-        skin.draw('oneclick', self)
+        skin.draw('oneclick', self, blend=config.FREEVO_USE_ALPHABLENDING)
 
     def prevSkin(self):
         """decrements the skin number round to the last skin"""
@@ -874,17 +874,17 @@ class WeatherDetailHandler:
         elif event == 'MENU_SELECT':
             self.weather.getForecast(force=1)
             skin.clear()
-            skin.draw('oneclick', self)
+            skin.draw('oneclick', self, blend=config.FREEVO_USE_ALPHABLENDING)
             return True
 
         elif event in ('MENU_DOWN', 'MENU_RIGHT'):
             self.nextSkin()
-            skin.draw('oneclick', self)
+            skin.draw('oneclick', self, blend=config.FREEVO_USE_ALPHABLENDING)
             return True
 
         elif event in ('MENU_UP', 'MENU_LEFT'):
             self.prevSkin()
-            skin.draw('oneclick', self)
+            skin.draw('oneclick', self, blend=config.FREEVO_USE_ALPHABLENDING)
             return True
 
         return False
@@ -916,8 +916,7 @@ class WeatherBaseScreen(skin.Area):
         self.xscale = lambda x: int(x * self.xmult)
         self.yscale = lambda y: int(y * self.ymult)
 
-        self.update_functions = (self.update_day, self.update_forecast,
-                                 self.update_week, self.update_map)
+        self.update_functions = (self.update_day, self.update_forecast, self.update_week, self.update_map)
 
     def day_item(self, x1, text, x2, value, y, align='left'):
         """ This is helper function for update_day
@@ -949,7 +948,6 @@ class WeatherBaseScreen(skin.Area):
         y += y_inc
         self.day_item(x_col1, _('UV Index'), x_col2, self.parent.weather.getUvType(), y)
 
-
         # draw current condition image
         x_start = self.content.x + self.xscale(480)
         y_start = self.content.y + self.yscale(40)
@@ -980,18 +978,18 @@ class WeatherBaseScreen(skin.Area):
 
         lines = []
         try:
-            lines.append('%s %s' % (_('As of:'), weather.getLastUpdated()))
-            lines.append('%s %s' % (_('at'), weather.getObservationStation()))
+            lines.append('%s: %s' % (_('As of'), weather.getLastUpdated()))
+            lines.append('%s: %s' % (_('at'), weather.getObservationStation()))
             lines.append('  %s' % (weather.getCurrentCondition()))
-            lines.append('  %s %s' % (_('Temperature:'), weather.getTemperature()))
-            lines.append('  %s %s' % (_('Dew Point:'), weather.getDewPoint()))
-            lines.append('  %s %s' % (_('Humidity:'), weather.getHumidity()))
-            lines.append('  %s %s' % (_('Visibility:'), weather.getVisibility()))
-            lines.append('  %s %s' % (_('Pressure:'), weather.getPressure()))
-            lines.append('  %s %s' % (_('Winds:'), weather.getWind()))
-            lines.append('%s' % (_('Tonight:')))
-            lines.append('  %s %s' % (_('Sunset:'), weather.getSunset()))
-            lines.append('  %s %s' % (_('Moon Phase:'), weather.getMoonPhase()))
+            lines.append('  %s: %s' % (_('Temperature'), weather.getTemperature()))
+            lines.append('  %s: %s' % (_('Dew Point'), weather.getDewPoint()))
+            lines.append('  %s: %s' % (_('Humidity'), weather.getHumidity()))
+            lines.append('  %s: %s' % (_('Visibility'), weather.getVisibility()))
+            lines.append('  %s: %s' % (_('Pressure'), weather.getPressure()))
+            lines.append('  %s: %s' % (_('Winds'), weather.getWind()))
+            lines.append('%s:' % (_('Tonight')))
+            lines.append('  %s: %s' % (_('Sunset'), weather.getSunset()))
+            lines.append('  %s: %s' % (_('Moon Phase'), weather.getMoonPhase()))
         except Exception, error:
             print error
             import traceback, sys
@@ -1021,7 +1019,7 @@ class WeatherBaseScreen(skin.Area):
 
     def update_week(self):
         """ update the weeks forecast
-        @todo: this can be improved
+        TODO this can be improved
         """
         _debug_('update_week()', 2)
 
@@ -1034,13 +1032,13 @@ class WeatherBaseScreen(skin.Area):
             x = x_start + self.xscale(pos)
             y = y_start
 
-            self.week_item(x, y, Unicode(self.parent.weather.date[day]), self.key_font, 160)
+            self.week_item(x, y, _(Unicode(self.parent.weather.date[day])), self.key_font, 160)
 
             self.draw_image(self.parent.weather.weatherIcon[day],
                 (x, y + self.yscale(50), self.xscale(160), self.yscale(120)))
 
             y = y_start + self.yscale(200)
-            self.week_item(x, y, Unicode(self.parent.weather.weatherType[day]), self.small_font, 160)
+            self.week_item(x, y, _(Unicode(self.parent.weather.weatherType[day])), self.small_font, 160)
 
             y = y_start + self.yscale(240)
             self.week_item(x, y, _("LO"), self.val_font)
