@@ -2,7 +2,7 @@
 # -----------------------------------------------------------------------
 # Shuttles VFD interface plug-in
 # -----------------------------------------------------------------------
-# $Id: vfd.py 10406 2008-02-21 20:47:49Z duncan $
+# $Id: vfd.py 11167 2008-11-16 17:51:46Z duncan $
 #
 # Notes:
 #    To activate, put the following line in local_conf.py:
@@ -272,22 +272,21 @@ class PluginInterface(plugin.DaemonPlugin):
     """
     Display context info on Shuttle's VFD (Versatile Front-panel Display)
 
-    @requires:
-        - pyusb: installed (http://pyusb.berlios.de/) with name patch as
-                the default module is called usb and conflicts with freevo's
-                usb plugin.
-        -  pyusb-0.X.X-name.patch contrib/patches
+    B{Requires} pyusb installed U{http://pyusb.berlios.de/} with name patch as
+    the default module is called usb and conflicts with freevo's usb plugin.
+
+    B{Requires} pyusb-0.X.X-name.patch contrib/patches
 
     To activate this plugin, just put the following line at the end of your
-    local_conf.py file:
+    local_conf.py file::
 
-    | plugin.activate('vfd')
+        plugin.activate('vfd')
     """
     __author__           = 'Duncan Webb'
     __author_email__     = 'duncan@freevo.org'
     __maintainer__       = __author__
     __maintainer_email__ = __author_email__
-    __version__          = '$Revision: 10406 $'
+    __version__          = '$Revision: 11167 $'
 
     def send(self, data):
         """
@@ -592,9 +591,10 @@ class PluginInterface(plugin.DaemonPlugin):
 
         # Check if audio is detached
         # When in detached mode, do not draw the player screen
-        if plugin.getbyname('audio.detachbar'):
-            if type == 'player' and plugin.getbyname('audio.detachbar').status != 0:
-                return
+        if type == 'player':
+            if plugin.getbyname('audio.detachbar'):
+                if plugin.getbyname('audio.detachbar').state != 1: #BAR_HIDE
+                    return
 
         if type == 'player':
             sname = "%s_%s" % (object.type, type)
