@@ -1,8 +1,8 @@
 # -*- coding: iso-8859-1 -*-
 # -----------------------------------------------------------------------
-# area.py - An area for the Freevo skin
+# An area for the Freevo skin
 # -----------------------------------------------------------------------
-# $Id: area.py 10562 2008-03-22 21:57:15Z duncan $
+# $Id: area.py 11000 2008-09-12 13:26:15Z duncan $
 #
 # Notes:
 #
@@ -53,8 +53,8 @@ import pygame
 import stat
 import types
 
-import osd
 import config
+import osd
 import util
 
 from skin_utils import *
@@ -62,6 +62,9 @@ from skin import eval_attr
 import xml_skin
 import screen
 
+from util.benchmark import benchmark
+benchmarking = config.DEBUG_BENCHMARKING
+
 SkinObjects = screen.SkinObjects
 
 
@@ -70,6 +73,7 @@ class Geometry:
     """
     Simple object with x, y, with, height values
     """
+    @benchmark(benchmarking)
     def __init__(self, x, y, width, height):
         self.x = x
         self.y = y
@@ -85,6 +89,7 @@ class Skin_Area:
     def update_content_needed
     def update_content
     """
+    @benchmark(benchmarking)
     def __init__(self, name, imagecachesize=5):
         self.area_name = name
         self.area_val  = None
@@ -94,14 +99,14 @@ class Skin_Area:
         self.screen    = screen.get_singleton()
         self.objects   = SkinObjects()
 
-        self.imagecache = util.objectcache.ObjectCache(imagecachesize,
-                                                       desc='%s_image' % self.name)
+        self.imagecache = util.objectcache.ObjectCache(imagecachesize, desc='%s_image' % self.name)
 
     def __str__(self):
         return 'area_name=%s area_val=%s redraw=%s layout=%s name=%s' % \
-            ( self.area_name, self.area_val, self.redraw, self.layout, self.name)
+            (self.area_name, self.area_val, self.redraw, self.layout, self.name)
 
 
+    @benchmark(benchmarking)
     def update_content_needed(self):
         """
         this area needs a content update
@@ -109,6 +114,7 @@ class Skin_Area:
         return True
 
 
+    @benchmark(benchmarking)
     def update_content(self):
         """
         there is no content in this area
@@ -116,13 +122,16 @@ class Skin_Area:
         pass
 
 
-    def draw(self, settings, obj, menu, display_style=0, widget_type='menu',
-             force_redraw=False):
+    @benchmark(benchmarking)
+    def draw(self, settings, obj, menu, display_style=0, widget_type='menu', force_redraw=False):
         """
         this is the main draw function. This function draws the background,
         checks if redraws are needed and calls the two update functions
         for the different types of areas
         """
+        _debug_('Skin_Area.draw(settings=%r, obj=%r, menu=%r, display_style=%r, widget_type=%r, force_redraw=%r)' % \
+            (settings, obj, menu, display_style, widget_type, force_redraw), 3)
+
         self.display_style = display_style
         self.xml_settings  = settings
         self.widget_type   = widget_type
@@ -174,15 +183,13 @@ class Skin_Area:
         # maybe we are NOW invisible
         if visible and not area.visible and old_area:
             self.screen.update('background', (old_area.x, old_area.y,
-                                              old_area.x + old_area.width,
-                                              old_area.y + old_area.height))
+                old_area.x + old_area.width, old_area.y + old_area.height))
             self.objects = SkinObjects()
 
         if not self.area_name == 'plugin':
             if not area.visible or not self.layout:
                 self.objects = SkinObjects()
                 return
-
             self.tmp_objects = SkinObjects()
             self.__draw_background__()
         else:
@@ -196,61 +203,53 @@ class Skin_Area:
         self.update_content()
 
         # check which parts need an update
-        bg_rect = ( osd.width, osd.height, 0, 0 )
-        a_rect  = ( osd.width, osd.height, 0, 0 )
-        c_rect  = ( osd.width, osd.height, 0, 0 )
+        bg_rect = (osd.width, osd.height, 0, 0)
+        a_rect  = (osd.width, osd.height, 0, 0)
+        c_rect  = (osd.width, osd.height, 0, 0)
 
         for b in self.tmp_objects.bgimages:
             try:
                 self.objects.bgimages.remove(b)
             except ValueError:
-                bg_rect = ( min(bg_rect[0], b[0]), min(bg_rect[1], b[1]),
-                            max(bg_rect[2], b[2]), max(bg_rect[3], b[3]) )
+                bg_rect = (min(bg_rect[0], b[0]), min(bg_rect[1], b[1]),
+                           max(bg_rect[2], b[2]), max(bg_rect[3], b[3]))
 
         for b in self.objects.bgimages:
-            bg_rect = ( min(bg_rect[0], b[0]), min(bg_rect[1], b[1]),
-                        max(bg_rect[2], b[2]), max(bg_rect[3], b[3]) )
-
-
+            bg_rect = (min(bg_rect[0], b[0]), min(bg_rect[1], b[1]),
+                       max(bg_rect[2], b[2]), max(bg_rect[3], b[3]))
 
         for b in self.tmp_objects.rectangles:
             try:
                 self.objects.rectangles.remove(b)
             except ValueError:
-                a_rect = ( min(a_rect[0], b[0]), min(a_rect[1], b[1]),
-                           max(a_rect[2], b[2]), max(a_rect[3], b[3]) )
+                a_rect = (min(a_rect[0], b[0]), min(a_rect[1], b[1]),
+                          max(a_rect[2], b[2]), max(a_rect[3], b[3]))
 
         for b in self.objects.rectangles:
-            a_rect = ( min(a_rect[0], b[0]), min(a_rect[1], b[1]),
-                       max(a_rect[2], b[2]), max(a_rect[3], b[3]) )
-
-
+            a_rect = (min(a_rect[0], b[0]), min(a_rect[1], b[1]),
+                      max(a_rect[2], b[2]), max(a_rect[3], b[3]))
 
         for b in self.tmp_objects.images:
             try:
                 self.objects.images.remove(b)
             except ValueError:
-                c_rect = ( min(c_rect[0], b[0]), min(c_rect[1], b[1]),
-                           max(c_rect[2], b[2]), max(c_rect[3], b[3]) )
+                c_rect = (min(c_rect[0], b[0]), min(c_rect[1], b[1]),
+                          max(c_rect[2], b[2]), max(c_rect[3], b[3]))
 
         for b in self.objects.images:
-            c_rect = ( min(c_rect[0], b[0]), min(c_rect[1], b[1]),
-                       max(c_rect[2], b[2]), max(c_rect[3], b[3]) )
-
-
+            c_rect = (min(c_rect[0], b[0]), min(c_rect[1], b[1]),
+                      max(c_rect[2], b[2]), max(c_rect[3], b[3]))
 
         for b in self.tmp_objects.text:
             try:
                 self.objects.text.remove(b)
             except ValueError:
-                c_rect = ( min(c_rect[0], b[0]), min(c_rect[1], b[1]),
-                           max(c_rect[2], b[2]), max(c_rect[3], b[3]) )
+                c_rect = (min(c_rect[0], b[0]), min(c_rect[1], b[1]),
+                          max(c_rect[2], b[2]), max(c_rect[3], b[3]))
 
         for b in self.objects.text:
-            c_rect = ( min(c_rect[0], b[0]), min(c_rect[1], b[1]),
-                       max(c_rect[2], b[2]), max(c_rect[3], b[3]) )
-
-
+            c_rect = (min(c_rect[0], b[0]), min(c_rect[1], b[1]),
+                      max(c_rect[2], b[2]), max(c_rect[3], b[3]))
 
         # send the update information to the screen object
         if bg_rect[0] < bg_rect[2]:
@@ -269,17 +268,15 @@ class Skin_Area:
         if config.DEBUG_SKIN:
             if self.layout:
                 name = self.area_name + u':' + self.layout.label
-
                 content = self.calc_geometry(self.layout.content, copy_object=1)
-                self.tmp_objects.skin_area = (content.x, content.y,
-                        content.width, content.height,
-                        name)
+                self.tmp_objects.skin_area = (content.x, content.y, content.width, content.height, name)
 
         # save and exit
         self.objects = self.tmp_objects
         self.screen.draw(self.objects)
 
 
+    @benchmark(benchmarking)
     def scan_for_text_view(self, menu):
         """
         scan if we have to fall back to text view. This will be done if some
@@ -365,6 +362,7 @@ class Skin_Area:
         self.use_text_view        = True
 
 
+    @benchmark(benchmarking)
     def calc_geometry(self, object, copy_object=0):
         """
         calculate the real values of the object (e.g. content) based
@@ -399,6 +397,7 @@ class Skin_Area:
         return object
 
 
+    @benchmark(benchmarking)
     def get_item_rectangle(self, rectangle, item_w, item_h):
         """
         calculates the values for a rectangle to fit item_w and item_h
@@ -439,6 +438,7 @@ class Skin_Area:
         return max(item_w, r.width), max(item_h, r.height), r
 
 
+    @benchmark(benchmarking)
     def init_vars(self, settings, display_type, widget_type = 'menu'):
         """
         check which layout is used and set variables for the object
@@ -465,7 +465,7 @@ class Skin_Area:
                 try:
                     area = area.style[0]
                 except IndexError:
-                    print 'index error for %s %s' % (self.display_style, widget_type)
+                    _debug_('index error for %s %s' % (self.display_style, widget_type), DWARNING)
                     raise
 
             if area[0] and (not self.use_text_view):
@@ -473,7 +473,7 @@ class Skin_Area:
             elif area[1]:
                 area = area[1]
             else:
-                print 'want to fall back, but no text view defined'
+                _debug_('want to fall back, but no text view defined', DWARNING)
                 area = area[0]
 
         else:
@@ -498,7 +498,7 @@ class Skin_Area:
                 try:
                     area = area.areas[self.area_name]
                 except (KeyError, AttributeError):
-                    print 'no skin information for %s:%s' % (widget_type, self.area_name)
+                    _debug_('no skin information for %s:%s' % (widget_type, self.area_name), DWARNING)
                     area = xml_skin.Area(self.area_name)
                     area.visible = False
 
@@ -520,6 +520,7 @@ class Skin_Area:
         return redraw
 
 
+    @benchmark(benchmarking)
     def __draw_background__(self):
         """
         draw the <background> of the area
@@ -571,18 +572,15 @@ class Skin_Area:
                     cname = '%s-%s-%s' % (imagefile, bg.width, bg.height)
                     image = self.imagecache[cname]
                     if not image:
-                        cache = vfs.getoverlay('%s.raw-%sx%s' % (imagefile, bg.width,
-                                                                 bg.height))
-                        if os.path.isfile(cache) and \
-                               os.stat(cache)[stat.ST_MTIME] > \
-                               os.stat(imagefile)[stat.ST_MTIME]:
-                            image = pygame.image.fromstring(file(cache).read(), (bg.width,bg.height), 'RGBA')
+                        cache = vfs.getoverlay('%s.raw-%sx%s' % (imagefile, bg.width, bg.height))
+                        if os.path.isfile(cache) and os.stat(cache)[stat.ST_MTIME] > os.stat(imagefile)[stat.ST_MTIME]:
+                            image = pygame.image.fromstring(file(cache).read(), (bg.width, bg.height), 'RGBA')
                             self.imagecache[cname] = image
 
                     if not image:
                         image = osd.loadbitmap(imagefile)
                         if image:
-                            image = pygamesurface_imlib2_scale(image, (bg.width,bg.height))
+                            image = pygamesurface_imlib2_scale(image, (bg.width, bg.height))
                             f = vfs.open(cache, 'w')
                             f.write(pygame.image.tostring(image, 'RGBA'))
                             f.close()
@@ -603,42 +601,48 @@ class Skin_Area:
     # drawimage
     # drawstring
 
+    @benchmark(benchmarking)
     def drawroundbox(self, x, y, width, height, rect, redraw=True):
         """
         draw a round box ... or better stores the information about this call
         in a variable. The real drawing is done inside draw()
+
+        @param x: x position
+        @param y: y position
+        @param width: width of box
+        @param height: height of box
+        @param rect: a rectangle
+        @param redraw: redraw
         """
         try:
-            self.tmp_objects.rectangles.append(( x, y, x + width, y + height, rect.bgcolor,
-                                                 rect.size, rect.color, rect.radius ))
-        except AttributeError:
-            self.tmp_objects.rectangles.append(( x, y, x + width, y + height, rect[0],
-                                                 rect[1], rect[2], rect[3] ))
-
+            self.tmp_objects.rectangles.append((x, y, x + width, y + height, rect.bgcolor, rect.size, rect.color,
+                rect.radius))
+        except AttributeError, why:
+            self.tmp_objects.rectangles.append((x, y, x + width, y + height, rect[0], rect[1], rect[2], rect[3]))
 
 
+    @benchmark(benchmarking)
     def drawstring(self, text, font, content, x=-1, y=-1, width=None, height=None,
-                   align_h = None, align_v = None, mode='hard', ellipses='...',
-                   dim=True):
+                   align_h=None, align_v=None, mode='hard', ellipses='...', dim=True):
         """
         writes a text ... or better stores the information about this call
         in a variable. The real drawing is done inside draw()
         """
 
         if not text:
-            return (0,0,0,0)
+            return (0, 0, 0, 0)
 
         # set default values from 'content'
         if x == -1:
             if hasattr(content, 'x'):
                 x = content.x
             else:
-                print 'No content.x for %s' % (self.name)
+                _debug_('No content.x for %s' % (self.name), DWARNING)
         if y == -1:
             if hasattr(content, 'y'):
                 y = content.y
             else:
-                print 'No content.y for %s' % (self.name)
+                _debug_('No content.y for %s' % (self.name), DWARNING)
 
         if width == None:
             width  = content.width
@@ -660,9 +664,10 @@ class Skin_Area:
             height2 = font.h + 2
 
         self.tmp_objects.text.append((x, y, x+width, y+height2, text, font, height,
-                                            align_h, align_v, mode, ellipses, dim))
+            align_h, align_v, mode, ellipses, dim))
 
 
+    @benchmark(benchmarking)
     def loadimage(self, image, val, redraw=True):
         """
         load an image (use self.imagecache)
@@ -684,7 +689,7 @@ class Skin_Area:
         if not cimage:
             cimage = osd.loadbitmap(image)
             if not cimage:
-                return
+                return None, 0, 0
             if w == -1:
                 w = h  * cimage.get_width() / cimage.get_height()
             if h == -1:
@@ -694,46 +699,43 @@ class Skin_Area:
             if w > 0 and h > 0:
                 cimage = pygamesurface_imlib2_scale(cimage, (w, h))
             self.imagecache[cname] = cimage
-        return cimage
+        return cimage, w, h
 
 
+    @benchmark(benchmarking)
     def drawimage(self, image, val, background=False):
         """
         draws an image ... or better stores the information about this call
         in a variable. The real drawing is done inside draw()
         """
-
         if not image:
-            return 0,0
+            return 0, 0
 
         if isstring(image):
             if isinstance(val, tuple):
-                image = self.loadimage(String(image), val[2:])
+                image, w, h = self.loadimage(String(image), val[2:])
             else:
-                image = self.loadimage(String(image), val)
+                image, w, h = self.loadimage(String(image), val)
 
         if not image:
-            return 0,0
+            return 0, 0
 
         if isinstance(val, tuple):
             if background:
                 o = self.tmp_objects.bgimages
             else:
                 o = self.tmp_objects.images
-            o.append((val[0], val[1], val[0] + image.get_width(),
-                      val[1] + image.get_height(), image))
+            o.append((val[0], val[1], val[0] + image.get_width(), val[1] + image.get_height(), image))
             return image.get_width(), image.get_height()
 
         try:
             if background or val.label == 'background':
-                self.tmp_objects.bgimages.append((val.x, val.y, val.x + val.width,
-                                                  val.y + val.height, image))
+                self.tmp_objects.bgimages.append((val.x, val.y, val.x + val.width, val.y + val.height, image))
                 return val.width, val.height
         except:
             pass
 
-        self.tmp_objects.images.append((val.x, val.y, val.x + val.width,
-                                        val.y + val.height, image))
+        self.tmp_objects.images.append((val.x, val.y, val.x + val.width, val.y + val.height, image))
         return val.width, val.height
 
 
