@@ -2,7 +2,7 @@
 # -----------------------------------------------------------------------
 # This is the Freevo TV Guide module.
 # -----------------------------------------------------------------------
-# $Id: tvguide.py 10560 2008-03-22 21:37:30Z duncan $
+# $Id: tvguide.py 11000 2008-09-12 13:26:15Z duncan $
 #
 # Notes:
 # Todo:
@@ -42,7 +42,7 @@ from item import Item
 from programitem import ProgramItem
 
 from util.benchmark import benchmark
-benchmarking = False
+benchmarking = config.DEBUG_BENCHMARKING
 
 import tv.epg_xmltv
 from tv.epg_types import TvProgram
@@ -251,6 +251,7 @@ class TVGuide(Item):
         ## PLAY_END: Show the guide again
         elif event == PLAY_END:
             self.show()
+            self.jump_to_now(self.selected)
             return True
 
         # FIX or REMOVE:
@@ -356,7 +357,9 @@ class TVGuide(Item):
         jump to now in the tv guide.
         """
         _debug_('jump_to_now(old_selected=%r)' % (old_selected,), 2)
-        start_time = time.time()
+        (year, mon, mday, hour, min, sec, wday, yday, isdst) = time.localtime()
+        min = min > 30 and 30 or 0
+        start_time = time.mktime((year, mon, mday, hour, min, sec, wday, yday, isdst))
         stop_time = start_time + self.hours_per_page * 60 * 60
         start_channel = self.start_channel
 
