@@ -2,7 +2,7 @@
 # -----------------------------------------------------------------------
 # V4L2 python interface.
 # -----------------------------------------------------------------------
-# $Id: v4l2.py 10559 2008-03-22 11:42:02Z duncan $
+# $Id: v4l2.py 11054 2008-09-20 19:32:45Z duncan $
 #
 # Notes: http://bytesex.org/v4l/spec/
 # Todo:
@@ -30,7 +30,6 @@
 
 
 import string
-import freq
 import os
 import struct
 import array
@@ -38,6 +37,7 @@ import fcntl
 import sys
 
 import config
+import freq
 
 
 # Different formats depending on word size
@@ -250,24 +250,24 @@ class Videodev:
     def __init__(self, device):
         _debug_('Videodev.__init__(device=%r)' % (device,), 2)
         self.chanlist = None
-        self.device = os.open(device, os.O_TRUNC)
-        if self.device < 0:
-            sys.exit('Error: %d' % self.device)
-        else:
-            _debug_('Video opened for %r' % device)
+        self.device = -1
+        try:
+            self.device = os.open(device, os.O_TRUNC)
+        except OSError, why:
+            #self.device = os.open(device, os.O_RDONLY)
+            _debug_('Cannot open video device %r: %s' % (device, why), DERROR)
+            return
 
+        _debug_('Video opened for %r' % device)
         results           = self.querycap()
-        self.driver       = results[0]
-        self.card         = results[1]
-        self.bus_info     = results[2]
+        self.driver       = results[0][:results[0].find('\0')]
+        self.card         = results[1][:results[1].find('\0')]
+        self.bus_info     = results[2][:results[2].find('\0')]
         self.version      = results[3]
         self.capabilities = results[4]
         self.inputs       = self.enuminputs()
         self.standards    = self.enumstds()
         self.controls     = self.enumcontrols()
-        #print 'inputs=%r' % self.inputs
-        #print 'standards=%r' % self.standards
-        #print 'controls=%r' % self.controls
 
 
     def getdriver(self):
@@ -440,6 +440,9 @@ class Videodev:
         val = struct.pack(ENUMSTD_ST, num, 0, "", 0, 0, 0)
         r = fcntl.ioctl(self.device, i32(ENUMSTD_NO), val)
         res = struct.unpack(ENUMSTD_ST, r)
+        (index, id, name, numerator, denominator, framelines) = res
+        _debug_('enumstd: index=%r, id=0x%08x, name=%r, numerator=%r, denominator=%r, framelines=%r' % \
+            (index, id, name.strip('\0'), numerator, denominator, framelines), 2)
         _debug_('enumstd: val=%r, %d, res=%r' % (val, len(val), res), 3)
         return res
 
@@ -763,7 +766,7 @@ class Videodev:
         key = self.ctrlname(name)
         if not self.controls.has_key(key):
             _debug_('control \"%s\" does not exists' % (name), DWARNING)
-            return None
+            return 0
         (id, type, name, min, max, step, default, flags, value) = self.controls[key]
         return value
 
@@ -804,9 +807,10 @@ class Videodev:
 
 
     def init_settings(self):
-        """ initialise the V4L2 setting
-        """
+        """ initialise the V4L2 setting """
         _debug_('init_settings()', 2)
+        if self.device < 0:
+            return False
         (v_norm, v_input, v_clist, v_dev) = config.TV_SETTINGS.split()
         self.inputs = self.enuminputs()
         self.standards = self.enumstds()
@@ -814,15 +818,16 @@ class Videodev:
         self.setstdbyname(v_norm)
         self.setchanlist(v_clist)
 
-        # XXX TODO: make a good way of setting the input
-        # self.setinput(....)
+        # TODO: make a good way of setting the input
+        # self.setinput(...)
+        return True
 
 
     def print_settings(self):
         _debug_('print_settings()', 2)
         print 'Driver: %s' % self.driver.strip('\0')
         print 'Card: %s' % self.card.strip('\0')
-        print 'Version: %02d.%02d' % (self.version / 256, self.version % 256)
+        print 'Version: %08x (%02u.%02u)' % (self.version, self.version / 256, self.version % 256)
         print 'Capabilities: 0x%08x' % int(self.capabilities)
 
         print "Enumerating supported Standards."
