@@ -1,12 +1,8 @@
 # -*- coding: iso-8859-1 -*-
 # -----------------------------------------------------------------------
-# util/fileops.py - Some File Operation Utilities
+# Some File Operation Utilities
 # -----------------------------------------------------------------------
-# $Id: fileops.py 10252 2008-01-01 11:47:50Z duncan $
-#
-# Notes:
-# Todo:
-#
+# $Id: fileops.py 11190 2008-11-22 10:47:49Z duncan $
 # -----------------------------------------------------------------------
 # Freevo - A Home Theater PC framework
 # Copyright (C) 2002 Krister Lagerstrom, et al.
@@ -28,13 +24,18 @@
 #
 # -----------------------------------------------------------------------
 
+"""
+Some File Operation Utilities
+"""
 
 import os
 import sys
-import stat
-import statvfs
+from stat import *
+from statvfs import *
 import string
 import copy
+import subprocess
+from subprocess import PIPE
 try:
     import cPickle as pickle
 except ImportError:
@@ -43,9 +44,10 @@ import fnmatch
 import traceback
 
 # image stuff
-import kaa.imlib2 as imlib2
+import kaa.imlib2
 from kaa.metadata.image import EXIF as exif
 
+from util.benchmark import benchmark
 
 if float(sys.version[0:3]) < 2.3:
     PICKLE_PROTOCOL = 1
@@ -59,6 +61,8 @@ if traceback.extract_stack()[0][0].find('install.py') == -1:
     # import stuff from util.misc
     import misc
 
+    benchmarking = config.DEBUG_BENCHMARKING
+    benchmarkcall = config.DEBUG_BENCHMARKCALL
 
 #
 #
@@ -66,6 +70,7 @@ if traceback.extract_stack()[0][0].find('install.py') == -1:
 #
 #
 
+@benchmark(benchmarking, benchmarkcall)
 def getdirnames(dirname, softlinks=True, sort=True):
     """
     Get all subdirectories in the given directory.
@@ -86,6 +91,7 @@ def getdirnames(dirname, softlinks=True, sort=True):
 
 
 
+@benchmark(benchmarking, benchmarkcall)
 def gzopen(file):
     """
     open a gzip file and return the fd. If it's not a gzip file, try
@@ -102,6 +108,7 @@ def gzopen(file):
     return f
 
 
+@benchmark(benchmarking, benchmarkcall)
 def readfile(filename):
     """
     return the complete file as list
@@ -112,6 +119,7 @@ def readfile(filename):
     return ret
 
 
+#@benchmark(benchmarking, benchmarkcall)
 def freespace(path):
     """
     freespace(path) -> integer
@@ -120,12 +128,13 @@ def freespace(path):
     """
     try:
         s = os.statvfs(path)
-        return s[statvfs.F_BAVAIL] * long(s[statvfs.F_BSIZE])
-    except OSError, e:
-        print '\"%s\": %s' % (path, e)
+        return s[F_BAVAIL] * long(s[F_BSIZE])
+    except OSError, why:
+        _debug_('"%r: %s' % (path, why), DWARNING)
     return 0
 
 
+@benchmark(benchmarking, benchmarkcall)
 def totalspace(path):
     """
     totalspace(path) -> integer
@@ -134,12 +143,13 @@ def totalspace(path):
     """
     try:
         s = os.statvfs(path)
-        return s[statvfs.F_BLOCKS] * long(s[statvfs.F_BSIZE])
-    except OSError, e:
-        print '\"%s\": %s' % (path, e)
+        return s[F_BLOCKS] * long(s[F_BSIZE])
+    except OSError, why:
+        _debug_('"%r: %s' % (path, why), DWARNING)
     return 0
 
 
+@benchmark(benchmarking, benchmarkcall)
 def touch(file):
     """
     touch a file (maybe create it)
@@ -152,6 +162,7 @@ def touch(file):
     return 0
 
 
+@benchmark(benchmarking, benchmarkcall)
 def rmrf_helper(result, dirname, names):
     """
     help function for rm -rf
@@ -164,6 +175,7 @@ def rmrf_helper(result, dirname, names):
     return result
 
 
+@benchmark(benchmarking, benchmarkcall)
 def rmrf(top=None):
     """
     Pure python version of 'rm -rf'
@@ -179,8 +191,8 @@ def rmrf(top=None):
         for d in files[1]:
             try:
                 os.rmdir(d)
-            except (IOError, OSError), e:
-                print 'rmrf:', e
+            except (IOError, OSError), why:
+                _debug_('rmrf: %s' % why, DWARNING)
 
 
 #
@@ -189,6 +201,7 @@ def rmrf(top=None):
 #
 #
 
+@benchmark(benchmarking, benchmarkcall)
 def match_suffix(filename, suffixlist):
     """
     Check if a filename ends in a given suffix, case is ignored.
@@ -203,6 +216,7 @@ def match_suffix(filename, suffixlist):
     return 0
 
 
+@benchmark(benchmarking, benchmarkcall)
 def match_files(dirname, suffix_list):
     """
     Find all files in a directory that has matches a list of suffixes.
@@ -210,10 +224,9 @@ def match_files(dirname, suffix_list):
     """
 
     try:
-        files = [ vfs.join(dirname, fname) for fname in os.listdir(dirname) if
-                  vfs.isfile(vfs.join(dirname, fname)) ]
-    except OSError:
-        print 'util:match_files(): Got error on dir = "%s"' % dirname
+        files = [ vfs.join(dirname, fname) for fname in os.listdir(dirname) if vfs.isfile(vfs.join(dirname, fname)) ]
+    except OSError, why:
+        _debug_('Cannot match files in "%s": %s' % (dirname, why), DWARNING)
         return []
 
     matches = [ fname for fname in files if match_suffix(fname, suffix_list) ]
@@ -223,6 +236,7 @@ def match_files(dirname, suffix_list):
     return matches
 
 
+@benchmark(benchmarking, benchmarkcall)
 def find_matches(files, suffix_list):
     """
     return all files in 'files' that match one of the suffixes in 'suffix_list'
@@ -234,6 +248,7 @@ def find_matches(files, suffix_list):
     return filter(lambda x: x[x.rfind('.')+1:].lower() in suffix_list, files)
 
 
+@benchmark(benchmarking, benchmarkcall)
 def match_files_recursively_helper(result, dirname, names):
     """
     help function for match_files_recursively
@@ -251,6 +266,7 @@ def match_files_recursively_helper(result, dirname, names):
     return result
 
 
+@benchmark(benchmarking, benchmarkcall)
 def match_files_recursively_skip_protected(result, dirname, names):
     """
     help function for match_files_recursively_skip_protected, skipping directories with a .password file
@@ -269,6 +285,7 @@ def match_files_recursively_skip_protected(result, dirname, names):
     return result
 
 
+@benchmark(benchmarking, benchmarkcall)
 def match_files_recursively(dir, suffix_list, skip_password=False):
     """
     get all files matching suffix_list in the dir and in it's subdirectories
@@ -286,6 +303,7 @@ def match_files_recursively(dir, suffix_list, skip_password=False):
     return matches
 
 
+@benchmark(benchmarking, benchmarkcall)
 def get_subdirs_recursively(dir):
     """
     get all subdirectories recursively in the given directory
@@ -300,6 +318,7 @@ def get_subdirs_recursively(dir):
     return matches
 
 
+@benchmark(benchmarking, benchmarkcall)
 def recursefolders(root, recurse=0, pattern='*', return_folders=0):
     """
     Before anyone asks why I didn't use os.path.walk; it's simple,
@@ -308,7 +327,7 @@ def recursefolders(root, recurse=0, pattern='*', return_folders=0):
 
     Here's how you use this function::
 
-        songs = recursefolders('/media/Music/Guttermouth',1,'*.mp3',1)
+        songs = recursefolders('/media/Music/Guttermouth', 1, '*.mp3', 1)
         for song in songs:
               print song
 
@@ -326,7 +345,7 @@ def recursefolders(root, recurse=0, pattern='*', return_folders=0):
 
     # expand pattern
     pattern = pattern or '*'
-    pat_list = string.splitfields( pattern , ';' )
+    pat_list = string.splitfields(pattern, ';')
 
     # check each file
     for name in names:
@@ -335,16 +354,14 @@ def recursefolders(root, recurse=0, pattern='*', return_folders=0):
         # grab if it matches our pattern and entry type
         for pat in pat_list:
             if fnmatch.fnmatch(name, pat):
-                if vfs.isfile(fullname) or \
-                   (return_folders and vfs.isdir(fullname)):
+                if vfs.isfile(fullname) or (return_folders and vfs.isdir(fullname)):
                     result.append(fullname)
                 continue
 
         # recursively scan other folders, appending results
         if recurse:
             if vfs.isdir(fullname) and not vfs.islink(fullname):
-                result = result + recursefolders( fullname, recurse,
-                                                  pattern, return_folders )
+                result += recursefolders(fullname, recurse, pattern, return_folders)
 
     return result
 
@@ -357,55 +374,89 @@ def recursefolders(root, recurse=0, pattern='*', return_folders=0):
 
 mounted_dirs = []
 
-def umount(dir):
+@benchmark(benchmarking, benchmarkcall)
+def is_mounted(dir):
     """
-    umount a directory
+    return if the dir is mounted
     """
-    if not dir:
-        return
+    _debug_('is_mounted(dir=%r)' % (dir,), 2)
     global mounted_dirs
-    if os.path.ismount(dir):
-        os.system("umount %s" % dir)
-        if not os.path.ismount(dir) and dir in mounted_dirs:
-            mounted_dirs.remove(dir)
+    return dir in mounted_dirs
 
 
-def mount(dir, force=0):
+mount_count = 0
+
+@benchmark(benchmarking, benchmarkcall)
+def mount(dir, force=False):
     """
     mount a directory
     """
+    global mount_count
+    if config.ROM_DRIVES_AUTOFS:
+        return
+    _debug_('mount(dir=%r, force=%r)' % (dir, force), 2)
     if not dir:
         return
     global mounted_dirs
     if not os.path.ismount(dir):
-        os.system("mount %s 2>/dev/null" % dir)
+        p = subprocess.Popen(['mount', dir], stdout=PIPE, stderr=PIPE)
+        rc = p.wait()
+        so, se = p.communicate()
+        if rc:
+            if rc in (1,):
+                _debug_('mounting %r: %s' % (dir, se), DWARNING)
+            elif rc == (32,):
+                _debug_('mounting %r: %s' % (dir, se), DWARNING)
+        else:
+            mount_count += 1
         if os.path.ismount(dir) and not dir in mounted_dirs:
             mounted_dirs.append(dir)
     if force and not dir in mounted_dirs:
         mounted_dirs.append(dir)
 
 
+@benchmark(benchmarking, benchmarkcall)
+def umount(dir):
+    """
+    umount a directory
+    """
+    global mount_count
+    if config.ROM_DRIVES_AUTOFS:
+        return
+    _debug_('umount(dir=%r)' % (dir,), 2)
+    if not dir:
+        return
+    global mounted_dirs
+    if os.path.ismount(dir):
+        p = subprocess.Popen(['umount', dir], stdout=PIPE, stderr=PIPE)
+        rc = p.wait()
+        so, se = p.communicate()
+        if rc:
+            if rc in (1,):
+                _debug_('unmounting %r: %s' % (dir, se), DWARNING)
+        else:
+            mount_count -= 1
+        if not os.path.ismount(dir) and dir in mounted_dirs:
+            mounted_dirs.remove(dir)
+
+
+@benchmark(benchmarking, benchmarkcall)
 def umount_all():
     """
     umount all mounted directories
     """
+    _debug_('umount_all()', 2)
     global mounted_dirs
     for d in copy.copy(mounted_dirs):
         umount(d)
 
 
-def is_mounted(dir):
-    """
-    return if the dir is mounted
-    """
-    global mounted_dirs
-    return dir in mounted_dirs
-
-
+@benchmark(benchmarking, benchmarkcall)
 def resolve_media_mountdir(*arg):
     """
     get the mount point of the media with media_id
     """
+    _debug_('resolve_media_mountdir(arg=%r)' % (arg,), 2)
     if len(arg) == 1 and isinstance(arg[0], dict):
         media_id = arg[0]['media_id']
         file     = arg[0]['file']
@@ -427,10 +478,12 @@ def resolve_media_mountdir(*arg):
     return mountdir, file
 
 
+@benchmark(benchmarking, benchmarkcall)
 def check_media(media_id):
     """
     check if media_id is a valid media in one of the drives
     """
+    _debug_('check_media(media_id=%r)' % (media_id,), 2)
     for media in config.REMOVABLE_MEDIA:
         if media_id == media.id:
             return media
@@ -444,6 +497,7 @@ def check_media(media_id):
 #
 #
 
+@benchmark(benchmarking, benchmarkcall)
 def read_pickle(file):
     """
     read a file with pickle
@@ -457,6 +511,7 @@ def read_pickle(file):
         return None
 
 
+@benchmark(benchmarking, benchmarkcall)
 def save_pickle(data, file):
     """
     save the data with pickle to the given file
@@ -468,13 +523,14 @@ def save_pickle(data, file):
         pickle.dump(data, f, PICKLE_PROTOCOL)
         f.close()
     except IOError:
-        print 'unable to save to cachefile %s' % file
+        _debug_('unable to save to cachefile %r' % (file,), DWARNING)
 
 
 #
 # cache utils
 #
 
+@benchmark(benchmarking, benchmarkcall)
 def read_thumbnail(filename):
     """
     return image cached inside filename
@@ -488,51 +544,41 @@ def read_thumbnail(filename):
     return data
 
 
+@benchmark(benchmarking, benchmarkcall)
 def create_thumbnail(filename, thumbnail=None):
     """
     cache image for faster access
     """
+    _debug_('create_thumbnail(filename=%r, thumbnail=%r)' % (filename, thumbnail), 2)
     thumb = vfs.getoverlay(filename + '.raw')
     image = None
 
     if thumbnail:
         try:
-            image = imlib2.open_from_memory(thumbnail)
-        except Exception, e:
-            print 'Invalid thumbnail for %s' % filename
-            if config.DEBUG:
-                print 'invalid thumbnail:', e
+            image = kaa.imlib2.open_from_memory(thumbnail)
+        except Exception, why:
+            _debug_('invalid thumbnail %r: %s', (filename, why), DINFO)
 
     if not image:
         if __freevo_app__ == 'main':
             try:
-                f=open(filename, 'rb')
-                tags=exif.process_file(f)
-                f.close()
-
+                tags = exif.process_file(open(filename, 'rb'))
                 if tags.has_key('JPEGThumbnail'):
-                    image = imlib2.open_from_memory(tags['JPEGThumbnail'])
-            except Exception, e:
-                print 'Error loading thumbnail %s' % filename
-                if config.DEBUG:
-                    print 'create_thumbnail:', e
+                    image = kaa.imlib2.open_from_memory(tags['JPEGThumbnail'])
+            except Exception, why:
+                _debug_('Error loading thumbnail %r: %s' % (filename, why), DWARNING)
 
         if not image or image.width < 100 or image.height < 100:
             try:
-                image = imlib2.open(filename)
-            except Exception, e:
-                print 'error caching image %s' % filename
-                if config.DEBUG:
-                    print 'caching image:', e
+                image = kaa.imlib2.open(filename)
+            except Exception, why:
+                _debug_('Cannot cache image %r: %s' % (filename, why), DWARNING)
                 return None
 
     try:
         if image.width > 255 or image.height > 255:
             image.thumbnail((255,255))
 
-        #print 'thumb:', thumb
-        #print 'image.mode:', image.mode
-
         if image.mode == 'P':
             image.mode = 'RGB'
         if image.mode == 'BGRA':
@@ -545,18 +591,19 @@ def create_thumbnail(filename, thumbnail=None):
         f.write(data[0])
         f.close()
         return data
-    except Exception, e:
-        print 'error caching image %s: %s' % (filename, e)
+    except Exception, why:
+        _debug_('Cannot cache image %r: %s' % (filename, why), DWARNING)
         return None
 
 
+@benchmark(benchmarking, benchmarkcall)
 def cache_image(filename, thumbnail=None, use_exif=False):
     """
     cache image for faster access, return cached image
     """
     thumb = vfs.getoverlay(filename + '.raw')
     try:
-        if os.stat(thumb)[stat.ST_MTIME] > os.stat(filename)[stat.ST_MTIME]:
+        if os.stat(thumb)[ST_MTIME] > os.stat(filename)[ST_MTIME]:
             data = read_thumbnail(thumb)
             if data:
                 return data
@@ -566,12 +613,13 @@ def cache_image(filename, thumbnail=None, use_exif=False):
     return create_thumbnail(filename, thumbnail)
 
 
+@benchmark(benchmarking, benchmarkcall)
 def scale_rectangle_to_max(size, max_size):
-    '''
+    """
     returns a scaled rectangle size fitting the size to max_size
     this can be used to scale images for viewing in a browser and
     retaining the aspect ratio
-    '''
+    """
     try:
         scaled_width = max_size[0]
         scaled_height = max_size[1]
@@ -585,35 +633,108 @@ def scale_rectangle_to_max(size, max_size):
     return (scaled_width, scaled_height)
 
 
-def www_thumbnail_path(filename):
-    '''returns the path to the thumbnail image for a given filename
-    '''
-    file_ext_index = filename.rindex(".")
-    file_ext = filename[file_ext_index:].lower()
-    if file_ext == ".gif":
-        file_ext = ".jpg"
+@benchmark(benchmarking, benchmarkcall)
+def www_image_path(filename, subdir='.thumbs'):
+    """
+    returns the path to the thumbnail image for a given filename
+    """
+    dirname = os.path.dirname(filename)
+    file_base, file_extn = os.path.splitext(filename)
+    file_extn = file_extn.lower()
+    if file_extn == ".gif":
+        file_extn = ".jpg"
     # the filename extension needs to be lowercase for imlib2 but we need to
     # keep the original filename for cache to be able to clean the files
-    imagepath = filename + file_ext
-    thumb_path = vfs.getwwwoverlay(imagepath)
-    return thumb_path
+    imagename = os.path.basename(file_base).lower() + file_extn
+    #imagepath = vfs.getwwwoverlay(imagename)
+    imagepath = os.path.join(dirname, subdir, imagename)
+    return imagepath
 
 
-def create_www_thumbnail(filename):
-    '''creates a webserver thumbnail image and returns its size.
-    '''
-    thumb_path = www_thumbnail_path(filename)
+@benchmark(benchmarking, benchmarkcall)
+def www_image(filename, subdir, force=False, getsize=False, size=128, verbose=0):
+    """
+    creates a webserver image image and returns its size.
+    """
+    orientation_map = {
+        #   vflip  hflip  orientate
+        1: (False, False, 0, 'Horizontal (normal)'),
+        2: (False, True,  0, 'Mirrored horizontal'),
+        3: (False, False, 2, 'Rotated 180'),
+        4: (True,  False, 0, 'Mirrored vertical'),
+        5: (False, True,  3, 'Mirrored horizontal then rotated 90 CCW'),
+        6: (False, False, 1, 'Rotated 90 CW'),
+        7: (False, True,  1, 'Mirrored horizontal then rotated 90 CW'),
+        8: (False, False, 3, 'Rotated 90 CCW'),
+    }
+
+    imagename = www_image_path(filename, subdir)
+    if verbose >= 2:
+        print 'Checking image %r, force=%r getsize=%r' % (imagename, force, getsize)
+    if os.path.exists(imagename):
+        if not force:
+            if getsize:
+                image = kaa.imlib2.open(imagename)
+                return image.size
+            return (0, 0)
+    else:
+        force = True
     try:
         try:
-            if not os.path.isdir(os.path.dirname(thumb_path)):
-                os.makedirs(os.path.dirname(thumb_path), mode=04775)
+            if not os.path.isdir(os.path.dirname(imagename)):
+                if verbose >= 1:
+                    print 'Make directory %r' % os.path.dirname(imagename)
+                os.makedirs(os.path.dirname(imagename), mode=04775)
         except IOError:
-            print 'error creating dir %s' % os.path.dirname(thumb_path)
+            _debug_('error creating dir %s' % os.path.dirname(imagename), DWARNING)
             raise IOError
-        image = imlib2.open(filename)
-        thumb = image.scale_preserve_aspect(config.WWW_IMAGE_THUMBNAIL_SIZE)
-        thumb.save(thumb_path)
-    except IOError, e:
-        print e
+        if force or os.stat(filename)[ST_MTIME] > os.stat(imagename)[ST_MTIME]:
+            orientation_str = ''
+            image = kaa.imlib2.open(filename)
+            try:
+                tags = exif.process_file(open(filename, 'rb'))
+                orientation = tags.get('Image Orientation').values[0]
+                flip_vert, flip_hori, orientate, orientate_str = orientation_map[orientation]
+                if orientation != 1:
+                    if flip_vert:
+                        image.flip_vertical()
+                        if verbose >= 2:
+                            print 'flipped vertically'
+                    if flip_hori:
+                        image.flip_horizontal()
+                        if verbose >= 2:
+                            print 'flipped horizontally'
+                    if orientate:
+                        if verbose >= 2:
+                            print orientate_str
+                        image.orientate(orientate)
+            except AttributeError, why:
+                pass
+            except Exception, why:
+                print why
+            if isinstance(size, tuple):
+                image = image.scale_preserve_aspect(size)
+            else:
+                image = image.scale_preserve_aspect((size, size))
+            image.save(imagename)
+            print 'Made image %r @ %s -> %r%s' % (imagename, size, image.size, orientation_str)
+    except IOError, why:
+        _debug_(why, DWARNING)
         return (0, 0)
     return image.size
+
+
+@benchmark(benchmarking, benchmarkcall)
+def www_thumb_create(filename, force=False, getsize=False, size=config.WWW_IMAGE_THUMBNAIL_SIZE, verbose=0):
+    """
+    creates a webserver image image and returns its size.
+    """
+    return www_image(filename, '.thumbs', force, getsize, size, verbose)
+
+
+@benchmark(benchmarking, benchmarkcall)
+def www_image_create(filename, force=False, getsize=False, size=config.WWW_IMAGE_SIZE, verbose=0):
+    """
+    creates a webserver image image and returns its size.
+    """
+    return www_image(filename, '.images', force, getsize, size, verbose)
