@@ -2,7 +2,7 @@
 # -----------------------------------------------------------------------
 # re-encode recorded TV programmes
 # -----------------------------------------------------------------------
-# $Id: reencode.py 10559 2008-03-22 11:42:02Z duncan $
+# $Id: reencode.py 11173 2008-11-17 21:34:29Z duncan $
 #
 # Notes:
 # ToDo:
@@ -29,12 +29,13 @@
 # -----------------------------------------------------------------------
 
 
+import os
 from os.path import join, split
+import config
 import plugin
 import menu
-import os
-import config
-from video.encodingclient import *
+import util
+from video.encodingclient import EncodingClientActions
 from gui.AlertBox import AlertBox
 from gui.PopupBox import PopupBox
 
@@ -53,13 +54,16 @@ class PluginInterface(plugin.ItemPlugin):
     def __init__(self):
         _debug_('reencode.PluginInterface.__init__(self)')
         plugin.ItemPlugin.__init__(self)
+        self.server = EncodingClientActions()
         self.title = ''
         self.source = ''
         self.output = ''
         self.resetprofile()
+        self.timeslice = [ None, None ]
 
     def resetprofile(self):
         self.profile = {}
+        self.timeslice = [ None, None ]
         self.profile['container'] = config.REENCODE_CONTAINER
         self.profile['resolution'] = config.REENCODE_RESOLUTION
         self.profile['videocodec'] = config.REENCODE_VIDEOCODEC
@@ -148,6 +152,8 @@ class PluginInterface(plugin.ItemPlugin):
         menu_items += [ menu.MenuItem(_('Start Encoding'), self.create_job, self.profile) ]
         menu_items += [ menu.MenuItem(_('Select Encoding Profile'), action=self.select_profile) ]
         menu_items += [ menu.MenuItem(_('Modify Container'), action=self.mod_container) ]
+        menu_items += [ menu.MenuItem(_('Modify Start Time'), action=self.mod_start_time) ]
+        menu_items += [ menu.MenuItem(_('Modify End Time'), action=self.mod_end_time) ]
         menu_items += [ menu.MenuItem(_('Modify Resolution'), action=self.mod_resolution) ]
         menu_items += [ menu.MenuItem(_('Modify Video Codec'), action=self.mod_videocodec) ]
         menu_items += [ menu.MenuItem(_('Modify Video Bitrate'), action=self.mod_videobitrate) ]
@@ -161,6 +167,7 @@ class PluginInterface(plugin.ItemPlugin):
         menuw.pushmenu(encoding_menu)
         menuw.refresh()
 
+
     def select_profile(self, arg=None, menuw=None):
         _debug_('select_profile(self, arg=None, menuw=None)', 2)
         menu_items = []
@@ -173,6 +180,46 @@ class PluginInterface(plugin.ItemPlugin):
         menuw.pushmenu(encoding_menu)
         menuw.refresh()
 
+
+    def set_start_time(self, arg=None, menuw=None):
+        self.timeslice[0] = arg
+        if menuw:
+            menuw.back_one_menu(arg='reload')
+
+
+    def set_end_time(self, arg=None, menuw=None):
+        self.timeslice[1] = arg
+        if menuw:
+            menuw.back_one_menu(arg='reload')
+
+
+    def _select_time(self, arg=None, menuw=None, which=None):
+        bookmarkfile = util.get_bookmarkfile(self.item.filename)
+        if not os.path.exists(bookmarkfile):
+            self.error(_('No bookmarks are set for this video'))
+            return
+        menu_items = []
+        menu_items = [ menu.MenuItem(_('Do not set'), action=which, arg=None),]
+        for line in util.readfile(bookmarkfile):
+            sec = int(line)
+            hour = int(sec/3600)
+            min = int((sec-(hour*3600))/60)
+            time = '%0.2d:%0.2d:%0.2d' % (hour,min,sec % 60)
+            menu_items.append(menu.MenuItem(time, action=which, arg=sec))
+        encoding_menu = menu.Menu(_('Select Time'), menu_items, item_types = 'video encoding menu')
+        encoding_menu.infoitem = self
+        menuw.pushmenu(encoding_menu)
+        menuw.refresh()
+
+
+    def mod_start_time(self, arg=None, menuw=None):
+        self._select_time(arg, menuw, self.set_start_time)
+
+
+    def mod_end_time(self, arg=None, menuw=None):
+        self._select_time(arg, menuw, self.set_end_time)
+
+
     def mod_container(self, arg=None, menuw=None):
         _debug_('mod_container(self, arg=%r, menuw=%r)' % (arg, menuw), 2)
         items = []
@@ -183,6 +230,7 @@ class PluginInterface(plugin.ItemPlugin):
         menuw.pushmenu(container_menu)
         menuw.refresh()
 
+
     def mod_resolution(self, arg=None, menuw=None):
         _debug_('mod_resolution(self, arg=%r, menuw=%r)' % (arg, menuw), 2)
         items = []
@@ -193,6 +241,7 @@ class PluginInterface(plugin.ItemPlugin):
         menuw.pushmenu(resolution_menu)
         menuw.refresh()
 
+
     def mod_videocodec(self, arg=None, menuw=None):
         _debug_('mod_videocodec(self, arg=%r, menuw=%r)' % (arg, menuw), 2)
         items = []
@@ -203,6 +252,7 @@ class PluginInterface(plugin.ItemPlugin):
         menuw.pushmenu(videocodec_menu)
         menuw.refresh()
 
+
     def mod_videobitrate(self, arg=None, menuw=None):
         _debug_('mod_videobitrate(self, arg=%r, menuw=%r)' % (arg, menuw), 2)
         items = []
@@ -213,6 +263,7 @@ class PluginInterface(plugin.ItemPlugin):
         menuw.pushmenu(videobitrate_menu)
         menuw.refresh()
 
+
     def mod_audiocodec(self, arg=None, menuw=None):
         _debug_('mod_audiocodec(self, arg=%r, menuw=%r)' % (arg, menuw), 2)
         items = []
@@ -223,6 +274,7 @@ class PluginInterface(plugin.ItemPlugin):
         menuw.pushmenu(audiocodec_menu)
         menuw.refresh()
 
+
     def mod_audiobitrate(self, arg=None, menuw=None):
         _debug_('mod_audiobitrate(self, arg=%r, menuw=%r)' % (arg, menuw), 2)
         items = []
@@ -233,6 +285,7 @@ class PluginInterface(plugin.ItemPlugin):
         menuw.pushmenu(audiobitrate_menu)
         menuw.refresh()
 
+
     def mod_numpasses(self, arg=None, menuw=None):
         _debug_('mod_numpasses(self, arg=%r, menuw=%r)' % (arg, menuw), 2)
         items = []
@@ -243,6 +296,7 @@ class PluginInterface(plugin.ItemPlugin):
         menuw.pushmenu(numpasses_menu)
         menuw.refresh()
 
+
     def mod_numthreads(self, arg=None, menuw=None):
         _debug_('mod_numthreads(self, arg=%r, menuw=%r)' % (arg, menuw), 2)
         items = []
@@ -253,6 +307,7 @@ class PluginInterface(plugin.ItemPlugin):
         menuw.pushmenu(numthreads_menu)
         menuw.refresh()
 
+
     def mod_videofilter(self, arg=None, menuw=None):
         _debug_('mod_videofilter(self, arg=%r, menuw=%r)' % (arg, menuw), 2)
         items = []
@@ -263,6 +318,7 @@ class PluginInterface(plugin.ItemPlugin):
         menuw.pushmenu(videofilter_menu)
         menuw.refresh()
 
+
     def select_encoding_profile(self, arg=None, menuw=None):
         _debug_('select_encoding_profile(self, arg=%r, menuw=%r)' % (arg, menuw), 2)
 
@@ -314,6 +370,7 @@ class PluginInterface(plugin.ItemPlugin):
         if menuw:
             menuw.back_one_menu(arg='reload')
 
+
     def alter_prop(self, arg=(None, None), menuw=None):
         _debug_('alter_prop(self, arg=%r, menuw=%r)' % (arg, menuw), 2)
         (prop, val) = arg
@@ -362,8 +419,8 @@ class PluginInterface(plugin.ItemPlugin):
 
         box = PopupBox(text=_('Please wait, analyzing video...'))
         box.show()
-        (status, resp) = initEncodeJob(self.source, self.output, self.title)
-        _debug_('initEncodeJob:status:%s resp:%s' % (status, resp))
+        (status, resp) = self.server.initEncodingJob(self.source, self.output, self.title)
+        _debug_('initEncodingJob:status:%s resp:%s' % (status, resp))
         box.destroy()
         if not status:
             self.error(resp)
@@ -371,42 +428,49 @@ class PluginInterface(plugin.ItemPlugin):
 
         idnr = resp
 
-        (status, resp) = setContainer(idnr, profile['container'])
+        (status, resp) = self.server.setTimeslice(idnr, self.timeslice)
+        _debug_('setTimeslice:status:%s resp:%s' % (status, resp))
+        if not status:
+            self.error(resp)
+            return
+
+        (status, resp) = self.server.setContainer(idnr, profile['container'])
         _debug_('setContainer:status:%s resp:%s' % (status, resp))
         if not status:
             self.error(resp)
             return
 
         multipass = profile['numpasses'] > 1
-        (status, resp) = setVideoCodec(idnr, profile['videocodec'], 0, multipass, profile['videobitrate'], profile['altprofile'])
+        (status, resp) = self.server.setVideoCodec(idnr, profile['videocodec'], 0, multipass,
+            profile['videobitrate'], profile['altprofile'])
         _debug_('setVideoCodec:status:%s resp:%s' % (status, resp))
         if not status:
             self.error(resp)
             return
 
-        (status, resp) = setAudioCodec(idnr, profile['audiocodec'], profile['audiobitrate'])
+        (status, resp) = self.server.setAudioCodec(idnr, profile['audiocodec'], profile['audiobitrate'])
         _debug_('setAudioCodec:status:%s resp:%s' % (status, resp))
         if not status:
             self.error(resp)
             return
 
-        (status, resp) = setNumThreads(idnr, profile['numthreads'])
+        (status, resp) = self.server.setNumThreads(idnr, profile['numthreads'])
         _debug_('setNumThreads:status:%s resp:%s' % (status, resp))
         if not status:
             self.error(resp)
             return
 
-        (status, resp) = setVideoRes(idnr, profile['resolution'])
+        (status, resp) = self.server.setVideoRes(idnr, profile['resolution'])
         _debug_('setVideoRes:status:%s resp:%s' % (status, resp))
         if not status:
             self.error(resp)
             return
 
-        #(status, resp) = setVideoFilters(idnr, vfilters)
+        #(status, resp) = self.server.setVideoFilters(idnr, vfilters)
         #_debug_('setVideoFilters:status:%s resp:%s' % (status, resp))
 
         #And finally, qeue and start the job
-        (status, resp) = queueIt(idnr, True)
+        (status, resp) = self.server.queueIt(idnr, True)
         _debug_('queueIt:status:%s resp:%s' % (status, resp))
 
         if not status:
