@@ -2,7 +2,7 @@
 # -----------------------------------------------------------------------
 # Item for video objects
 # -----------------------------------------------------------------------
-# $Id: videoitem.py 10559 2008-03-22 11:42:02Z duncan $
+# $Id: videoitem.py 11067 2008-09-23 19:16:22Z duncan $
 #
 # Notes:
 #
@@ -30,6 +30,10 @@
 # -----------------------------------------------------------------------
 
 
+"""
+Item for video objects.
+"""
+
 import os
 import re
 import md5
@@ -50,20 +54,57 @@ from item  import Item, FileInformation
 from event import *
 from skin.widgets import ScrollableTextScreen
 
-class VideoItem(Item):
+from util.benchmark import benchmark
+benchmarking = config.DEBUG_BENCHMARKING
+benchmarkcall = config.DEBUG_BENCHMARKCALL
 
+class VideoItem(Item):
+    """
+    A class describing a video item.
+
+    @ivar autovars: a list of variables.
+    @ivar type: the item type, video for this item.
+    @ivar variants: a list of audio variants.
+    @ivar subitems: a list of files or tracks to play.
+    @ivar current_subitem: the current video item.
+    @ivar media_id: media id when the item is on removable media
+    @ivar subtitle_file: file name of a file containing subtitles.
+    @ivar audio_file: file name of a file containing audio.
+    @ivar mplayer_options: additional options for mplayer.
+    @ivar tv_show: if the item is a tv programme.
+    @ivar video_width: width of the video.
+    @ivar video_height: height of the video.
+    @ivar selected_subtitle: current subtitle track.
+    @ivar selected_audio: current audio track.
+    @ivar elapsed: seconds that the item has played.
+    @ivar possible_players: a list of possible players.
+    @ivar player: current player.
+    @ivar player_rating: rating of the current player.
+    """
+
+    @benchmark(benchmarking, benchmarkcall)
     def __init__(self, url, parent, info=None, parse=True):
+        """
+        Create an instance of a VideoItem
+
+        @param url: the pysudo URL for the VideoItem
+        @param parent: the parent of the VideoItem
+        @param info: controls if additional information is found
+        @type info: boolean
+        @param parse: controls if the url is parsed
+        @type parse: boolean
+        """
         self.autovars = []
         Item.__init__(self, parent)
         self.type = 'video'
 
-        self.variants          = []         # if this item has variants
-        self.subitems          = []         # more than one file/track to play
+        self.variants          = []
+        self.subitems          = []
         self.current_subitem   = None
         self.media_id          = ''
 
-        self.subtitle_file     = {}         # text subtitles
-        self.audio_file        = {}         # audio dubbing
+        self.subtitle_file     = {}
+        self.audio_file        = {}
 
         self.mplayer_options   = ''
         self.tv_show           = False
@@ -84,7 +125,6 @@ class VideoItem(Item):
         if info:
             self.info.set_variables(info)
 
-
         # deinterlacing and related things
         video_deinterlace = config.VIDEO_DEINTERLACE != None and config.VIDEO_DEINTERLACE or False
         self['deinterlace'] = video_deinterlace
@@ -141,12 +181,23 @@ class VideoItem(Item):
         else:
             self['deinterlace'] = False
 
+
+    #@benchmark(benchmarking, benchmarkcall)
     def __str__(self):
+        """
+        Create a string for a VideoItem instance.
+        @returns: string representation
+        """
         s = pformat(self, depth=2)
         return s
 
 
+    #@benchmark(benchmarking, benchmarkcall)
     def __repr__(self):
+        """
+        Create a raw string for a VideoItem instance.
+        @returns: string representation
+        """
         if hasattr(self, 'name'):
             s = '%s: %r' % (self.name, self.__class__)
         else:
@@ -154,13 +205,20 @@ class VideoItem(Item):
         return s
 
 
+    @benchmark(benchmarking, benchmarkcall)
     def set_url(self, url, info=True):
         """
-        Sets a new url to the item. Always use this function and not set 'url'
-        directly because this functions also changes other attributes, like
-        filename, mode, network_play and the list of possible players
+        Sets a new url to the item. This functions also changes other
+        attributes, like file name, mode, network_play and the list of possible
+        players.
+
+        B{WARNING}: This is called whenever self.url is set, therefore it is
+        strictly forbidden to set self.url directly in this function, (infinite
+        recursion!). Use self.__dict__['url'] instead!
         """
         Item.set_url(self, url, info)
+
+        # additional types of urls
         if url.startswith('dvd://') or url.startswith('vcd://'):
             self.network_play = False
             self.mimetype = self.url[:self.url.find('://')].lower()
@@ -182,36 +240,87 @@ class VideoItem(Item):
         elif url.endswith('.iso') and self.info['mime'] == 'video/dvd':
             self.mimetype = 'dvd'
             self.mode     = 'dvd'
-            self.url      = 'dvd' + self.url[4:] + '/'
+            self.__dict__['url'] = 'dvd' + self.url[4:] + '/'
 
+        # cover image
         if not self.image or (self.parent and self.image == self.parent.image):
             image = vfs.getoverlay(self.filename + '.raw')
             if os.path.exists(image):
                 self.image = image
                 self.files.image = image
 
-        # do the player rating based on the new url
+        # do a new player rating based on the new url
+        self.rating()
+
+
+    @benchmark(benchmarking, benchmarkcall)
+    def rating(self):
+        """
+        Calculate a new player rating for this item.
+
+        The decision which player to use for a VideoItem is based on this
+        rating.  First each player plugins is asked for a rate, on how good it
+        can play this VideoItem. There are three possible rates: good (=2),
+        possible (=1) and unplayable(=0). This rate is then weighted by a
+        factor of ten.  Next the user's choice of prefered players is checked.
+        The user can define a rank list of players with the variable
+        VIDEO_PREFERED_PLAYER in local_conf.py. The rank that is calculated
+        from this config variable is then added to the current rate. Last but
+        not least there is the possibility that for some reason the use of a
+        special player is forced.  In that case a value of 100 is added to the
+        rate.
+
+        In the end a sorted list of possible_players is created. All players in
+        this list have a rating for this special VideoItem of not less than 10.
+        The first one is the default one, the others are offered to the user as
+        choices in the "Alternate Player" menu.
+        """
+        # create a new player rating
+        self.possible_players =[]
+        self.player = None
+        self.player_rating = 0
+        # some debug info
+        try:
+            _debug_('rating: url=%r' % (self.url), 2)
+            _debug_('rating: mode=%r' % (self.mode), 2)
+            _debug_('rating: mimetype=%r' % (self.mimetype), 2)
+            _debug_('rating: network_play=%r' % (self.network_play), 2)
+        except Exception, e:
+            print 'videoitem.py: %s' %e
         for p in plugin.getbyname(plugin.VIDEO_PLAYER, True):
             rating = p.rate(self) * 10
-            if config.VIDEO_PREFERED_PLAYER == p.name:
-                rating += 1
+            if p.name in config.VIDEO_PREFERED_PLAYER:
+                # do we have a rank list for all possible players?
+                if type(config.VIDEO_PREFERED_PLAYER) in [list, tuple]:
+                    # get the rank of this player
+                    rank = config.VIDEO_PREFERED_PLAYER.index(p.name)
+                    # lower index means higher rank, therefor more calculations
+                    rank = len(config.VIDEO_PREFERED_PLAYER)-rank
+                    # finally increade the rating
+                    rating += rank
+                else: # it is more simple if just one player is prefered
+                    rating +=1
             if hasattr(self, 'force_player') and p.name == self.force_player:
                 rating += 100
             if (rating, p) not in self.possible_players:
                 self.possible_players += [(rating, p)]
-        self.possible_players = filter(lambda l: l[0] > 0, self.possible_players)
+        # just use players with a rating of at last 10
+        self.possible_players = filter(lambda l: l[0] >= 10, self.possible_players)
         # sort the players in the order of the rating
         self.possible_players.sort(lambda l, o: -cmp(l[0], o[0]))
         if len(self.possible_players) > 0:
             # choose the best player as default player
             self.player_rating, self.player = self.possible_players[0]
-        _debug_("url=%r possible_players=%r" % (self.url, self.possible_players,), 2)
+        _debug_("rating: url=%r possible_players=%r" % (self.url, self.possible_players,), 2)
 
 
+    @benchmark(benchmarking, benchmarkcall)
     def id(self):
         """
-        Return a unique id of the item. This id should be the same when the
-        item is rebuild later with the same information
+        This id should be the same when the item is rebuild later with the same
+        information.
+
+        @returns: a unique id of the item.
         """
         ret = self.url
         if self.subitems:
@@ -223,9 +332,12 @@ class VideoItem(Item):
         return ret
 
 
+    #@benchmark(benchmarking, benchmarkcall)
     def __getitem__(self, key):
         """
-        return the specific attribute
+        Get the item's attribute.
+
+        @returns: the specific attribute
         """
         if not self.info:
             return ''
@@ -258,8 +370,13 @@ class VideoItem(Item):
             if aspect:
                 return aspect
 
-        if key == 'runtime':
+        if key == 'rating':
+            if self.info['rating']:
+                return self.info['rating']
+            else:
+                return ''
 
+        if key == 'runtime':
             if self.info['runtime']:
                 if self.info['runtime'] != 'None':
                     return self.info['runtime']
@@ -276,7 +393,6 @@ class VideoItem(Item):
                         continue
                     total += length
                 total = '%s min' % str(int(total) / 60)
-
             else:
                 if self.info['length']:
                     total = self.info['length']
@@ -296,9 +412,12 @@ class VideoItem(Item):
         return Item.__getitem__(self, key)
 
 
+    #@benchmark(benchmarking, benchmarkcall)
     def sort(self, mode=None):
         """
-        Returns the string how to sort this item
+        Sort the video items.
+
+        @returns: the string how to sort this item
         """
         if mode == 'date' and self.mode == 'file' and os.path.isfile(self.filename):
             return u'%s%s' % (os.stat(self.filename).st_ctime, Unicode(self.filename))
@@ -312,13 +431,17 @@ class VideoItem(Item):
     # actions:
 
 
+    @benchmark(benchmarking, benchmarkcall)
     def actions(self):
         """
-        return a list of possible actions on this item.
+        Menu actions for a video item.
+
+        @returns: a list of possible actions on this item.
         """
         if not self.possible_players:
             return []
 
+        # DVD actions
         if self.url.startswith('dvd://') and self.url[-1] == '/':
             if self.player_rating >= 20:
                 items = [
@@ -326,11 +449,13 @@ class VideoItem(Item):
                     (self.dvd_vcd_title_menu, _('DVD title list'))
                 ]
             else:
+                # this player is not able to deal with menus
                 items = [
                     (self.dvd_vcd_title_menu, _('DVD title list')),
                     (self.play, _('Play default track'))
                 ]
 
+        # VCD actions
         elif self.url == 'vcd://':
             if self.player_rating >= 20:
                 items = [
@@ -343,6 +468,7 @@ class VideoItem(Item):
                     (self.play, _('Play default track'))
                 ]
 
+        # youtube
         elif self.url.startswith('youtube:'):
             popup=PopupBox('Resolving YouTube video URL....')
             popup.show()
@@ -375,6 +501,7 @@ class VideoItem(Item):
         return items
 
 
+    @benchmark(benchmarking, benchmarkcall)
     def show_details(self, arg=None, menuw=None):
         """
         Show more details
@@ -382,9 +509,10 @@ class VideoItem(Item):
         ShowDetails(menuw, self)
 
 
+    @benchmark(benchmarking, benchmarkcall)
     def show_variants(self, arg=None, menuw=None):
         """
-        show a list of variants in a menu
+        Show a list of variants in a menu
         """
         if not self.menuw:
             self.menuw = menuw
@@ -393,9 +521,10 @@ class VideoItem(Item):
         self.menuw.pushmenu(m)
 
 
+    @benchmark(benchmarking, benchmarkcall)
     def create_thumbnail(self, arg=None, menuw=None):
         """
-        create a thumbnail as image icon
+        Create a thumbnail as image icon
         """
         import util.videothumb
         pop = PopupBox(text=_('Please wait....'))
@@ -406,25 +535,27 @@ class VideoItem(Item):
         menuw.delete_submenu()
 
 
+    @benchmark(benchmarking, benchmarkcall)
     def play_max_cache(self, arg=None, menuw=None):
         """
-        play and use maximum cache with mplayer
+        Play and use maximum cache with mplayer
         """
         self.play(menuw=menuw, arg='-cache 65536')
 
 
+    @benchmark(benchmarking, benchmarkcall)
     def set_next_available_subitem(self):
         """
-        select the next available subitem. Loops on each subitem and checks if the
-        needed media is really there.  If the media is there, sets self.current_subitem
-        to the given subitem and returns 1.
+        Select the next available subitem. Loops on each subitem and checks if
+        the needed media is really there.  If the media is there, sets
+        self.current_subitem to the given subitem and returns True.
 
-        If no media has been found, we set self.current_subitem to None.  If the search
-        for the next available subitem did start from the beginning of the list, then
-        we consider that no media at all was available for any subitem: we return 0.
-        If the search for the next available subitem did not start from the beginning
-        of the list, then we consider that at least one media had been found in the
-        past: we return 1.
+        If no media has been found, we set self.current_subitem to None.  If
+        the search for the next available subitem did start from the beginning
+        of the list, then we consider that no media at all was available for
+        any subitem: we return False.  If the search for the next available subitem
+        did not start from the beginning of the list, then we consider that at
+        least one media had been found in the past: we return True.
         """
         if hasattr(self, 'conf_select_this_item'):
             # XXX bad hack, clean me up
@@ -432,14 +563,14 @@ class VideoItem(Item):
             del self.conf_select_this_item
             return True
 
-        cont = 1
-        from_start = 0
         si = self.current_subitem
+        from_start = False
+        cont = True
         while cont:
             if not si:
                 # No subitem selected yet: take the first one
                 si = self.subitems[0]
-                from_start = 1
+                from_start = True
             else:
                 pos = self.subitems.index(si)
                 # Else take the next one
@@ -449,29 +580,30 @@ class VideoItem(Item):
                 else:
                     # No next subitem
                     si = None
-                    cont = 0
+                    cont = False
             if si:
                 if (si.media_id or si.media):
                     # If the file is on a removeable media
                     if util.check_media(si.media_id):
                         self.current_subitem = si
-                        return 1
+                        return True
                     elif si.media and util.check_media(si.media.id):
                         self.current_subitem = si
-                        return 1
+                        return True
                 else:
                     # if not, it is always available
                     self.current_subitem = si
-                    return 1
+                    return True
+
         self.current_subitem = None
         return not from_start
 
 
+    @benchmark(benchmarking, benchmarkcall)
     def play(self, arg=None, menuw=None):
         """
-        play the item.
+        Play the item.
         """
-
         if not self.player or self.player_rating < 10:
             AlertBox(text=_('No player for this item found')).show()
             return
@@ -581,6 +713,7 @@ class VideoItem(Item):
                 AlertBox(text=error, handler=self.error_handler).show()
 
 
+    @benchmark(benchmarking, benchmarkcall)
     def error_handler(self):
         """
         error handler if play doesn't work to send the end event and stop
@@ -590,6 +723,7 @@ class VideoItem(Item):
         self.stop()
 
 
+    @benchmark(benchmarking, benchmarkcall)
     def stop(self, arg=None, menuw=None):
         """
         execute commands if defined
@@ -597,13 +731,11 @@ class VideoItem(Item):
         if config.VIDEO_POST_PLAY:
             os.system(config.VIDEO_POST_PLAY)
 
-        """
-        stop playing
-        """
         if self.player:
             self.player.stop()
 
 
+    @benchmark(benchmarking, benchmarkcall)
     def dvd_vcd_title_menu(self, arg=None, menuw=None):
         """
         Generate special menu for DVD/VCD/SVCD content
@@ -631,7 +763,7 @@ class VideoItem(Item):
             i.set_url(self.url + str(titlenum+1), False)
             i.info = copy.copy(self.info)
             # copy the attributes from mmpython about this track
-            i.info.mmdata = self.info.mmdata['tracks'][titlenum]
+            i.info.discinfo = self.info.discinfo['tracks'][titlenum]
             i.info.set_variables(self.info.get_variables())
             i.info_type       = 'track'
             i.files           = None
@@ -639,14 +771,15 @@ class VideoItem(Item):
             items.append(i)
 
 
-        moviemenu = menu.Menu(self.name, items, umount_all = 1, fxd_file=self.skin_fxd)
+        moviemenu = menu.Menu(self.name, items, umount_all=1, fxd_file=self.skin_fxd)
         moviemenu.item_types = 'video'
         self.menuw.pushmenu(moviemenu)
 
 
+    @benchmark(benchmarking, benchmarkcall)
     def settings(self, arg=None, menuw=None):
         """
-        create a menu with 'settings'
+        Create a menu with 'settings'
         """
         if not self.menuw:
             self.menuw = menuw
@@ -654,9 +787,10 @@ class VideoItem(Item):
         self.menuw.pushmenu(confmenu)
 
 
+    @benchmark(benchmarking, benchmarkcall)
     def eventhandler(self, event, menuw=None):
         """
-        eventhandler for this item
+        Eventhandler for this item
         """
         # when called from mplayer.py, there is no menuw
         if not menuw:
@@ -718,6 +852,7 @@ class ShowDetails(ScrollableTextScreen):
     """
     Screen to show more details
     """
+    @benchmark(benchmarking, benchmarkcall)
     def __init__(self, menuw, movie):
         if movie is None:
             name = _('No Information Available')
@@ -734,13 +869,15 @@ class ShowDetails(ScrollableTextScreen):
             else:
                 # or just the description, if there is no subtitle
                 description = desc + u'\n\n'
-            # add some additional inofs if they are available
+            # add some additional infos if they are available
             if movie['genre']:
                 description += _('Genre') + u' : '+movie['genre'] + u'\n'
             if movie['length']:
                 description +=  _('Length')+ u' : '+movie['length'] + u'\n'
             if movie['year']:
                 description +=  _('Year')+u' : '+movie['year'] + u'\n'
+            if movie['rating']:
+                description +=  _('Rating')+u' : '+movie['rating'] + u'\n'
 
         # that's all, we can show this to the user
         ScrollableTextScreen.__init__(self, 'tvguideinfo', description)
@@ -750,6 +887,7 @@ class ShowDetails(ScrollableTextScreen):
         self.show(menuw)
 
 
+    @benchmark(benchmarking, benchmarkcall)
     def getattr(self, name):
         if name == 'title':
             return self.name
@@ -760,6 +898,7 @@ class ShowDetails(ScrollableTextScreen):
         return u''
 
 
+    @benchmark(benchmarking, benchmarkcall)
     def eventhandler(self, event, menuw=None):
         """
         eventhandler for the programm description display
