@@ -2,7 +2,7 @@
 # -----------------------------------------------------------------------
 # Classes useful for the web interface.
 # -----------------------------------------------------------------------
-# $Id: web_types.py 10410 2008-02-21 20:49:20Z duncan $
+# $Id: web_types.py 11106 2008-10-13 18:32:11Z duncan $
 #
 # Notes:
 # Todo:
@@ -38,6 +38,9 @@ import socket
 
 from twisted.web.woven import page
 from twisted.web.resource import Resource
+from util.benchmark import benchmark
+benchmarking = config.DEBUG_BENCHMARKING
+benchmarkcall = config.DEBUG_BENCHMARKCALL
 
 TRUE = 1
 FALSE = 0
@@ -45,6 +48,7 @@ FALSE = 0
 
 class FreevoPage(page.Page):
 
+    @benchmark(benchmarking, benchmarkcall)
     def __init__(self, model=None, template=None):
         #print 'FreevoPage.__init__(model=%r, template=%r)' % (model, template)
 
@@ -71,6 +75,7 @@ class FreevoResource(Resource):
     Otherwise a error messages is returned and shown to the user.
     """
 
+    @benchmark(benchmarking, benchmarkcall)
     def render(self, request):
         """ render methode of this class.
 
@@ -96,9 +101,11 @@ class FreevoResource(Resource):
             # authentication was successfull
             # thus we return the self._render methode
             # which hopefully will do something usefull
+            self.session = request.getSession()
             return self._render(request)
 
 
+    @benchmark(benchmarking, benchmarkcall)
     def auth_user(self, username, password):
         """ check of username and password
 
@@ -128,11 +135,13 @@ class HTMLResource:
     subclass which represents the webpage, use all this methodes to build
     the html string  and return HTMLResource.res in the end.
     """
+    @benchmark(benchmarking, benchmarkcall)
     def __init__(self):
         # create empty result string which must be filled with life
         self.res =''
 
 
+    @benchmark(benchmarking, benchmarkcall)
     def printContentType(self, content_type='text/html'):
         """ Content"""
         # debug print
@@ -141,7 +150,8 @@ class HTMLResource:
         self.res += 'Content-type: %s\n\n' % content_type
 
 
-    def printHeader(self, title='unknown page', style=None, script=None, selected='Help', prefix=0):
+    @benchmark(benchmarking, benchmarkcall)
+    def printHeader(self, title='unknown page', style=None, script=None, rss=None, selected='Help', prefix=0):
         """ Header
 
         This produces the header of a freevo page with the navigation bar.
@@ -183,6 +193,9 @@ class HTMLResource:
         # java script
         if script != None:
             self.res += '\t<script language="JavaScript" type="text/JavaScript" src="'+script+'"></script>\n'
+        # rss
+        if rss is not None:
+            self.res += '\t<link rel="alternate" href="'+rss+'" type="application/rss+xml" title="" id="gallery" />\n'
         # end of <head>
         self.res += '</head>\n'
         # here starts the body
@@ -195,19 +208,14 @@ class HTMLResource:
         # start the bar, it is build as a list
         self.res += '<div id="header">\n<ul>'
         # this items should be shown in the tab bar
-        items = [(_('Home'),_('Home'),'%sindex.rpy' % str(strprefix)),
-                 (_('TV Guide'),_('View TV Listings'),'%sguide.rpy' % str(strprefix)),
-                 (_('Scheduled Recordings'),_('View Scheduled Recordings'),'%srecord.rpy' % str(strprefix)),
-                 (_('Favorites'),_('View Favorites'),'%sfavorites.rpy' % str(strprefix)),
-                 (_('Media Library'),_('View Media Library'),'%slibrary.rpy' % str(strprefix)),
-                 (_('Manual Recording'),_('Schedule a Manual Recording'),'%smanualrecord.rpy' % str(strprefix)),
-                 (_('Search'),_('Advanced Search Page'),'%ssearch.rpy' % str(strprefix)),
-                 (_('Help'),_('View Online Help and Documentation'),'%shelp/' % str(strprefix))]
+        items = []
+        for label, title, link in config.WWW_PAGES:
+            items.append((label, title, os.path.join(str(strprefix), link)))
         # maybe also a personal web pages
         if hasattr(config, 'WWW_PERSONAL_PAGE'):
             for page in config.WWW_PERSONAL_PAGE:
-                items.append((page[0], page[1], '%s%s' % (str(strprefix), page[2])))
-        if hasattr(config, 'ICECAST_WWW_PAGE') and config.ICECAST_WWW_PAGE:
+                items.append((page[0], page[1], os.path.join(str(strprefix), page[2])))
+        if hasattr(config, 'WWW_ICECAST_PAGE') and config.WWW_ICECAST_PAGE:
             items.append((_('Icecast List'),_('Change Icecast List'),'%siceslistchanger.rpy' % (strprefix)))
         # go through the items and create the bar
         for i in items:
@@ -223,6 +231,7 @@ class HTMLResource:
         self.res += '\n<!-- Main Content -->\n';
 
 
+    @benchmark(benchmarking, benchmarkcall)
     def tableOpen(self, opts=''):
         """ Opens a table
 
@@ -232,6 +241,7 @@ class HTMLResource:
         self.res += "<table "+opts+">\n"
 
 
+    @benchmark(benchmarking, benchmarkcall)
     def tableClose(self):
         """
         Close a table
@@ -240,6 +250,7 @@ class HTMLResource:
         self.res += "</table>\n"
 
 
+    @benchmark(benchmarking, benchmarkcall)
     def tableHeadOpen(self, opts=''):
         """ Open a table header line
 
@@ -249,6 +260,7 @@ class HTMLResource:
         self.res += "  <thead "+opts+">\n"
 
 
+    @benchmark(benchmarking, benchmarkcall)
     def tableHeadClose(self, opts=''):
         """
         Closes a table header line
@@ -257,6 +269,7 @@ class HTMLResource:
         self.res += "  </thead>\n"
 
 
+    @benchmark(benchmarking, benchmarkcall)
     def tableBodyOpen(self, opts=''):
         """ Opens a table body
 
@@ -266,6 +279,7 @@ class HTMLResource:
         self.res += "  <tbody "+opts+">\n"
 
 
+    @benchmark(benchmarking, benchmarkcall)
     def tableBodyClose(self, opts=''):
         """
         Closes a table body
@@ -274,6 +288,7 @@ class HTMLResource:
         self.res += "  </tbody>\n"
 
 
+    @benchmark(benchmarking, benchmarkcall)
     def tableFootOpen(self, opts=''):
         """ Opens a table footer
 
@@ -283,6 +298,7 @@ class HTMLResource:
         self.res += "  <tfoot "+opts+">\n"
 
 
+    @benchmark(benchmarking, benchmarkcall)
     def tableFootClose(self, opts=''):
         """
         Closes a table footer.
@@ -291,6 +307,7 @@ class HTMLResource:
         self.res += "  </tfoot>\n"
 
 
+    @benchmark(benchmarking, benchmarkcall)
     def tableRowOpen(self, opts=''):
         """ Opens a table row
 
@@ -300,6 +317,7 @@ class HTMLResource:
         self.res += "     <tr "+opts+">\n"
 
 
+    @benchmark(benchmarking, benchmarkcall)
     def tableRowClose(self):
         """
         Closes a table row
@@ -308,6 +326,7 @@ class HTMLResource:
         self.res += "     </tr>\n"
 
 
+    @benchmark(benchmarking, benchmarkcall)
     def tableCell(self, data='', opts=''):
         """ Creates a table cell
 
@@ -318,6 +337,7 @@ class HTMLResource:
         self.res += "       <td "+opts+">"+data+"</td>\n"
 
 
+    @benchmark(benchmarking, benchmarkcall)
     def formValue(self, form=None, key=None):
         #print 'formValue(form=%r, key=%r)' % (form, key)
         if not form or not key:
@@ -330,6 +350,7 @@ class HTMLResource:
         return val
 
 
+    @benchmark(benchmarking, benchmarkcall)
     def printFooter(self):
         """
         Closes the html document
@@ -338,6 +359,7 @@ class HTMLResource:
         self.res += '</body>\n</html>\n'
 
 
+    @benchmark(benchmarking, benchmarkcall)
     def printSearchForm(self):
         """ Creates the simple search form"""
 
@@ -348,6 +370,7 @@ class HTMLResource:
     </form>
     """
 
+    @benchmark(benchmarking, benchmarkcall)
     def printAdvancedSearchForm(self):
         """ Creates the advanced search form.
 
@@ -364,6 +387,7 @@ class HTMLResource:
     </form>
     """
 
+    @benchmark(benchmarking, benchmarkcall)
     def printMessages(self, messages):
         """
         Prints a message
@@ -375,6 +399,7 @@ class HTMLResource:
             self.res += "   <li>%s</li>\n" % m
             self.res += "</ul>\n"
 
+    @benchmark(benchmarking, benchmarkcall)
     def printMessagesFinish(self, messages):
         """
         Print messages and add the search form, links and footer.
@@ -385,6 +410,7 @@ class HTMLResource:
         self.printLinks()
         self.printFooter()
 
+    @benchmark(benchmarking, benchmarkcall)
     def printLinks(self, prefix=0):
         """
         Print Link
@@ -393,12 +419,13 @@ class HTMLResource:
         #print 'printLinks(prefix=%r)' % (prefix)
         #
         #try:
-        #    if config.ICECAST_WWW_PAGE:
+        #    if config.WWW_ICECAST_PAGE:
         #        self.res += '<a href="%siceslistchanger.rpy">Change Icecast List</a>' % strprefix
         #except AttributeError:
         #    pass
         return
 
+    @benchmark(benchmarking, benchmarkcall)
     def printBreadcrumb(self, media, mediadirs, dir):
         """
         ???
@@ -415,6 +442,7 @@ class HTMLResource:
 
         return breadcrumb
 
+    @benchmark(benchmarking, benchmarkcall)
     def printPassword(self, password):
         """
         ???
@@ -435,6 +463,7 @@ class HTMLResource:
         //-->
         </script>"""
 
+    @benchmark(benchmarking, benchmarkcall)
     def printImagePopup(self):
         """
         Opens a Popup to display an image
@@ -472,6 +501,7 @@ class HTMLResource:
         }
         </script> """
 
+    @benchmark(benchmarking, benchmarkcall)
     def printWebRemote(self):
         """ Prints web remote
 
