@@ -2,7 +2,7 @@
 # -----------------------------------------------------------------------
 # freevo_config.py - System configuration
 # -----------------------------------------------------------------------
-# $Id: freevo_config.py 10508 2008-03-12 07:24:34Z duncan $
+# $Id: freevo_config.py 11209 2008-12-01 18:38:10Z duncan $
 #
 # Notes:
 #    This file contains the freevo settings. To change the settings
@@ -108,125 +108,130 @@ from event import *
 # of the config file doesn't match, Freevo won't start. If the minor version
 # is different, there will be only a warning
 
-LOCAL_CONF_VERSION = 5.23
+LOCAL_CONF_VERSION = 5.26
 
 # Description of changes in each new version
 FREEVO_CONF_CHANGES = [
     (2.0,
-     '''Changed xmame_SDL to just xmame'''), ]
+     """Changed xmame_SDL to just xmame"""),
+    (2.1,
+     """Added vlc"""),
+    (2.2,
+     """Added unzip"""),
+]
 
 LOCAL_CONF_CHANGES = [
     (1.1,
-     '''ROM_DRIVES are autodetected if left empty.
+     """ROM_DRIVES are autodetected if left empty.
     Added AUDIO_RANDOM_PLAYLIST (default on).
     Added COVER_DIR for covers for files on CDs etc.
     Added AUDIO_COVER_REGEXP for selection of covers for music files.
     Changed MPlayer default args.
-    Changed TV_SETTINGS to /dev/video0.'''),
+    Changed TV_SETTINGS to /dev/video0."""),
     (2.0,
-     '''Remote control config has changed from Freevo Python files to the
+     """Remote control config has changed from Freevo Python files to the
      standard Lirc program config files, see freevo_config.py for
-     more info.'''),
+     more info."""),
     (2.1,
-     '''Added MPLAYER_ARGS_AUDIOCD for audio cd playback settings.'''),
+     """Added MPLAYER_ARGS_AUDIOCD for audio cd playback settings."""),
     (3.0,
-     '''New skin engine. The new engine has no automatic TV overscan support,
+     """New skin engine. The new engine has no automatic TV overscan support,
      you need to set OSD_OVERSCAN_X and OSD_OVERSCAN_Y. There are also new variables
      for this engine: MAIN_MENU_ITEMS and FORCE_SKIN_LAYOUT. The games menu
-     will be activated automaticly if setup.py found mame or snes'''),
+     will be activated automaticly if setup.py found mame or snes"""),
     (3.1,
-     '''Renamed TV_SHOW_IMAGE_DIR to TV_SHOW_DATA_DIR. This directory now can
-     also contain fxd files with gloabl informations and mplayer options'''),
+     """Renamed TV_SHOW_IMAGE_DIR to TV_SHOW_DATA_DIR. This directory now can
+     also contain fxd files with gloabl informations and mplayer options"""),
     (3.2,
-     '''Removed MPLAYER_ARGS_* and added a hash MPLAYER_ARGS to set args for
+     """Removed MPLAYER_ARGS_* and added a hash MPLAYER_ARGS to set args for
      all different kinds of files. Also added MPLAYER_SOFTWARE_SCALER to use
-     the software scaler for fast CPUs'''),
+     the software scaler for fast CPUs"""),
     (3.3,
-     '''Added AUDIO_FORMAT_STRING to customize the audio item title generation'''),
+     """Added AUDIO_FORMAT_STRING to customize the audio item title generation"""),
     (3.4,
-     '''Removed RC_MPLAYER_CMDS for video and audio. Set special handling (and
+     """Removed RC_MPLAYER_CMDS for video and audio. Set special handling (and
      other key mappings with the variable EVENTS. See event.py for possible
-     events'''),
+     events"""),
     (3.5,
-     '''Added xine support (see xine section in freevo_config.py),
+     """Added xine support (see xine section in freevo_config.py),
      MPLAYER_AUTOCROP for 16:9 tv sets, ONLY_SCAN_DATADIR to make freevo start
-     faster and TVGUIDE_HOURS_PER_PAGE customize the tv guide'''),
+     faster and TVGUIDE_HOURS_PER_PAGE customize the tv guide"""),
     (3.7,
-     '''Added USE_MEDIAID_TAG_NAMES as directory based variable and
-     HIDE_UNUSABLE_DISCS to hide discs in the wrong menus and empty drives'''),
+     """Added USE_MEDIAID_TAG_NAMES as directory based variable and
+     HIDE_UNUSABLE_DISCS to hide discs in the wrong menus and empty drives"""),
     (3.8,
-     '''Restructured GAMES_ITEMS and added XMLTV_GRABBER and XMLTV_DAYS for the
+     """Restructured GAMES_ITEMS and added XMLTV_GRABBER and XMLTV_DAYS for the
      tv_grab helper script. Also added USE_NETWORK to deactivate everything
-     that needs a network connection.'''),
+     that needs a network connection."""),
     (3.9,
-     '''Add MPLAYER_SET_AUDIO_DELAY to correct AV sync problems'''),
+     """Add MPLAYER_SET_AUDIO_DELAY to correct AV sync problems"""),
     (3.91,
-     '''Add SKIN_FORCE_TEXTVIEW_STYLE and SKIN_MEDIAMENU_FORCE_TEXTVIEW to add
-     more control when to switch to text view.'''),
+     """Add SKIN_FORCE_TEXTVIEW_STYLE and SKIN_MEDIAMENU_FORCE_TEXTVIEW to add
+     more control when to switch to text view."""),
     (4.00,
-     '''Reworked the directory settings: MOVIE_PLAYLISTS and AUDIO_RANDOM_PLAYLIST
+     """Reworked the directory settings: MOVIE_PLAYLISTS and AUDIO_RANDOM_PLAYLIST
      are removed, the new variables to control a directory style are
      DIRECTORY_CREATE_PLAYLIST, DIRECTORY_ADD_PLAYLIST_FILES,
      DIRECTORY_ADD_RANDOM_PLAYLIST and DIRECTORY_AUTOPLAY_ITEMS. The directory
      updated now uses stat, set DIRECTORY_USE_STAT_FOR_CHANGES = 0 if you have
-     problems with it.'''),
+     problems with it."""),
     (4.01,
-     '''Removed SUFFIX_VIDEO_FILES and replaced it with SUFFIX_VIDEO_MPLAYER_FILES
+     """Removed SUFFIX_VIDEO_FILES and replaced it with SUFFIX_VIDEO_MPLAYER_FILES
      and SUFFIX_VIDEO_XINE_FILES. Use PREFERED_VIDEO_PLAYER to choose a prefered
-     player.'''),
+     player."""),
     (4.02,
-     '''Added CHILDAPP_DEBUG to debug all kinds of childapps. MPLAYER_DEBUG will be
+     """Added CHILDAPP_DEBUG to debug all kinds of childapps. MPLAYER_DEBUG will be
      removed soon. Renamed PREFERED_VIDEO_PLAYER to VIDEO_PREFERED_PLAYER and
-     added AUDIO_PREFERED_PLAYER.'''),
+     added AUDIO_PREFERED_PLAYER."""),
     (4.03,
-     '''Removed MOVIE_DATA_DIR and COVER_DIR. It has been replaved by the new
-     virtual filesystem controlled by OVERLAY_DIR'''),
+     """Removed MOVIE_DATA_DIR and COVER_DIR. It has been replaved by the new
+     virtual filesystem controlled by OVERLAY_DIR"""),
     (5.00,
-     '''Changed some config variables. Use \'./freevo convert_config\' to convert
-     your local_conf.py to change the variable names'''),
+     """Changed some config variables. Use \'./freevo convert_config\' to convert
+     your local_conf.py to change the variable names"""),
     (5.01,
-     '''Add AUDIO_SHOW_VIDEOFILES to enable video files in the audio menu'''),
+     """Add AUDIO_SHOW_VIDEOFILES to enable video files in the audio menu"""),
     (5.02,
-     '''Add XINE_ARGS_DEF to set xine arguments and OSD_BUSYICON_TIMER to show
-     a busy icon when the menu takes too much time building'''),
+     """Add XINE_ARGS_DEF to set xine arguments and OSD_BUSYICON_TIMER to show
+     a busy icon when the menu takes too much time building"""),
     (5.03,
-     '''Add UMASK to set umask for files in vfs''' ),
+     """Add UMASK to set umask for files in vfs""" ),
     (5.04,
-     '''SKIN_XML_FILE set to nothing as default, SKIN_START_LAYOUT is removed.
-     When SKIN_XML_FILE is not set, the skin will remember the last settings'''),
+     """SKIN_XML_FILE set to nothing as default, SKIN_START_LAYOUT is removed.
+     When SKIN_XML_FILE is not set, the skin will remember the last settings"""),
     (5.05,
-     '''Use MMPYTHON_CREATE_MD5_ID with current mmpython cvs to have a second
-     way to generate the disc ids in case they are not unique on your system'''),
+     """Use MMPYTHON_CREATE_MD5_ID with current mmpython cvs to have a second
+     way to generate the disc ids in case they are not unique on your system"""),
     (5.06,
-     '''Add MEDIAINFO_USE_MEMORY. Setting this variable will keep all cache
+     """Add MEDIAINFO_USE_MEMORY. Setting this variable will keep all cache
      files in memory. Startup will be slower, but for large directories, this
-     will speed up entering the dir'''),
+     will speed up entering the dir"""),
     (5.07,
-     '''Add MENU_ARROW_NAVIGATION to change navigation style. New one is default
-     now. Also added OSD_EXTRA_FONT_PATH to search for fonts'''),
+     """Add MENU_ARROW_NAVIGATION to change navigation style. New one is default
+     now. Also added OSD_EXTRA_FONT_PATH to search for fonts"""),
     (5.08,
-     '''Change MENU_ARROW_NAVIGATION to old style and make blurr the new default
-     skin. Also added RESTART_SYS_CMD, OSD_DIM_TEXT and OSD_UPDATE_COMPLETE_REDRAW.'''),
+     """Change MENU_ARROW_NAVIGATION to old style and make blurr the new default
+     skin. Also added RESTART_SYS_CMD, OSD_DIM_TEXT and OSD_UPDATE_COMPLETE_REDRAW."""),
     (5.09,
-     '''Add CACHE_IMAGES to turn off image caching. A new variable is
-     IMAGEVIEWER_BLEND_MODE to control the blending effect in the image viewer'''),
+     """Add CACHE_IMAGES to turn off image caching. A new variable is
+     IMAGEVIEWER_BLEND_MODE to control the blending effect in the image viewer"""),
     (5.11,
-     '''Add IMAGEVIEWER_OSD to customize the osd and VIDEO_AUTOJOIN to auto join
-     movies with more than one file'''),
+     """Add IMAGEVIEWER_OSD to customize the osd and VIDEO_AUTOJOIN to auto join
+     movies with more than one file"""),
     (5.12,
-     '''Added TV_RECORD_SERVER_UID to set the uid for the recordserver and
+     """Added TV_RECORD_SERVER_UID to set the uid for the recordserver and
      TV_RECORDFILE_SUFFIX for the suffix. If your TV_RECORDFILE_MASK contains
-     the suffix, please remove it here'''),
+     the suffix, please remove it here"""),
     (5.13,
-     '''Added TV_RECORD_SERVER_GID to set the gid for the recordserver. If you
-     use TV_RECORD_SERVER_UID, the gui _must_ match one of the users gids''' ),
+     """Added TV_RECORD_SERVER_GID to set the gid for the recordserver. If you
+     use TV_RECORD_SERVER_UID, the gui _must_ match one of the users gids""" ),
     (5.14,
-     '''Add IMAGEVIEWER_DURATION for auto slideshows''' ),
+     """Add IMAGEVIEWER_DURATION for auto slideshows""" ),
     (5.15,
-     '''Add two variables for mplayer post processing: MPLAYER_VF_INTERLACED and
-     MPLAYER_VF_PROGRESSIVE''' ),
+     """Add two variables for mplayer post processing: MPLAYER_VF_INTERLACED and
+     MPLAYER_VF_PROGRESSIVE""" ),
     (5.16,
-     '''Removed the recordable setting in VIDEO_GROUPS, please remove this setting.
+     """Removed the recordable setting in VIDEO_GROUPS, please remove this setting.
      Added xmltv-1.2 this requires elementtree
      Added XINE_HAS_NO_LIRC to see if '--no-lirc' should be passed to xine
      Added XINE_TV_VO_DEV, XINE_TV_AO_DEV, and XINE_TV_TIMESHIFT_FILEMASK for the
@@ -238,9 +243,9 @@ LOCAL_CONF_CHANGES = [
      Added AUTOSHUTDOWN_ settings to turn off and on the machine automatically
      Added Multi-tuner support to allow recording and watching at the same time
      Added plug-in "upsoon" to stop the player when a recording is about to start
-     Added OSD_FORCE_FONTNAME and OSD_FORCE_FONTSIZE for asian fonts''' ),
+     Added OSD_FORCE_FONTNAME and OSD_FORCE_FONTSIZE for asian fonts""" ),
     (5.17,
-     '''Using the name of the helper in local_conf.py
+     """Using the name of the helper in local_conf.py
      Changed the TV_RECORD_SERVER_* to RECORDSERVER_*,
      Added optional RECORDSERVER_DEBUG, if not defined uses DEBUG
      Changed WWW_PORT to WEBSERVER_PORT = 80
@@ -254,17 +259,17 @@ LOCAL_CONF_CHANGES = [
      Added plug-in: XM online
      Added helpers: makevdev
      Added servers: encodingserver, rssserver
-     Added USE_SDL_KEYBOARD to specify if generic keyboard handler should be used
+     Added SYS_USE_KEYBOARD to specify if generic keyboard handler should be used
      Added EVENT_DEVS and EVENTMAP for the new Linux event device handler
      Added VIDEO_PRE_PLAY and VIDEO_POST_PLAY to allow external commands to be run
      Added CD_RIP_ for the cd backup plug-in
-     ''' ),
+     """ ),
     (5.18,
-     '''Added tv.recodings_manager plug-in to show what has been watched, TVRM_*,
+     """Added tv.recodings_manager plug-in to show what has been watched, TVRM_*,
      Removed TV_RECORD_PADDING, use TV_RECORD_PADDING_PRE and TV_RECORD_PADDING_POST
-     ''' ),
+     """ ),
     (5.19,
-     '''Changed rss.feeds field separator to use a ';' instead of a ','
+     """Changed rss.feeds field separator to use a ';' instead of a ','
      Changed weather locations to add a language code as the third parameter
      Moved video.reencode to video.reencode-old and video.reencode2 to video.reencode
      Added MAJOR_AUDIO_CTRL_MUTE to be able to choose a differente control for mute in the Alsa mixer plugin
@@ -273,9 +278,9 @@ LOCAL_CONF_CHANGES = [
      Added RSS_DOWNLOAD for a place to save downloaded data
      Added IMAGE_EXCLUDE as a regular expression to exclude images such as thumbnails
      Added TV_RECORD_FAVORITE_MARGIN to allow favourites to be added to the schedule within a tolerance value
-     ''' ),
+     """ ),
     (5.20,
-     '''Added PERSONAL_WWW_PAGE config item to allow private web pages in the webserver
+     """Added PERSONAL_WWW_PAGE config item to allow private web pages in the webserver
      Added LOGGING, can be one of CRITICAL, ERROR, WARNING, INFO, DEBUG or NOTSET
      Added RECORDSERVER_LOGGING to allow different levels of errors to be reported
      Changed VIDEO_INTERLACING to VIDEO_DEINTERLACE to be more consistent with autovars
@@ -286,9 +291,9 @@ LOCAL_CONF_CHANGES = [
      Added SHOPPINGCART_CLOBBER to allow a move to clobber an existing file
      Added XINE_BOOKMARK to enable the resume function to work with xine
      Added CACHE_CROPDETECT to enable caching of crop detection using encodingcode
-     ''' ),
+     """ ),
     (5.21,
-     '''Added OS_STATICDIR, FREEVO_STATICDIR, OS_LOGDIR and FREEVO_LOGDIR
+     """Added OS_STATICDIR, FREEVO_STATICDIR, OS_LOGDIR and FREEVO_LOGDIR
      Change static data to use /var/lib/freevo or ~/.freevo, including TV_RECORD_SCHEDULE, TV_LOGOS,
      XMLTV_FILE, you may also prefer OVERLAY_DIR to be FREEVO_STATICDIR+'/overlay',
      Added a plugin that adds a submenu entry for ejecting rom drives and binds the default action of
@@ -339,15 +344,15 @@ LOCAL_CONF_CHANGES = [
         XINE_TV_PROGRESSIVE_SEEK_INCREMENT
      Added TV_RECORD_YEAR_FORMAT to allow the from of the year in TV fxd files to be specified
      Moved plug-in "upsoon" to "tv.upsoon"
-     '''),
+     """),
     (5.22,
-     '''Added RECORDSERVER_SECRET and RECORDSERVER_PORT2=18002 for kaa.rpc
+     """Added RECORDSERVER_SECRET and RECORDSERVER_PORT2=18002 for kaa.rpc
      Renamed audio plug-in audio.playlist to audio.playlists
      Added ENCODINGSERVER_SAVE_DIR for re-encoded DVDs
      Added TV_CHANNELS_COMPARE as a lambda to sort the channels
-     '''),
+     """),
     (5.23,
-     ''' Added XMLTV_TIMEZONE to allow the time zone to be specified
+     """ Added XMLTV_TIMEZONE to allow the time zone to be specified
      Added OSD_X11_CURSORS to allow custom cursor to be set, stops xine showing a cursor
      Changed TV_RECORD_SCHEDULE to be a pickle file, this will delete existing favorites
      Added TV_RECORD_FAVORITES and TV_RECORD_FAVORITES_LIST to keep favorites separate
@@ -356,7 +361,34 @@ LOCAL_CONF_CHANGES = [
      Changed RESTART_SYS_CMD to SYS_RESTART_CMD for consistency
      Changed SHUTDOWN_SYS_ENABLE to SYS_SHUTDOWN_ENABLE for consistency
      Removed RECORDSERVER_PORT2 as it is no longer needed, using RECORDSERVER_PORT instead
-     '''),
+     """),
+    (5.24,
+     """ Added POLL_TIME to allow custom poll rates to be set, default 0.01 seconds
+     """),
+    (5.25,
+     """ Added OSD_IDLEBAR_PADDING to allow the space between idlebar items to be set
+     Added OSD_IDLEBAR_FONT and OSD_IDLEBAR_CLOCK_FONT to allow idlebar fonts to be set
+     Added MPLAYER_AO_DEV_OPTS for audio device options
+     Changed MPLAYER_VO_DEV_OPTS, removed need for leading ':'
+     Added ROM_DRIVES_AUTOFS to allow an autmounter to be used for ROM drives
+     Moved freeboxtv to tv plug-ins
+     Added MPLAYER_AUDIO_CACHE_KB, MPLAYER_AUDIO_CACHE_MIN_PERCENT and MPLAYER_AUDIO_NETWORK_OPTS to allow changing the default cache amount
+     Added SPEAK_WELCOME and SPEAK_SHUTDOWN for customized welcome and shutdown messages in speak plugin
+     Added FREEVO_USE_ALPHABLENDING to enable alpha blending transitions between screen changes. False by default
+     Renamed audio.mplayervis to audio.mplayervis1, so that audio.mplayervis2 will get noticed
+     Added SYS_USE_MOUSE option for enabling mouse support if needed. False by default
+     """),
+    (5.26,
+     """ Added VIDEO_AUTOJOIN_REGEX to allow joining video files based on a regular expression
+     Renamed USE_NETWORK to SYS_USE_NETWORK
+     Renamed USE_SDL_KEYBOARD to SYS_USE_KEYBOARD
+     Added SYS_USE_JOYSTICK to allow a joystick device to be used
+     Added DEBUG_BENCHMARKING can be used to time and trace function calls
+     Added DEBUG_BENCHMARKCALL can be used to print the arguments and results of function calls
+     Removed MPLAYER_AUDIO_CACHE_KB, MPLAYER_AUDIO_CACHE_MIN_PERCENT and MPLAYER_AUDIO_NETWORK_OPTS, it broke detach
+     Added WWW_IMAGE_SIZE and WWW_IMAGE_THUMBNAIL_SIZE for Cooliris support
+     Added VIDEO_AUTOJOIN_REGEX to allow more control when joining video files
+     """),
 ]
 
 
@@ -365,16 +397,14 @@ LOCAL_CONF_CHANGES = [
 
 FREEVO_CONF_VERSION = setup_freevo.CONFIG_VERSION
 
-if int(str(CONF.version).split('.')[0]) != \
-   int(str(FREEVO_CONF_VERSION).split('.')[0]):
-    print "\nERROR: The version information in freevo_config.py does't"
+if int(str(CONF.version).split('.')[0]) != int(str(FREEVO_CONF_VERSION).split('.')[0]):
+    print "ERROR: The version information in freevo_config.py does't"
     print 'match the version in %s.' % freevoconf
     print 'please rerun "freevo setup" to generate a new freevo.conf'
     print_config_changes(FREEVO_CONF_VERSION, CONF.version, FREEVO_CONF_CHANGES)
     sys.exit(1)
 
-if int(str(CONF.version).split('.')[1]) != \
-   int(str(FREEVO_CONF_VERSION).split('.')[1]):
+if int(str(CONF.version).split('.')[1]) != int(str(FREEVO_CONF_VERSION).split('.')[1]):
     print 'WARNING: freevo_config.py was changed, please rerun "freevo setup"'
     print_config_changes(FREEVO_CONF_VERSION, CONF.version, FREEVO_CONF_CHANGES)
 
@@ -383,6 +413,10 @@ if int(str(CONF.version).split('.')[1]) != \
 # General freevo settings:
 # ======================================================================
 
+POLL_TIME             = 0.01         # time in seconds that the poll handlers are called
+                                     # a lower rate of 0.05 is less demanding for less
+                                     # powerful processors
+
 AUDIO_DEVICE          = '/dev/dsp'   # e.g.: /dev/dsp0, /dev/audio, /dev/alsa/?
 AUDIO_INPUT_DEVICE    = '/dev/dsp1'  # e.g.: /dev/dsp0, /dev/audio, /dev/alsa/?
 
@@ -414,6 +448,9 @@ START_FULLSCREEN_X = 0               # Start in fullscreen mode if using x11 or
 #
 ROM_DRIVES = None
 
+ROM_DRIVES_AUTOFS = False  # Indicates that an automounter daemon is being used.
+                           # Does not try to mount/umount the media.
+
 #
 # hide discs from the wrong menu (e.g. VCDs in audio menu) and empty discs
 #
@@ -440,7 +477,6 @@ SYS_RESTART_CMD  = 'shutdown -r now'  # like SYS_SHUTDOWN_CMD, only for reboot
 SYS_SHUTDOWN_ENABLE = 0  # Performs a whole system shutdown at SHUTDOWN!
                          # For standalone boxes.
 
-
 # ======================================================================
 # Main menu items
 # ======================================================================
@@ -477,7 +513,6 @@ plugin.activate('shutdown', level=90)
 
 # -- autoshutdown menu item configuration --
 
-# SYS_SHUTDOWN_CONFIRM
 # Set to True to popup dialog boxes for confirmation.
 # this applies to menu items only.
 AUTOSHUTDOWN_CONFIRM=True
@@ -485,7 +520,6 @@ AUTOSHUTDOWN_CONFIRM=True
 
 # -- autoshutdown timer configuration --
 
-# TIMER_TIMEOUT
 # Set the timeout in minutes after which the system
 # is shutdown. The allowed idle time and the running
 # processes (see below) are evaluated to determine if
@@ -656,7 +690,19 @@ MENU_ARROW_NAVIGATION = False
 # Process keyboard events from SDL. You want this unless you use only lirc
 # or event devices below.
 #
-USE_SDL_KEYBOARD = True
+SYS_USE_KEYBOARD = True
+
+#
+# Process keyboard events from SDL. You want this unless you use only lirc
+# or event devices below.
+#
+SYS_USE_JOYSTICK = True
+
+#
+# Process mouse events from SDL/Pygame. You want this to control Freevo
+# with a mouse
+#
+SYS_USE_MOUSE = False
 
 #
 # Keymap to map keyboard keys to event strings. You can also add new keys
@@ -667,7 +713,7 @@ KEYMAP = DEFAULT_KEYMAP
 # List of /dev/input/event# devices to monitor. You can specify either the
 # device node (e.g. '/dev/input/event1') or the name of the device (e.g.
 # 'ATI Remote Wonder II'). If you monitor your keyboard both here and with
-# USE_SDL_KEYBOARD, then you will get duplicate events.
+# SYS_USE_KEYBOARD, then you will get duplicate events.
 #
 EVENT_DEVS = []
 
@@ -684,7 +730,7 @@ EVENTMAP = DEFAULT_EVENTMAP
 # the IMDB movie database for movie info, and Amazon for cover searches.
 # Set this to 0 if your computer isn't connected to a network.
 #
-USE_NETWORK = 1
+SYS_USE_NETWORK = True
 
 # HOST_ALIVE_CHECK tests if the given host is online.
 # Will be used to avoid extremely long automounter timeouts.
@@ -881,6 +927,11 @@ plugin.activate('tv.manual_record')
 # Enable this for joystick support:
 # plugin.activate('joy')
 
+# Speak plugin to output menu items via festival
+# plugin.activate('speak')
+SPEAK_WELCOME = ''
+SPEAK_SHUTDOWN = ''
+
 # ----------------------------------------------------------------------
 # CD Ripping
 # ----------------------------------------------------------------------
@@ -1035,23 +1086,25 @@ VIDEO_SHOW_DATA_DIR = None
 # The list of filename suffixes that are used to match the files that
 # are played wih MPlayer.
 #
-VIDEO_MPLAYER_SUFFIX = [ 'avi', 'mpg', 'mpeg', 'wmv', 'bin', 'rm',
-                         'divx', 'ogm', 'vob', 'asf', 'm2v', 'm2p',
-                         'mp4', 'viv', 'nuv', 'mov', 'iso',
-                         'nsv', 'mkv', 'ts', 'flv' ]
+VIDEO_MPLAYER_SUFFIX = [
+    'avi', 'mpg', 'mpeg', 'wmv', 'bin', 'rm', 'divx', 'ogm', 'vob', 'asf',
+    'm2v', 'm2p', 'mp4', 'viv', 'nuv', 'mov', 'iso', 'nsv', 'mkv', 'ogg',
+    'ts', 'flv',
+]
 
 #
 # The list of filename suffixes that are used to match the files that
 # are played wih Xine.
 #
-VIDEO_XINE_SUFFIX = [ 'avi', 'mpg', 'mpeg', 'rm', 'divx', 'ogm',
-                      'asf', 'm2v', 'm2p', 'mp4', 'mov', 'cue',
-                      'ts', 'iso', 'vob' ]
+VIDEO_XINE_SUFFIX = [
+    'avi', 'mpg', 'mpeg', 'rm', 'divx', 'ogm', 'asf', 'm2v', 'm2p', 'mp4',
+    'mov', 'cue', 'ts', 'iso', 'vob',
+]
 
 #
-# Preferred video player
+# Preferred video player (ranking, first one is most wanted!)
 #
-VIDEO_PREFERED_PLAYER = 'mplayer'
+VIDEO_PREFERED_PLAYER = ['mplayer', 'xine', 'vlc']
 
 #
 # Only scan OVERLAY_DIR and VIDEO_SHOW_DATA_DIR for fxd files containing
@@ -1067,6 +1120,11 @@ VIDEO_ONLY_SCAN_DATADIR = 1
 # item
 #
 VIDEO_AUTOJOIN = 1
+#
+# join files based on the regular expression
+# seaches for 1, 01, 001, etc before a '.'; possibly too simple
+#
+VIDEO_AUTOJOIN_REGEX='(0*1)\.'
 
 #
 # try to find out if deinterlacing is needed or not
@@ -1103,7 +1161,9 @@ AUDIO_ITEMS = None
 # The list of filename suffixes that are used to match the files that
 # are played as audio.
 #
-AUDIO_SUFFIX = [ 'mp3', 'ogg', 'wav','m4a', 'wma', 'aac', 'flac', 'mka','ac3' ]
+AUDIO_SUFFIX = [
+    'mp3', 'ogg', 'wav', 'm4a', 'wma', 'aac', 'flac', 'mka', 'ac3',
+]
 
 #
 # Regular expression used to recognize filenames which are likely to be
@@ -1146,7 +1206,7 @@ IMAGE_ITEMS = None
 # The list of filename suffixes that are used to match the files that
 # are used for the image viewer.
 #
-IMAGE_SUFFIX = [ 'jpg','gif','png', 'jpeg','bmp','tiff','psd' ]
+IMAGE_SUFFIX = [ 'jpg', 'gif', 'png', 'jpeg', 'bmp', 'tiff', 'psd' ]
 
 # The viewer now supports a new type of menu entry, a slideshow file.
 # It also has the slideshow alarm signal handler for automated shows.
@@ -1155,7 +1215,7 @@ IMAGE_SUFFIX = [ 'jpg','gif','png', 'jpeg','bmp','tiff','psd' ]
 IMAGE_SSHOW_SUFFIX = [ 'ssr' ]
 
 # The viewer can exclude certain types of images based on the regular expression list
-# eg IMAGE_EXCLUDE = [('thm','tn_')]
+# eg IMAGE_EXCLUDE = [('thm', 'tn_')]
 
 IMAGE_EXCLUDE = None
 
@@ -1186,25 +1246,32 @@ IMAGEVIEWER_REVERSED_IMAGES = 0
 #
 IMAGEVIEWER_OSD = [
     # First OSD info
-    [ (_('Title')+': ',      'name'),
-      (_('Description')+': ','description'),
-      (_('Author')+': ',     'author') ],
-
+    [ (_('Title')+': ',       'name'),
+      (_('Description')+': ', 'description'),
+      (_('Author')+': ',      'author')
+    ],
     # Second OSD info
     [ (_('Title')+': ',    'name'),
-      (_('Date')+': ' ,    'date'),
+      (_('Date')+': ',     'date'),
       ('W:',               'width'),
       ('H:',               'height'),
       (_('Model')+': ',    'hardware'),
-      (_('Software')+': ', 'software') ]
-    ]
+      (_('Software')+': ', 'software')
+    ],
+]
 
 #
 # Default duration for images in a playlist. If set to 0, you need
 # to press a button to get to the next image, a value > 0 is the time
 # in seconds for an auto slideshow
 #
-IMAGEVIEWER_DURATION = 0
+IMAGEVIEWER_DURATION = 3
+
+#
+# If set to True, the slideshow starts automaticaly entering the image viewer,
+# if set to False, it must be started manually with the play button:
+#
+IMAGEVIEWER_AUTOPLAY = True
 
 #
 # When viewing images on a TV screen where the pixels are not square
@@ -1423,6 +1490,14 @@ OSD_SOUNDS= {
     'menu.select'  : None
 }
 
+#
+# Padding between icons
+#
+OSD_IDLEBAR_PADDING = 20
+OSD_IDLEBAR_FONT = 'small0'
+OSD_IDLEBAR_CLOCK_FONT = 'clock'
+
+
 # ======================================================================
 # Freevo remote control settings:
 # ======================================================================
@@ -1485,13 +1560,18 @@ TVTIME_CMD = CONF.tvtime
 MPLAYER_CMD = CONF.mplayer
 
 MPLAYER_AO_DEV = 'oss:/dev/dsp'    # e.g.: oss,sdl,alsa, see mplayer docs
+MPLAYER_AO_DEV_OPTS = ''           # e.g.: 'some_var=vcal'
 
 if CONF.display == 'x11':
     MPLAYER_VO_DEV = 'xv,sdl,x11,' # X11 drivers in order of preference
 else:
     MPLAYER_VO_DEV = CONF.display  # e.g.: x11,mga,fbdev, see mplayer docs
 
-MPLAYER_VO_DEV_OPTS = ''           # e.g.: ':some_var=vcal'
+MPLAYER_VO_DEV_OPTS = ''           # e.g.: 'some_var=vcal'
+
+MPLAYER_AUDIO_CACHE_KB = 256
+MPLAYER_AUDIO_CACHE_MIN_PERCENT = 25
+MPLAYER_AUDIO_NETWORK_OPTS = '-cache %d -cache-min %d' % (MPLAYER_AUDIO_CACHE_KB, MPLAYER_AUDIO_CACHE_MIN_PERCENT)
 
 DVD_LANG_PREF = 'en,se,no'         # Order of preferred languages on DVD.
 DVD_SUBTITLE_PREF = ''             # Order of preferred subtitles on DVD.
@@ -1524,18 +1604,21 @@ MPLAYER_SOFTWARE_SCALER = ''
 # Mplayer arguments for different media formats. (eg DVDs, CDs, AVI files, etc)
 # Uses a default value if nothing else matches.
 #
-MPLAYER_ARGS = { 'dvd'    : '-cache 8192',
-                 'vcd'    : '-cache 4096',
-                 'cd'     : '-cache 1024 -cdda speed=2',
-                 'tv'     : '-nocache',
-                 'ivtv'   : '-cache 8192',
-                 'dvb'    : '-cache 1024',
-                 'avi'    : '-cache 5000 -idx',
-                 'rm'     : '-cache 5000 -forceidx',
-                 'rmvb'   : '-cache 5000 -forceidx',
-                 'webcam' : 'tv:// -tv driver=v4l:width=352:height=288:outfmt=yuy2:device=/dev/video2',
-                 'default': '-cache 5000'
-                 }
+MPLAYER_ARGS = {
+    'dvd'    : '-cache 8192',
+    'vcd'    : '-cache 4096',
+    'cd'     : '-cache 1024 -cdda speed=2',
+    'tv'     : '-nocache',
+    'ivtv'   : '-cache 8192',
+    'dvb'    : '-cache 1024',
+    'avi'    : '-cache 5000 -idx',
+    'flv'    : '-nocache -forceidx',
+    'mp4'    : '-nocache -forceidx',
+    'rm'     : '-cache 5000 -forceidx',
+    'rmvb'   : '-cache 5000 -forceidx',
+    'webcam' : 'tv:// -tv driver=v4l:width=352:height=288:outfmt=yuy2:device=/dev/video2',
+    'default': '-cache 5000'
+}
 
 #
 # Number of seconds before seek value times out. This is used when
@@ -1669,6 +1752,23 @@ TV_RECORD_REMOVE_COMMERCIALS = None
 # This will try to resolve scheduling conflicts and re-schedule when needed
 TV_RECORD_CONFLICT_RESOLUTION = None
 
+# This will automatically re-encode recordings with the default REENCODE settings
+# using the encoding server
+TV_REENCODE = False
+TV_REENCODE_REMOVE_SOURCE = False
+
+# Some default re-encode values
+REENCODE_CONTAINER = 'avi'
+REENCODE_RESOLUTION = 'Optimal'
+REENCODE_VIDEOCODEC = 'XviD'
+REENCODE_VIDEOBITRATE = 1000
+REENCODE_AUDIOCODEC = 'MPEG 1 Layer 3 (mp3)'
+REENCODE_AUDIOBITRATE = 128
+REENCODE_NUMPASSES = 1
+REENCODE_VIDEOFILTER = None
+REENCODE_NUMTHREADS = 1
+REENCODE_ALTPROFILE = None
+
 #
 # Watching TV
 #
@@ -1920,13 +2020,13 @@ TV_VIDEO_GROUPS = [
 #                ('26', 'TV3',               'E10'),
 #                ('27', 'TV4',               'E6'),
 #                ('10', 'Kanal 5',           'E7'),
-#                ('60', 'Fox Kids',          'E8', ('1234567','0600','1659')),
-#                ('16', 'TV6',               'E8', ('1234567','1700','2359'),
-#                                                  ('1234567','0000','0300')),
+#                ('60', 'Fox Kids',          'E8', ('1234567', '0600', '1659')),
+#                ('16', 'TV6',               'E8', ('1234567', '1700', '2359'),
+#                                                  ('1234567', '0000', '0300')),
 #                ('14', 'MTV Europe',        'E11') ]
 #
 # As you can see the list takes optional tuples:
-# ( 'DAYS', 'START','END')
+# ( 'DAYS', 'START', 'END')
 #
 # 1234567 in days means all days.
 # 12345 would mean monday to friday.
@@ -1992,10 +2092,25 @@ WEBSERVER_GID = 0
 #
 WEBSERVER_CACHEDIR = FREEVO_CACHEDIR
 
+# items to include on the web pages
+WWW_PAGES = [
+    #   Label                      Title                                  Page
+    (_('Home'),                 _('Home'),                               'index.rpy'),
+    (_('TV Guide'),             _('View TV Listings'),                   'guide.rpy'),
+    (_('Scheduled Recordings'), _('View Scheduled Recordings'),          'record.rpy'),
+    (_('Favorites'),            _('View Favorites'),                     'favorites.rpy'),
+    (_('Media Library'),        _('View Media Library'),                 'library.rpy'),
+    (_('Manual Recording'),     _('Schedule a Manual Recording'),        'manualrecord.rpy'),
+    (_('Search'),               _('Advanced Search Page'),               'search.rpy'),
+    (_('Help'),                 _('View Online Help and Documentation'), 'help/')
+]
+
 #
 # Some sizes for the images in the web library
+# Can be a tuple of sizes or a size
 #
-WWW_IMAGE_THUMBNAIL_SIZE = (200, 200)
+WWW_IMAGE_SIZE = (1024, 768)
+WWW_IMAGE_THUMBNAIL_SIZE = 256
 
 #
 # Username / Password combinations to login to the web interface.
@@ -2034,8 +2149,10 @@ WWW_JAVASCRIPT = 'scripts/display_prog-head.js'
 ENCODINGSERVER_UID = 0
 ENCODINGSERVER_GID = 0
 
+
 ENCODINGSERVER_IP   = 'localhost'
-ENCODINGSERVER_PORT = 6666
+ENCODINGSERVER_PORT = 18002
+ENCODINGSERVER_SECRET = 'secret2'
 
 # ======================================================================
 # Freevo builtin commdetect server settings:
@@ -2125,14 +2242,26 @@ DIRECTORY_USE_STAT_FOR_CHANGES = True
 #
 DEBUG_SKIN = 0
 
+#
+# Debug the idlebar, display boxes around each area.
+#
+DEBUG_IDLEBAR = 0
+DEBUG_IDLEBAR_FONT = 'tiny0'
+
 #
 # store output of started processes for debug
 # Set to 1 to log child application output to <app>_stdout.log and <app>_stderr.log
 #
 DEBUG_CHILDAPP = 0
 
+# Enable the timing of the event handler
 DEBUG_TIME = 0
 
+# Enable the benchmarking wrapper, when active prints the duration of a function call
+# We could be more specific and use a bit pattern to enable certain parts, like skins, tv, etc.
+DEBUG_BENCHMARKING = 0
+DEBUG_BENCHMARKCALL = False
+
 # The default logging level
 # can be one of CRITICAL (FATAL), ERROR, WARNING (WARN), INFO, DEBUG, NOTSET
 LOGGING = logging.INFO
@@ -2143,3 +2272,11 @@ LOGGING_RSSSERVER = logging.INFO
 
 # When logging is DEBUG or NOTSET then DEBUG level logs messages
 DEBUG = 0
+
+# enable the pdb (python debugger), don't set this unless you know how to use the debugger
+DEBUG_DEBUGGER = 0
+
+#
+# Activate blending between different menu screens
+#
+FREEVO_USE_ALPHABLENDING = False
